@@ -26,6 +26,8 @@
 #define ALSA_PCM_NEW_SW_PARAMS_API
 
 #include <alsa/asoundlib.h>
+#include <alsa/control.h>
+#include <alsa/error.h>
 #include <gst/gst.h>
 
 
@@ -37,7 +39,11 @@ GST_DEBUG_CATEGORY_EXTERN (alsa_debug);
 /* NOTE: these functions require a GObject *this and can only be used in 
    functions that return TRUE on success and FALSE on error */
 #define SIMPLE_ERROR_CHECK(value) G_STMT_START{ \
-  int err = (value); if (err < 0) { return FALSE; } \
+  int err = (value); \
+  if (err < 0) { \
+    GST_WARNING_OBJECT (this, "\"" #value "\": %s", snd_strerror (err)); \
+    return FALSE; \
+  } \
 }G_STMT_END
 
 
@@ -72,9 +78,23 @@ GST_DEBUG_CATEGORY_EXTERN (alsa_debug);
 
 #define GST_ALSA_MIN_RATE	8000
 #define GST_ALSA_MAX_RATE	192000
+#define GST_ALSA_MIN_PERIOD_CNT	0
+#define GST_ALSA_MAX_PERIOD_CNT	64
+#define GST_ALSA_MIN_PERIOD_SZ	2
+#define GST_ALSA_MAX_PERIOD_SZ	8192
+#define GST_ALSA_MIN_BUFFER_SZ	GST_ALSA_MIN_PERIOD_CNT*GST_ALSA_MIN_PERIOD_SZ
+#define GST_ALSA_MAX_BUFFER_SZ	65536
 #define GST_ALSA_MAX_TRACKS	64 /* we don't support more than 64 tracks */
 #define GST_ALSA_MAX_CHANNELS	32 /* tracks can have up to 32 channels */
 
+/* a few alsa functions return an int value and a 'direction', -1, 0 or +1
+  0 = exact value
+ -1 = real value is up to 1 before given value
+ +1 = real value is up to 1 after given value
+*/
+#define GST_ALSA_DIR_MIN(i) ((i == 1) ? 1 : 0)
+#define GST_ALSA_DIR_MAX(i) ((i ==-1) ?-1 : 0)
+
 /* Mono is 1 channel ; the 5.1 standard is 6 channels. The value for
    GST_ALSA_MAX_CHANNELS comes from alsa/mixer.h. */
 
@@ -96,7 +116,7 @@ enum {
   GST_ALSA_OPEN = GST_ELEMENT_FLAG_LAST,
   GST_ALSA_RUNNING,
   GST_ALSA_CAPS_NEGO,
-  GST_ALSA_FLAG_LAST = GST_ELEMENT_FLAG_LAST + 3,
+  GST_ALSA_FLAG_LAST = GST_ELEMENT_FLAG_LAST + 3
 };
 
 typedef enum {
@@ -135,27 +155,33 @@ struct _GstAlsa {
   gchar *			device;
   gchar *                       cardname;
   snd_pcm_t *			handle;
-  snd_pcm_info_t *		info;
   guint				pcm_caps;	/* capabilities of the pcm device, see GstAlsaPcmCaps */
   snd_output_t *		out;
 
   GstAlsaFormat *		format;		/* NULL if undefined */
   gboolean			mmap; 		/* use mmap transmit (fast) or read/write (sloooow) */
   GstAlsaTransmitFunction	transmit;
+  GstCaps *			cached_caps;	/* we cache caps to speed up get_caps */
 
   /* latency / performance parameters */
+  snd_pcm_uframes_t		desired_period_size;	/* as set by user */
   snd_pcm_uframes_t		period_size;
+  unsigned int			desired_period_count;	/* as set by user */
   unsigned int			period_count;
 
   gboolean			autorecover;
 
+  gboolean                      advanced_probing;
+
   /* clocking */
   GstAlsaClock *		clock;		/* our provided clock */
-  snd_pcm_uframes_t		transmitted; 	/* samples transmitted since last sync 
+  GstClock *			ext_clock;	/* externally set clock. */
+  snd_pcm_uframes_t		played; 	/* samples transmitted since last sync 
 						   This thing actually is our master clock.
 						   We will event insert silent samples or
 						   drop some to sync to incoming timestamps.
 						 */
+  snd_pcm_uframes_t		captured;
   GstClockTime			max_discont;	/* max difference between current
   						   playback timestamp and buffers timestamps
 						 */
@@ -185,6 +211,7 @@ GstCaps *		gst_alsa_fixate 	(GstPad *		pad,
 GstCaps *		gst_alsa_caps		(snd_pcm_format_t	format,
 						 gint			rate,
 						 gint			channels);
+gboolean                gst_alsa_set_default_format (GstAlsa * this);
 
 /* audio processing functions */
 inline snd_pcm_sframes_t	gst_alsa_update_avail	(GstAlsa * this);
@@ -206,6 +233,13 @@ inline GstClockTime		gst_alsa_bytes_to_timestamp 	(GstAlsa *		this,
 inline guint			gst_alsa_timestamp_to_bytes 	(GstAlsa *		this,
 								 GstClockTime	 	time);
 
+/* debugging functions (useful in gdb) - require running with --gst-debug=alsa:4 or better */
+void 				gst_alsa_sw_params_dump		(GstAlsa *		this, 
+								 snd_pcm_sw_params_t *	sw_params);
+void				gst_alsa_hw_params_dump		(GstAlsa *		this, 
+								 snd_pcm_hw_params_t *	hw_params);
+
+
 G_END_DECLS
 
 #endif /* __GST_ALSA_H__ */
