@@ -89,6 +89,7 @@ gst_alsa_clock_init (GstAlsaClock * clock)
   gst_object_set_name (GST_OBJECT (clock), "GstAlsaClock");
 
   clock->start_time = GST_CLOCK_TIME_NONE;
+  clock->fallback_time = GST_CLOCK_TIME_NONE;
 }
 
 GstAlsaClock *
@@ -135,6 +136,7 @@ gst_alsa_clock_stop (GstAlsaClock * clock)
       GST_TIMEVAL_TO_TIME (timeval) -
       gst_clock_get_event_time (GST_CLOCK (clock));
   clock->start_time = GST_CLOCK_TIME_NONE;
+  clock->fallback_time = GST_CLOCK_TIME_NONE;
 }
 
 static GstClockTime
@@ -172,8 +174,11 @@ gst_alsa_clock_wait (GstClock * clock, GstClockEntry * entry)
   GTimeVal t;
 
   entry_time = gst_alsa_clock_get_internal_time (clock);
-  diff = GST_CLOCK_ENTRY_TIME (entry) - gst_clock_get_time (clock);
-
+  if (GST_CLOCK_TIME_IS_VALID (this->fallback_time)) {
+    diff = GST_CLOCK_ENTRY_TIME (entry) - this->fallback_time;
+  } else {
+    diff = GST_CLOCK_ENTRY_TIME (entry) - gst_clock_get_time (clock);
+  }
   if (diff < 0)
     return GST_CLOCK_ENTRY_EARLY;
 
@@ -202,6 +207,11 @@ gst_alsa_clock_wait (GstClock * clock, GstClockEntry * entry)
     g_get_current_time (&t);
     glib_cur = GST_TIMEVAL_TO_TIME (t);
   }
+  if (glib_start + diff * 1.5 <= glib_cur) {
+    this->fallback_time = GST_CLOCK_ENTRY_TIME (entry);
+  } else {
+    this->fallback_time = GST_CLOCK_TIME_NONE;
+  }
 
   return entry->status;
 }
