@@ -21,6 +21,7 @@
 #include "config.h"
 #endif
 
+#include "_stdint.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -32,6 +33,9 @@
 #include <gst/gst-i18n-plugin.h>
 #include <gst/gst.h>
 
+#include <dvdread/dvd_reader.h>
+#include <dvdread/ifo_read.h>
+
 #include <dvdnav/dvdnav.h>
 #include <dvdnav/nav_print.h>
 
@@ -98,7 +102,7 @@ typedef enum
 DVDNavSrcPauseMode;
 
 /* Interval of time to sleep during pauses. */
-#define DVDNAVSRC_PAUSE_INTERVAL (GST_SECOND / 20)
+#define DVDNAVSRC_PAUSE_INTERVAL (GST_SECOND / 30)
 
 /* The DVD domain types. */
 typedef enum
@@ -121,18 +125,20 @@ struct _DVDNavSrc
 
   /* Location */
   gchar *location;
+  gchar *last_uri;
 
+  gint64 pending_offset;        /* Offset to use for next discont event */
   gboolean did_seek;
+  gboolean new_seek;
+  gboolean seek_pending;
   gboolean need_flush;
-  gboolean need_newmedia;
 
   /* Timing */
   GstClock *clock;              /* The clock for this element. */
 
   /* Pause handling */
   DVDNavSrcPauseMode pause_mode;        /* The current pause mode. */
-  GstClockTime pause_end;       /* The clock time for the end of the
-                                   pause. */
+  GstClockTime pause_remain;    /* Remaining duration of the pause */
 
   /* Highligh handling */
   int button;                   /* The currently highlighted button
@@ -154,6 +160,17 @@ struct _DVDNavSrc
 
   GstBuffer *cur_buf;           /* Current output buffer. See
                                    dvdnavsrc_get. */
+
+  GstClockTime pgc_length;      /* Length of the current 
+                                   program chain (title) */
+  GstClockTime cell_start;      /* Start of the current cell */
+  GstClockTime pg_start;        /* Start of the current program
+                                 * within the PGC */
+
+  gint cur_vts;                 /* Current VTS being read */
+  vmgi_mat_t vmgm_attr;         /* VMGM domain info */
+  GArray *vts_attrs;            /* Array of vts_attributes_t structures 
+                                   cached from the VMG ifo */
 };
 
 struct _DVDNavSrcClass
@@ -248,7 +265,7 @@ static void dvdnavsrc_print_event (DVDNavSrc * src,
 #endif /* GST_DISABLE_GST_DEBUG */
 static void dvdnavsrc_update_streaminfo (DVDNavSrc * src);
 static void dvdnavsrc_set_domain (DVDNavSrc * src);
-static void dvdnavsrc_update_highlight (DVDNavSrc * src);
+static void dvdnavsrc_update_highlight (DVDNavSrc * src, gboolean force);
 static void dvdnavsrc_user_op (DVDNavSrc * src, int op);
 static GstElementStateReturn dvdnavsrc_change_state (GstElement * element);
 
@@ -392,10 +409,13 @@ dvdnavsrc_init (DVDNavSrc * src)
   gst_element_add_pad (GST_ELEMENT (src), src->srcpad);
 
   src->location = g_strdup ("/dev/dvd");
+  src->last_uri = NULL;
 
+  src->pending_offset = -1;
   src->did_seek = FALSE;
+  src->new_seek = FALSE;
+  src->seek_pending = FALSE;
   src->need_flush = FALSE;
-  src->need_newmedia = TRUE;
 
   /* Pause mode is initially inactive. */
   src->pause_mode = DVDNAVSRC_PAUSE_OFF;
@@ -419,6 +439,13 @@ dvdnavsrc_init (DVDNavSrc * src)
 
   /* No current output buffer. */
   src->cur_buf = NULL;
+
+  src->pgc_length = GST_CLOCK_TIME_NONE;
+  src->cell_start = 0;
+  src->pg_start = 0;
+
+  src->vts_attrs = NULL;
+  src->cur_vts = 0;
 }
 
 static void
@@ -430,6 +457,11 @@ dvdnavsrc_finalize (GObject * object)
   if (src->cur_buf != NULL) {
     gst_buffer_unref (src->cur_buf);
   }
+
+  g_free (src->last_uri);
+
+  if (src->vts_attrs)
+    g_array_free (src->vts_attrs, TRUE);
 }
 
 static gboolean
@@ -832,7 +864,7 @@ dvdnavsrc_set_domain (DVDNavSrc * src)
  * necessary.
  */
 static void
-dvdnavsrc_update_highlight (DVDNavSrc * src)
+dvdnavsrc_update_highlight (DVDNavSrc * src, gboolean force)
 {
   int button = 0;
   pci_t *pci;
@@ -865,7 +897,7 @@ dvdnavsrc_update_highlight (DVDNavSrc * src)
   DVDNAV_CALL (dvdnav_get_highlight_area, (pci, button, 0, &area), src);
 
   /* Check if we have a new button number, or a new highlight region. */
-  if (button != src->button ||
+  if (button != src->button || force ||
       memcmp (&area, &(src->area), sizeof (dvdnav_highlight_area_t)) != 0) {
     memcpy (&(src->area), &area, sizeof (dvdnav_highlight_area_t));
 
@@ -1127,11 +1159,6 @@ dvdnavsrc_print_event (DVDNavSrc * src, guint8 * data, int event, int len)
     }
       break;
     case DVDNAV_CELL_CHANGE:
-    {
-      /*dvdnav_cell_change_event_t *event =
-         (dvdnav_cell_change_event_t *)data; */
-      /* FIXME: Print something relevant here. */
-    }
       break;
     case DVDNAV_NAV_PACKET:
     {
@@ -1200,25 +1227,32 @@ dvdnavsrc_make_dvd_event (DVDNavSrc * src, const gchar * event_name,
   return event;
 }
 
+static void
+dvdnavsrc_structure_set_uint64 (GstStructure * structure, const char *name,
+    guint64 val)
+{
+  GValue gvalue = { 0 };
+  g_value_init (&gvalue, G_TYPE_UINT64);
+  g_value_set_uint64 (&gvalue, val);
+  gst_structure_set_value (structure, name, &gvalue);
+}
+
 static GstEvent *
 dvdnavsrc_make_dvd_nav_packet_event (DVDNavSrc * src, const pci_t * pci)
 {
   GstEvent *event;
   GstStructure *structure;
-  GValue start_ptm = { 0 }, end_ptm = {
-  0};
 
-  /* Store the time values in GValues. */
-  g_value_init (&start_ptm, G_TYPE_UINT64);
-  g_value_set_uint64 (&start_ptm, MPEGTIME_TO_GSTTIME (pci->pci_gi.vobu_s_ptm));
-  g_value_init (&end_ptm, G_TYPE_UINT64);
-  g_value_set_uint64 (&end_ptm, MPEGTIME_TO_GSTTIME (pci->pci_gi.vobu_e_ptm));
-
-  /* Add the values to a new structure. */
+  /* Build the event structure. */
   structure = gst_structure_new ("application/x-gst-dvd",
       "event", G_TYPE_STRING, "dvd-nav-packet", NULL);
-  gst_structure_set_value (structure, "start_ptm", &start_ptm);
-  gst_structure_set_value (structure, "end_ptm", &end_ptm);
+
+  dvdnavsrc_structure_set_uint64 (structure, "start_ptm",
+      MPEGTIME_TO_GSTTIME (pci->pci_gi.vobu_s_ptm));
+  dvdnavsrc_structure_set_uint64 (structure, "end_ptm",
+      MPEGTIME_TO_GSTTIME (pci->pci_gi.vobu_e_ptm));
+  dvdnavsrc_structure_set_uint64 (structure, "cell_start", src->cell_start);
+  dvdnavsrc_structure_set_uint64 (structure, "pg_start", src->pg_start);
 
   /* Create the DVD event and put the structure into it. */
   event = gst_event_new (GST_EVENT_ANY);
@@ -1241,7 +1275,7 @@ dvdnavsrc_make_clut_change_event (DVDNavSrc * src, const guint * clut)
 {
   GstEvent *event;
   GstStructure *structure;
-  guchar name[16];
+  gchar name[16];
   int i;
 
   structure = gst_structure_new ("application/x-gst-dvd",
@@ -1269,6 +1303,150 @@ dvdnavsrc_make_clut_change_event (DVDNavSrc * src, const guint * clut)
   return event;
 }
 
+/* Use libdvdread to read and cache info from the IFO file about
+ * streams in each VTS
+ */
+static gboolean
+read_vts_info (DVDNavSrc * src)
+{
+  dvd_reader_t *dvdi;
+  ifo_handle_t *ifo;
+  gint i;
+  gint n_vts;
+
+  if (src->vts_attrs) {
+    g_array_free (src->vts_attrs, TRUE);
+    src->vts_attrs = NULL;
+  }
+
+  dvdi = DVDOpen (src->location);
+  if (!dvdi)
+    return FALSE;
+
+  ifo = ifoOpen (dvdi, 0);
+  if (!ifo) {
+    GST_ERROR ("Can't open VMG info");
+    return FALSE;
+  }
+  n_vts = ifo->vts_atrt->nr_of_vtss;
+  memcpy (&src->vmgm_attr, ifo->vmgi_mat, sizeof (vmgi_mat_t));
+  ifoClose (ifo);
+
+  GST_DEBUG ("Reading IFO info for %d VTSs", n_vts);
+  src->vts_attrs = g_array_sized_new (FALSE, TRUE, sizeof (vtsi_mat_t),
+      n_vts + 1);
+  if (!src->vts_attrs)
+    return FALSE;
+  g_array_set_size (src->vts_attrs, n_vts + 1);
+
+  for (i = 1; i <= n_vts; i++) {
+    ifo = ifoOpen (dvdi, i);
+    if (!ifo) {
+      GST_ERROR ("Can't open VTS %d", i);
+      return FALSE;
+    }
+
+    GST_DEBUG ("VTS %d, Menu has %d audio %d subpictures. "
+        "Title has %d and %d", i,
+        ifo->vtsi_mat->nr_of_vtsm_audio_streams,
+        ifo->vtsi_mat->nr_of_vtsm_subp_streams,
+        ifo->vtsi_mat->nr_of_vts_audio_streams,
+        ifo->vtsi_mat->nr_of_vts_subp_streams);
+
+    memcpy (&g_array_index (src->vts_attrs, vtsi_mat_t, i),
+        ifo->vtsi_mat, sizeof (vtsi_mat_t));
+
+    ifoClose (ifo);
+  }
+  DVDClose (dvdi);
+  return TRUE;
+}
+
+static GstEvent *
+dvdnav_build_titlelang_event (DVDNavSrc * src)
+{
+  vtsi_mat_t *vts_attr;
+  audio_attr_t *a_attrs;
+  subp_attr_t *s_attrs;
+  gint n_audio, n_subp;
+  GstEvent *e;
+  GstStructure *s;
+  gint i;
+  gchar lang_code[3] = { '\0', '\0', '\0' };
+  gchar *t;
+
+  if (src->vts_attrs == NULL || src->cur_vts >= src->vts_attrs->len) {
+    if (src->vts_attrs)
+      GST_ERROR ("No stream info for VTS %d (have %d)", src->cur_vts,
+          src->vts_attrs->len);
+    else
+      GST_ERROR ("No stream info");
+    return NULL;
+  }
+
+  if (src->domain == DVDNAVSRC_DOMAIN_VMGM) {
+    vts_attr = NULL;
+    a_attrs = &src->vmgm_attr.vmgm_audio_attr;
+    n_audio = MIN (1, src->vmgm_attr.nr_of_vmgm_audio_streams);
+    s_attrs = &src->vmgm_attr.vmgm_subp_attr;
+    n_subp = MIN (1, src->vmgm_attr.nr_of_vmgm_subp_streams);
+  } else {
+    vts_attr = &g_array_index (src->vts_attrs, vtsi_mat_t, src->cur_vts);
+    a_attrs = vts_attr->vts_audio_attr;
+    n_audio = vts_attr->nr_of_vts_audio_streams;
+    s_attrs = vts_attr->vts_subp_attr;
+    n_subp = vts_attr->nr_of_vts_subp_streams;
+  }
+
+  /* build event */
+  e = gst_event_new (GST_EVENT_ANY);
+  s = e->event_data.structure.structure =
+      gst_structure_new ("application/x-gst-event",
+      "event", G_TYPE_STRING, "dvd-lang-codes", NULL);
+
+  /* audio */
+  for (i = 0; i < n_audio; i++) {
+    const audio_attr_t *a = a_attrs + i;
+
+    t = g_strdup_printf ("audio-%d-format", i);
+    gst_structure_set (s, t, G_TYPE_INT, (int) a->audio_format, NULL);
+    g_free (t);
+
+    GST_DEBUG ("Audio stream %d is format %d", i, (int) a->audio_format);
+
+    if (a->lang_type) {
+      t = g_strdup_printf ("audio-%d-language", i);
+      lang_code[0] = (a->lang_code >> 8) & 0xff;
+      lang_code[1] = a->lang_code & 0xff;
+      gst_structure_set (s, t, G_TYPE_STRING, lang_code, NULL);
+      g_free (t);
+
+      GST_DEBUG ("Audio stream %d is language %s", i, lang_code);
+    } else
+      GST_DEBUG ("Audio stream %d - no language", i, lang_code);
+  }
+
+  /* subtitle */
+  for (i = 0; i < n_subp; i++) {
+    const subp_attr_t *u = s_attrs + i;
+
+    t = g_strdup_printf ("subtitle-%d-language", i);
+    if (u->type) {
+      lang_code[0] = (u->lang_code >> 8) & 0xff;
+      lang_code[1] = u->lang_code & 0xff;
+      gst_structure_set (s, t, G_TYPE_STRING, lang_code, NULL);
+    } else {
+      gst_structure_set (s, t, G_TYPE_STRING, "MENU", NULL);
+    }
+    g_free (t);
+
+    GST_DEBUG ("Subtitle stream %d is language %s", i,
+        lang_code[0] ? lang_code : "NONE");
+  }
+
+  return e;
+}
+
 static void
 dvdnavsrc_loop (GstElement * element)
 {
@@ -1279,30 +1457,49 @@ dvdnavsrc_loop (GstElement * element)
 
   g_return_if_fail (dvdnavsrc_is_open (src));
 
-  if (src->did_seek || src->need_newmedia) {
-    GstEvent *event;
-
-    src->did_seek = FALSE;
-    GST_INFO_OBJECT (src, "sending discont");
-
-    event = gst_event_new_discontinuous (src->need_newmedia, 0);
-
-    src->need_flush = FALSE;
-    src->need_newmedia = FALSE;
-    gst_pad_push (src->srcpad, GST_DATA (event));
-    return;
-  }
-
-  if (src->need_flush) {
-    src->need_flush = FALSE;
-    GST_INFO_OBJECT (src, "sending flush");
-    gst_pad_push (src->srcpad, GST_DATA (gst_event_new_flush ()));
+  if (src->new_seek) {
+    dvdnavsrc_tca_seek (src, src->title, src->chapter, src->angle);
+    src->new_seek = FALSE;
     return;
   }
 
   /* Loop processing blocks until there is data to send. */
   send_data = NULL;
   while (send_data == NULL) {
+    if (src->need_flush) {
+      if (src->pause_mode != DVDNAVSRC_PAUSE_OFF) {
+        DVDNAV_CALL (dvdnav_still_skip, (src->dvdnav), src);
+        src->pause_mode = DVDNAVSRC_PAUSE_OFF;
+      }
+
+      src->need_flush = FALSE;
+      GST_INFO_OBJECT (src, "sending flush");
+      gst_pad_push (src->srcpad, GST_DATA (gst_event_new_flush ()));
+      dvdnavsrc_update_highlight (src, TRUE);
+    }
+
+    if (src->pause_mode == DVDNAVSRC_PAUSE_OFF) {
+      if (src->did_seek) {
+        GstEvent *event;
+
+        src->did_seek = FALSE;
+        GST_INFO_OBJECT (src, "sending discont");
+
+        if (src->pending_offset != -1) {
+          event = gst_event_new_discontinuous (FALSE, GST_FORMAT_BYTES,
+              src->pending_offset, GST_FORMAT_UNDEFINED);
+          src->pending_offset = -1;
+        } else
+          event = gst_event_new_discontinuous (FALSE, GST_FORMAT_UNDEFINED);
+
+        gst_pad_push (src->srcpad, GST_DATA (event));
+
+        /* Sent a discont, make sure to enable highlight */
+        src->button = 0;
+        dvdnavsrc_update_highlight (src, TRUE);
+      }
+    }
+
     if (src->cur_buf == NULL) {
       src->cur_buf = gst_buffer_new_and_alloc (DVD_VIDEO_LB_LEN);
       if (src->cur_buf == NULL) {
@@ -1327,7 +1524,6 @@ dvdnavsrc_loop (GstElement * element)
       case DVDNAV_STILL_FRAME:
       {
         dvdnav_still_event_t *info = (dvdnav_still_event_t *) data;
-        GstClockTime current_time = gst_element_get_time (GST_ELEMENT (src));
 
         if (src->pause_mode == DVDNAVSRC_PAUSE_OFF) {
           dvdnavsrc_print_event (src, data, event, len);
@@ -1336,12 +1532,13 @@ dvdnavsrc_loop (GstElement * element)
           if (info->length == 0xff) {
             GST_INFO_OBJECT (src, "starting unlimited pause");
             src->pause_mode = DVDNAVSRC_PAUSE_UNLIMITED;
+            src->pause_remain = -1;
           } else {
             src->pause_mode = DVDNAVSRC_PAUSE_LIMITED;
-            src->pause_end = current_time + info->length * GST_SECOND;
+            src->pause_remain = info->length * GST_SECOND;
             GST_INFO_OBJECT (src,
-                "starting limited pause: %d seconds at %llu until %llu",
-                info->length, current_time, src->pause_end);
+                "starting limited pause: %d seconds at %llu",
+                info->length, gst_element_get_time (GST_ELEMENT (src)));
           }
 
           /* For the moment, send the first empty event to let
@@ -1355,40 +1552,67 @@ dvdnavsrc_loop (GstElement * element)
         }
 
         if (src->pause_mode == DVDNAVSRC_PAUSE_UNLIMITED ||
-            current_time < src->pause_end) {
+            src->pause_remain > 0) {
           GstEvent *event;
 
-          /* We are in pause mode. Make this element sleep for a
-             fraction of a second. */
-          if (current_time + DVDNAVSRC_PAUSE_INTERVAL > src->pause_end) {
-            gst_element_wait (GST_ELEMENT (src), src->pause_end);
-          } else {
-            gst_element_wait (GST_ELEMENT (src),
-                current_time + DVDNAVSRC_PAUSE_INTERVAL);
-          }
-          /* Send an empty event to keep the pipeline going. */
-          /* FIXME: Use an interrupt/filler event here. */
-          event = gst_event_new (GST_EVENT_EMPTY);
+          /* Send a filler event to keep the pipeline going */
+          event = gst_event_new_filler_stamped (GST_CLOCK_TIME_NONE,
+              MIN (src->pause_remain, DVDNAVSRC_PAUSE_INTERVAL));
           send_data = GST_DATA (event);
-          GST_EVENT_TIMESTAMP (event) =
-              gst_element_get_time (GST_ELEMENT (src));
+
+          GST_DEBUG_OBJECT (src,
+              "Pause mode %d, Sending filler at %" G_GUINT64_FORMAT
+              ", dur %" G_GINT64_FORMAT ", remain %" G_GINT64_FORMAT,
+              src->pause_mode, gst_element_get_time (GST_ELEMENT (src)),
+              MIN (src->pause_remain, DVDNAVSRC_PAUSE_INTERVAL),
+              src->pause_remain);
+
+          if (src->pause_mode == DVDNAVSRC_PAUSE_LIMITED) {
+            if (src->pause_remain < DVDNAVSRC_PAUSE_INTERVAL)
+              src->pause_remain = 0;
+            else
+              src->pause_remain -= DVDNAVSRC_PAUSE_INTERVAL;
+          }
+
+          /* If pause isn't finished, discont because time isn't actually
+           * advancing */
+          if (src->pause_mode == DVDNAVSRC_PAUSE_UNLIMITED ||
+              src->pause_remain > 0)
+            src->did_seek = TRUE;
 
           break;
         } else {
           /* We reached the end of the pause. */
           src->pause_mode = DVDNAVSRC_PAUSE_OFF;
           DVDNAV_CALL (dvdnav_still_skip, (src->dvdnav), src);
+
+          /* Schedule a discont to reset the time */
+          src->did_seek = TRUE;
         }
       }
         break;
 
       case DVDNAV_WAIT:
+      {
+#if 0
+        GstEvent *event;
+
         /* FIXME: We should really wait here until the fifos are
            empty, but I have no idea how to do that.  In the mean time,
            just clean the wait state. */
         GST_INFO_OBJECT (src, "sending wait");
+
+        gst_element_wait (GST_ELEMENT (src),
+            gst_element_get_time (GST_ELEMENT (src)) + 1.5 * GST_SECOND);
+#endif
         DVDNAV_CALL (dvdnav_wait_skip, (src->dvdnav), src);
+
+#if 0
+        event = gst_event_new_filler ();
+        send_data = GST_DATA (event);
+#endif
         break;
+      }
 
       case DVDNAV_STOP:
         GST_INFO_OBJECT (src, "sending eos");
@@ -1399,6 +1623,18 @@ dvdnavsrc_loop (GstElement * element)
         break;
 
       case DVDNAV_CELL_CHANGE:
+      {
+        dvdnav_cell_change_event_t *event = (dvdnav_cell_change_event_t *) data;
+
+        src->pgc_length = MPEGTIME_TO_GSTTIME (event->pgc_length);
+        src->cell_start = MPEGTIME_TO_GSTTIME (event->cell_start);
+        src->pg_start = MPEGTIME_TO_GSTTIME (event->pg_start);
+
+        GST_LOG_OBJECT (src, "New cell. PGC Len %" GST_TIME_FORMAT
+            " cell_start %" GST_TIME_FORMAT ", pg_start %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (src->pgc_length),
+            GST_TIME_ARGS (src->cell_start), GST_TIME_ARGS (src->pg_start));
+      }
         dvdnavsrc_update_streaminfo (src);
         break;
 
@@ -1415,7 +1651,7 @@ dvdnavsrc_loop (GstElement * element)
           }
         }
 
-        dvdnavsrc_update_highlight (src);
+        dvdnavsrc_update_highlight (src, FALSE);
 
         /* Send a dvd nav packet event. */
         send_data = GST_DATA (dvdnavsrc_make_dvd_nav_packet_event (src, pci));
@@ -1429,11 +1665,33 @@ dvdnavsrc_loop (GstElement * element)
 
       case DVDNAV_VTS_CHANGE:
       {
+        GstEvent *e;
+        dvdnav_vts_change_event_t *event = (dvdnav_vts_change_event_t *) data;
+
         dvdnavsrc_set_domain (src);
 
+        if (src->domain == DVDNAVSRC_DOMAIN_VMGM)
+          src->cur_vts = 0;
+        else
+          src->cur_vts = event->new_vtsN;
+
         send_data = GST_DATA (dvdnavsrc_make_dvd_event (src,
                 "dvd-vts-change", "domain",
                 G_TYPE_INT, (gint) src->domain, NULL));
+
+        e = gst_event_new_discontinuous (FALSE, GST_FORMAT_UNDEFINED);
+        gst_pad_push (src->srcpad, GST_DATA (e));
+
+        if (src->domain == DVDNAVSRC_DOMAIN_VTSM ||
+            src->domain == DVDNAVSRC_DOMAIN_VTS ||
+            src->domain == DVDNAVSRC_DOMAIN_VMGM) {
+          e = dvdnav_build_titlelang_event (src);
+          if (!e) {
+            GST_ELEMENT_ERROR (src, LIBRARY, FAILED,
+                (_("Invalid title information on DVD.")), GST_ERROR_SYSTEM);
+          }
+          gst_pad_push (src->srcpad, GST_DATA (e));
+        }
       }
         break;
 
@@ -1498,7 +1756,7 @@ dvdnavsrc_loop (GstElement * element)
       case DVDNAV_HIGHLIGHT:
         dvdnavsrc_print_event (src, data, event, len);
 
-        dvdnavsrc_update_highlight (src);
+        dvdnavsrc_update_highlight (src, FALSE);
         break;
 
       case DVDNAV_HOP_CHANNEL:
@@ -1506,7 +1764,8 @@ dvdnavsrc_loop (GstElement * element)
 
         src->button = 0;
         src->pause_mode = DVDNAVSRC_PAUSE_OFF;
-        send_data = GST_DATA (gst_event_new_flush ());
+        src->need_flush = TRUE;
+        // send_data = GST_DATA (gst_event_new_flush ());
         break;
 
       default:
@@ -1515,6 +1774,7 @@ dvdnavsrc_loop (GstElement * element)
     }
   }
 
+  src->seek_pending = FALSE;
   gst_pad_push (src->srcpad, send_data);
 }
 
@@ -1536,6 +1796,12 @@ dvdnavsrc_open (DVDNavSrc * src)
 
   GST_FLAG_SET (src, DVDNAVSRC_OPEN);
 
+  if (dvdnav_set_PGC_positioning_flag (src->dvdnav, 1) != DVDNAV_STATUS_OK) {
+    GST_ELEMENT_ERROR (src, LIBRARY, FAILED,
+        (_("Failed to set PGC based seeking.")), GST_ERROR_SYSTEM);
+    return FALSE;
+  }
+
   /* Read the first block before seeking to force a libdvdnav internal
    * call to vm_start, otherwise it ignores our seek position.
    * This happens because vm_start sets the domain to the first-play (FP)
@@ -1597,13 +1863,18 @@ dvdnavsrc_change_state (GstElement * element)
     case GST_STATE_NULL_TO_READY:
       break;
     case GST_STATE_READY_TO_PAUSED:
+      if (!read_vts_info (src)) {
+        GST_ELEMENT_ERROR (src, LIBRARY, FAILED,
+            (_("Could not read title information for DVD.")), GST_ERROR_SYSTEM);
+        return GST_STATE_FAILURE;
+      }
       if (!dvdnavsrc_is_open (src)) {
         if (!dvdnavsrc_open (src)) {
           return GST_STATE_FAILURE;
         }
       }
       src->streaminfo = NULL;
-      src->need_newmedia = TRUE;
+      src->did_seek = TRUE;
       break;
     case GST_STATE_PAUSED_TO_PLAYING:
       break;
@@ -1650,9 +1921,51 @@ dvdnav_handle_navigation_event (DVDNavSrc * src, GstEvent * event)
   g_return_val_if_fail (event != NULL, FALSE);
 
   if (strcmp (event_type, "key-press") == 0) {
-    const char *key = gst_structure_get_string (structure, "key");
+    const gchar *key = gst_structure_get_string (structure, "key");
+
+    if (key == NULL)
+      return TRUE;
+
+    if (g_str_equal (key, "Return")) {
+      dvdnav_button_activate (src->dvdnav,
+          dvdnav_get_current_nav_pci (src->dvdnav));
+    } else if (g_str_equal (key, "Left")) {
+      dvdnav_left_button_select (src->dvdnav,
+          dvdnav_get_current_nav_pci (src->dvdnav));
+    } else if (g_str_equal (key, "Right")) {
+      dvdnav_right_button_select (src->dvdnav,
+          dvdnav_get_current_nav_pci (src->dvdnav));
+    } else if (g_str_equal (key, "Up")) {
+      dvdnav_upper_button_select (src->dvdnav,
+          dvdnav_get_current_nav_pci (src->dvdnav));
+    } else if (g_str_equal (key, "Down")) {
+      dvdnav_lower_button_select (src->dvdnav,
+          dvdnav_get_current_nav_pci (src->dvdnav));
+    } else if (g_str_equal (key, "m")) {
+      dvdnav_menu_call (src->dvdnav, DVD_MENU_Escape);
+    } else if (g_str_equal (key, "t")) {
+      dvdnav_menu_call (src->dvdnav, DVD_MENU_Title);
+    } else if (g_str_equal (key, "r")) {
+      dvdnav_menu_call (src->dvdnav, DVD_MENU_Root);
+    } else if (g_str_equal (key, "comma")) {
+      int title = 0;
+      int part = 0;
+
+      if (dvdnav_current_title_info (src->dvdnav, &title, &part) && title > 0
+          && part > 1) {
+        dvdnav_part_play (src->dvdnav, title, part - 1);
+        src->did_seek = TRUE;
+      }
+    } else if (g_str_equal (key, "period")) {
+      int title = 0;
+      int part = 0;
+
+      if (dvdnav_current_title_info (src->dvdnav, &title, &part) && title > 0) {
+        dvdnav_part_play (src->dvdnav, title, part + 1);
+        src->did_seek = TRUE;
+      }
+    }
 
-    g_assert (key != NULL);
     GST_DEBUG ("dvdnavsrc got a keypress: %s", key);
   } else if (strcmp (event_type, "mouse-move") == 0) {
     double x, y;
@@ -1663,12 +1976,13 @@ dvdnav_handle_navigation_event (DVDNavSrc * src, GstEvent * event)
     dvdnav_mouse_select (src->dvdnav,
         dvdnav_get_current_nav_pci (src->dvdnav), (int) x, (int) y);
 
-    dvdnavsrc_update_highlight (src);
+    dvdnavsrc_update_highlight (src, FALSE);
   } else if (strcmp (event_type, "mouse-button-release") == 0) {
     double x, y;
 
     gst_structure_get_double (structure, "pointer_x", &x);
     gst_structure_get_double (structure, "pointer_y", &y);
+    GST_DEBUG_OBJECT (src, "Got click at %g, %g", x, y);
 
     dvdnav_mouse_activate (src->dvdnav,
         dvdnav_get_current_nav_pci (src->dvdnav), (int) x, (int) y);
@@ -1697,21 +2011,34 @@ dvdnavsrc_event (GstPad * pad, GstEvent * event)
       int parts, part, new_part;
       int angles, angle, new_angle;
       int origin;
+      guint32 curoff, len;
 
       format = GST_EVENT_SEEK_FORMAT (event);
       offset = GST_EVENT_SEEK_OFFSET (event);
 
+      /* Disallow seek before the DVD VM has started or if we haven't finished a 
+       * previous seek yet */
+      if (dvdnav_get_position (src->dvdnav, &curoff, &len) != DVDNAV_STATUS_OK
+          || src->seek_pending)
+        goto error;
+
+      GST_DEBUG_OBJECT (src, "Seeking to %lld, format %d, method %d\n", offset,
+          format, GST_EVENT_SEEK_METHOD (event));
+
       switch (format) {
         case GST_FORMAT_BYTES:
           switch (GST_EVENT_SEEK_METHOD (event)) {
             case GST_SEEK_METHOD_SET:
               origin = SEEK_SET;
+              src->pending_offset = offset;
               break;
             case GST_SEEK_METHOD_CUR:
               origin = SEEK_CUR;
+              src->pending_offset = curoff + offset;
               break;
             case GST_SEEK_METHOD_END:
               origin = SEEK_END;
+              src->pending_offset = len + offset;
               break;
             default:
               goto error;
@@ -1720,6 +2047,7 @@ dvdnavsrc_event (GstPad * pad, GstEvent * event)
                   origin) != DVDNAV_STATUS_OK) {
             goto error;
           }
+          break;
         default:
           if (format == sector_format) {
             switch (GST_EVENT_SEEK_METHOD (event)) {
@@ -1818,6 +2146,7 @@ dvdnavsrc_event (GstPad * pad, GstEvent * event)
           }
       }
       src->did_seek = TRUE;
+      src->seek_pending = TRUE;
       src->need_flush = GST_EVENT_SEEK_FLAGS (event) & GST_SEEK_FLAG_FLUSH;
       break;
     }
@@ -1846,8 +2175,8 @@ dvdnavsrc_get_formats (GstPad * pad)
   int i;
   static GstFormat formats[] = {
     GST_FORMAT_BYTES,
+    GST_FORMAT_TIME,
     /*
-       GST_FORMAT_TIME,
        GST_FORMAT_DEFAULT,
      */
     0,                          /* filled later */
@@ -1929,7 +2258,12 @@ dvdnavsrc_query (GstPad * pad, GstQueryType type,
 
   switch (type) {
     case GST_QUERY_TOTAL:
-      if (*format == sector_format) {
+      if (*format == GST_FORMAT_TIME) {
+        if (src->pgc_length != GST_CLOCK_TIME_NONE) {
+          *value = src->pgc_length;
+        } else
+          res = FALSE;
+      } else if (*format == sector_format) {
         if (dvdnav_get_position (src->dvdnav, &pos, &len)
             != DVDNAV_STATUS_OK) {
           res = FALSE;
@@ -1939,7 +2273,7 @@ dvdnavsrc_query (GstPad * pad, GstQueryType type,
         if (dvdnav_get_position (src->dvdnav, &pos, &len) != DVDNAV_STATUS_OK) {
           res = FALSE;
         }
-        *value = len * DVD_SECTOR_SIZE;
+        *value = (gint64) (len) * DVD_SECTOR_SIZE;
       } else if (*format == title_format) {
         if (dvdnav_get_number_of_titles (src->dvdnav, &titles)
             != DVDNAV_STATUS_OK) {
@@ -1969,6 +2303,11 @@ dvdnavsrc_query (GstPad * pad, GstQueryType type,
           res = FALSE;
         }
         *value = pos;
+      } else if (*format == GST_FORMAT_BYTES) {
+        if (dvdnav_get_position (src->dvdnav, &pos, &len) != DVDNAV_STATUS_OK) {
+          res = FALSE;
+        }
+        *value = (gint64) (pos) * DVD_SECTOR_SIZE;
       } else if (*format == title_format) {
         if (dvdnav_current_title_info (src->dvdnav, &title, &part)
             != DVDNAV_STATUS_OK) {
@@ -2011,7 +2350,7 @@ dvdnavsrc_uri_get_type (void)
 static gchar **
 dvdnavsrc_uri_get_protocols (void)
 {
-  static gchar *protocols[] = { "dvdnav", NULL };
+  static gchar *protocols[] = { "dvd", "dvdnav", NULL };
 
   return protocols;
 }
@@ -2019,17 +2358,77 @@ dvdnavsrc_uri_get_protocols (void)
 static const gchar *
 dvdnavsrc_uri_get_uri (GstURIHandler * handler)
 {
-  return "dvdnav://";
+  DVDNavSrc *src = DVDNAVSRC (handler);
+
+  g_free (src->last_uri);
+  src->last_uri =
+      g_strdup_printf ("dvd://%d,%d,%d", src->title, src->chapter, src->angle);
+
+  return src->last_uri;
 }
 
 static gboolean
 dvdnavsrc_uri_set_uri (GstURIHandler * handler, const gchar * uri)
 {
+  DVDNavSrc *src = DVDNAVSRC (handler);
   gboolean ret;
   gchar *protocol = gst_uri_get_protocol (uri);
 
-  ret = (protocol && !strcmp (protocol, "dvdnav")) ? TRUE : FALSE;
+  ret = (protocol &&
+      (!strcmp (protocol, "dvdnav") ||
+          !strcmp (protocol, "dvd"))) ? TRUE : FALSE;
   g_free (protocol);
+  protocol = NULL;
+
+  if (!ret)
+    return ret;
+
+  /*
+   * Parse out the new t/c/a and seek to them
+   */
+  {
+    gchar *location = NULL;
+    gchar **strs;
+    gchar **strcur;
+    gint pos = 0;
+
+    location = gst_uri_get_location (uri);
+
+    if (!location)
+      return ret;
+
+    strcur = strs = g_strsplit (location, ",", 0);
+    while (strcur && *strcur) {
+      gint val;
+
+      if (!sscanf (*strcur, "%d", &val))
+        break;
+
+      switch (pos) {
+        case 0:
+          if (val != src->title) {
+            src->title = val;
+            src->new_seek = TRUE;
+          }
+          break;
+        case 1:
+          if (val != src->chapter) {
+            src->chapter = val;
+            src->new_seek = TRUE;
+          }
+          break;
+        case 2:
+          src->angle = val;
+          break;
+      }
+
+      strcur++;
+      pos++;
+    }
+
+    g_strfreev (strs);
+    g_free (location);
+  }
 
   return ret;
 }
@@ -2048,7 +2447,7 @@ dvdnavsrc_uri_handler_init (gpointer g_iface, gpointer iface_data)
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  if (!gst_element_register (plugin, "dvdnavsrc", GST_RANK_NONE,
+  if (!gst_element_register (plugin, "dvdnavsrc", GST_RANK_PRIMARY,
           GST_TYPE_DVDNAVSRC))
     return FALSE;
 
