@@ -28,7 +28,11 @@
 extern "C" {
 #endif /* __cplusplus */
 
+#ifdef HAVE_LAME_UNINSTALLED
+#include <include/lame.h>
+#else
 #include <lame/lame.h>
+#endif
 
 #define GST_TYPE_LAME \
   (gst_lame_get_type())
