@@ -35,6 +35,7 @@
 #define GST_IS_MAD_CLASS(obj) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_MAD))
 
+
 typedef struct _GstMad GstMad;
 typedef struct _GstMadClass GstMadClass;
 
@@ -49,14 +50,14 @@ struct _GstMad
   struct mad_stream stream;
   struct mad_frame frame;
   struct mad_synth synth;
-  guchar *tempbuffer;
-  glong tempsize;               /* used to keep track of partial buffers */
-  gboolean need_sync;
+  guchar *tempbuffer;           /* temporary buffer to serve to mad */
+  glong tempsize;               /* running count of temp buffer size */
   GstClockTime last_ts;
   guint64 base_byte_offset;
   guint64 bytes_consumed;       /* since the base_byte_offset */
   guint64 total_samples;        /* the number of samples since the sync point */
 
+  gboolean in_error;            /* set when mad's in an error state */
   gboolean restart;
   guint64 segment_start;
 
@@ -73,13 +74,16 @@ struct _GstMad
   GstTagList *tags;
 
   /* negotiated format */
-  gint rate;
-  gint channels;
+  gint rate, pending_rate;
+  gint channels, pending_channels;
+  gint times_pending;
 
+  gboolean caps_set;            /* used to keep track of whether to change/update caps */
   GstIndex *index;
   gint index_id;
 
   gboolean check_for_xing;
+  gboolean xing_found;
 };
 
 struct _GstMadClass
@@ -112,6 +116,9 @@ enum
       /* FILL ME */
 };
 
+GST_DEBUG_CATEGORY_STATIC (mad_debug);
+#define GST_CAT_DEFAULT mad_debug
+
 static GstStaticPadTemplate mad_src_template_factory =
 GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -121,15 +128,20 @@ GST_STATIC_PAD_TEMPLATE ("src",
         "signed = (boolean) true, "
         "width = (int) 16, "
         "depth = (int) 16, "
-        "rate = (int) [ 11025, 48000 ], " "channels = (int) [ 1, 2 ]")
+        "rate = (int) { 8000, 11025, 12000, 16000, 22050, 24000, 32000, 44100, 48000 }, "
+        "channels = (int) [ 1, 2 ]")
     );
 
+/* FIXME: make three caps, for mpegversion 1, 2 and 2.5 */
 static GstStaticPadTemplate mad_sink_template_factory =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("audio/mpeg, "
-        "mpegversion = (int) 1, " "layer = (int) [ 1, 3 ]")
+        "mpegversion = (int) 1, "
+        "layer = (int) [ 1, 3 ], "
+        "rate = (int) { 8000, 11025, 12000, 16000, 22050, 24000, 32000, 44100, 48000 }, "
+        "channels = (int) [ 1, 2 ]")
     );
 
 static void gst_mad_base_init (gpointer g_class);
@@ -183,9 +195,11 @@ gst_mad_get_type (void)
       0,
       (GInstanceInitFunc) gst_mad_init,
     };
+
     mad_type =
         g_type_register_static (GST_TYPE_ELEMENT, "GstMad", &mad_info, 0);
   }
+  GST_DEBUG_CATEGORY_INIT (mad_debug, "mad", 0, "mad mp3 decoding");
   return mad_type;
 }
 
@@ -201,6 +215,7 @@ gst_mad_layer_get_type (void)
     {MAD_LAYER_III, "3", "III"},
     {0, NULL, NULL},
   };
+
   if (!mad_layer_type) {
     mad_layer_type = g_enum_register_static ("GstMadLayer", mad_layer);
   }
@@ -220,6 +235,7 @@ gst_mad_mode_get_type (void)
     {MAD_MODE_STEREO, "3", "Stereo"},
     {0, NULL, NULL},
   };
+
   if (!mad_mode_type) {
     mad_mode_type = g_enum_register_static ("GstMadMode", mad_mode);
   }
@@ -239,6 +255,7 @@ gst_mad_emphasis_get_type (void)
     {MAD_EMPHASIS_RESERVED, "3", "Reserved"},
     {0, NULL, NULL},
   };
+
   if (!mad_emphasis_type) {
     mad_emphasis_type = g_enum_register_static ("GstMadEmphasis", mad_emphasis);
   }
@@ -284,7 +301,7 @@ gst_mad_class_init (GstMadClass * klass)
           FALSE, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_IGNORE_CRC,
       g_param_spec_boolean ("ignore_crc", "Ignore CRC", "Ignore CRC errors",
-          FALSE, G_PARAM_READWRITE));
+          TRUE, G_PARAM_READWRITE));
 
   /* register tags */
 #define GST_TAG_LAYER    "layer"
@@ -333,7 +350,6 @@ gst_mad_init (GstMad * mad)
 
   mad->tempbuffer = g_malloc (MAD_BUFFER_MDLEN * 3);
   mad->tempsize = 0;
-  mad->need_sync = TRUE;
   mad->base_byte_offset = 0;
   mad->bytes_consumed = 0;
   mad->total_samples = 0;
@@ -348,8 +364,9 @@ gst_mad_init (GstMad * mad)
   mad->tags = NULL;
 
   mad->half = FALSE;
-  mad->ignore_crc = FALSE;
+  mad->ignore_crc = TRUE;
   mad->check_for_xing = TRUE;
+  mad->xing_found = FALSE;
   GST_FLAG_SET (mad, GST_ELEMENT_EVENT_AWARE);
 }
 
@@ -409,6 +426,7 @@ gst_mad_get_event_masks (GstPad * pad)
     {GST_EVENT_SEEK, GST_SEEK_METHOD_SET | GST_SEEK_FLAG_FLUSH},
     {0,}
   };
+
   return gst_mad_src_event_masks;
 }
 
@@ -526,6 +544,7 @@ gst_mad_get_query_types (GstPad * pad)
     GST_QUERY_POSITION,
     0
   };
+
   return gst_mad_src_query_types;
 }
 
@@ -617,6 +636,8 @@ index_seek (GstMad * mad, GstPad * pad, GstEvent * event)
       GST_EVENT_SEEK_FORMAT (event),
       GST_EVENT_SEEK_OFFSET (event));
 
+  GST_DEBUG ("index seek");
+
   if (!entry)
     return FALSE;
 
@@ -661,6 +682,8 @@ normal_seek (GstMad * mad, GstPad * pad, GstEvent * event)
   const GstFormat *peer_formats;
   gboolean res;
 
+  GST_DEBUG ("normal seek");
+
   format = GST_FORMAT_TIME;
 
   /* first bring the src_format to TIME */
@@ -671,6 +694,7 @@ normal_seek (GstMad * mad, GstPad * pad, GstEvent * event)
     return FALSE;
   }
 
+  GST_DEBUG ("seek to time %" GST_TIME_FORMAT, GST_TIME_ARGS (src_offset));
   /* shave off the flush flag, we'll need it later */
   flush = GST_EVENT_SEEK_FLAGS (event) & GST_SEEK_FLAG_FLUSH;
 
@@ -691,7 +715,8 @@ normal_seek (GstMad * mad, GstPad * pad, GstEvent * event)
       GstEvent *seek_event;
 
       /* conversion succeeded, create the seek */
-      seek_event = gst_event_new_seek (format | GST_SEEK_METHOD_SET | flush,
+      seek_event =
+          gst_event_new_seek (format | GST_EVENT_SEEK_METHOD (event) | flush,
           desired_offset);
       /* do the seek */
       if (gst_pad_send_event (GST_PAD_PEER (mad->sinkpad), seek_event)) {
@@ -720,6 +745,7 @@ gst_mad_src_event (GstPad * pad, GstEvent * event)
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK_SEGMENT:
+      GST_DEBUG ("forwarding seek event to sink pad");
       gst_event_ref (event);
       if (gst_pad_send_event (GST_PAD_PEER (mad->sinkpad), event)) {
         /* seek worked, we're done, loop will exit */
@@ -728,10 +754,17 @@ gst_mad_src_event (GstPad * pad, GstEvent * event)
       break;
       /* the all-formats seek logic */
     case GST_EVENT_SEEK:
-      if (mad->index)
-        res = index_seek (mad, pad, event);
-      else
-        res = normal_seek (mad, pad, event);
+      GST_DEBUG ("forwarding seek event to sink pad");
+      gst_event_ref (event);
+      if (gst_pad_send_event (GST_PAD_PEER (mad->sinkpad), event)) {
+        /* seek worked, we're done, loop will exit */
+        res = TRUE;
+      } else {
+        if (mad->index)
+          res = index_seek (mad, pad, event);
+        else
+          res = normal_seek (mad, pad, event);
+      }
       break;
 
     default:
@@ -816,7 +849,7 @@ gst_mad_update_info (GstMad * mad)
 G_STMT_START{							\
   if (mad->header.h1 != header->h1 || mad->new_header) {	\
     mad->header.h1 = header->h1;				\
-     changed = TRUE;						\
+    changed = TRUE;						\
   };								\
 } G_STMT_END
 
@@ -834,10 +867,7 @@ G_STMT_START{							\
   CHECK_HEADER (layer, "layer");
   CHECK_HEADER (mode, "mode");
   CHECK_HEADER (emphasis, "emphasis");
-
-  if (header->bitrate != mad->header.bitrate || mad->new_header) {
-    mad->header.bitrate = header->bitrate;
-  }
+  mad->header.bitrate = header->bitrate;
   mad->new_header = FALSE;
 
   if (changed) {
@@ -856,11 +886,13 @@ G_STMT_START{							\
         GST_TAG_LAYER, mad->header.layer,
         GST_TAG_MODE, mode->value_nick,
         GST_TAG_EMPHASIS, emphasis->value_nick, NULL);
-    gst_element_found_tags (GST_ELEMENT (mad), list);
-    gst_tag_list_free (list);
+    if (!mad->xing_found) {
+      gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
+          GST_TAG_BITRATE, mad->header.bitrate, NULL);
+    }
+    gst_element_found_tags_for_pad (GST_ELEMENT (mad), mad->srcpad, 0, list);
   }
 #undef CHECK_HEADER
-
 }
 
 static void
@@ -869,6 +901,7 @@ gst_mad_handle_event (GstPad * pad, GstBuffer * buffer)
   GstEvent *event = GST_EVENT (buffer);
   GstMad *mad = GST_MAD (gst_pad_get_parent (pad));
 
+  GST_DEBUG ("handling event %d", GST_EVENT_TYPE (event));
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_DISCONTINUOUS:
     {
@@ -885,46 +918,59 @@ gst_mad_handle_event (GstPad * pad, GstBuffer * buffer)
         if (gst_formats_contains (formats, GST_EVENT_DISCONT_OFFSET (event,
                     i).format)) {
           gint64 value = GST_EVENT_DISCONT_OFFSET (event, i).value;
-          gint64 time;
+          gint64 time, total_samples;
           GstFormat format;
           GstEvent *discont;
 
           /* see how long the input bytes take */
-          format = GST_FORMAT_TIME;
-          if (!gst_pad_convert (pad,
-                  GST_EVENT_DISCONT_OFFSET (event, i).format,
-                  value, &format, &time)) {
-            time = 0;
+          if (GST_EVENT_DISCONT_OFFSET (event, i).format != GST_FORMAT_TIME) {
+            format = GST_FORMAT_TIME;
+            if (!gst_pad_convert (pad,
+                    GST_EVENT_DISCONT_OFFSET (event, i).format,
+                    value, &format, &time)) {
+              continue;
+            }
+          } else {
+            time = value;
           }
 
           /* for now, this is the best we can do to get the total number
-           * of samples */
+           * of samples. This is suboptimal because the incoming event
+           * might contain this information already (although I believe
+           * that this doesn't happen anywhere so far). */
           format = GST_FORMAT_DEFAULT;
           if (!gst_pad_convert (mad->srcpad,
-                  GST_FORMAT_TIME, time, &format, &mad->total_samples)) {
-            mad->total_samples = 0;
+                  GST_FORMAT_TIME, time, &format, &total_samples)) {
+            continue;
           }
-
-          gst_event_unref (event);
+          mad->total_samples = total_samples;
 
           if (GST_PAD_IS_USABLE (mad->srcpad)) {
             discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME,
                 time, NULL);
             gst_pad_push (mad->srcpad, GST_DATA (discont));
           }
-          break;
+          gst_event_unref (event);
+          goto done;
         }
       }
+      GST_WARNING ("Failed to retrieve sample position");
+      /* this isn't really correct? */
+      gst_pad_event_default (pad, event);
+    done:
       mad->tempsize = 0;
       /* we don't need to restart when we get here */
       mad->restart = FALSE;
       break;
     }
+    case GST_EVENT_EOS:
+      mad->caps_set = FALSE;    /* could be a new stream */
+      gst_pad_event_default (pad, event);
+      break;
     default:
       gst_pad_event_default (pad, event);
       break;
   }
-  return;
 }
 
 static gboolean
@@ -946,7 +992,7 @@ gst_mad_check_restart (GstMad * mad)
  * This code has been kindly relicensed to LGPL by Thibaut Mattern and 
  * Bastien Nocera
  */
-#define BE_32(x) GINT32_FROM_BE(*(gint*)(x))
+#define BE_32(x) GST_READ_UINT32_BE(x)
 
 #define FOURCC_TAG( ch0, ch1, ch2, ch3 )		\
 	( (long)(unsigned char)(ch3) |			\
@@ -971,11 +1017,18 @@ is_xhead (unsigned char *buf)
 
 
 #undef LOG
-//#define LOG
+/*#define LOG*/
 #ifdef LOG
 #define lprintf(x...) g_print(x)
 #else
-#define lprintf(x...)
+# if defined(G_HAVE_GNUC_VARARGS)
+# define lprintf(x...)
+# else
+static inline void
+lprintf (x, ...)
+{
+}
+# endif
 #endif
 
 static int
@@ -988,23 +1041,18 @@ mpg123_parse_xing_header (struct mad_header *header,
   int xflags, xframes, xbytes, xvbr_scale;
   int abr;
   guint8 xtoc[XING_TOC_LENGTH];
-
-  /* This should be the MPEG Audio version ID 
-   * (version 2.5, 2 or 1) least significant byte, but mad doesn't 
-   * provide that, so assume it's always MPEG 1
-   */
-  int lsf_bit = 1;
+  int lsf_bit = !(header->flags & MAD_FLAG_LSF_EXT);
 
   xframes = xbytes = 0;
 
   /* offset of the Xing header */
   if (lsf_bit) {
-    if (header->mode != MAD_MODE_STEREO)
+    if (header->mode != MAD_MODE_SINGLE_CHANNEL)
       ptr += (32 + 4);
     else
       ptr += (17 + 4);
   } else {
-    if (header->mode != MAD_MODE_STEREO)
+    if (header->mode != MAD_MODE_SINGLE_CHANNEL)
       ptr += (17 + 4);
     else
       ptr += (9 + 4);
@@ -1092,6 +1140,84 @@ mpg123_parse_xing_header (struct mad_header *header,
 
 /* End of Xine code */
 
+/* internal function to check if the header has changed and thus the
+ * caps need to be reset.  Only call during normal mode, not resyncing */
+static gboolean
+gst_mad_check_caps_reset (GstMad * mad)
+{
+  guint nchannels;
+  guint rate, old_rate = mad->rate;
+
+  nchannels = MAD_NCHANNELS (&mad->frame.header);
+
+#if MAD_VERSION_MINOR <= 12
+  rate = mad->header.sfreq;
+#else
+  rate = mad->frame.header.samplerate;
+#endif
+  if (mad->stream.options & MAD_OPTION_HALFSAMPLERATE) {
+    GST_INFO_OBJECT (mad,
+        "MAD_OPTION_HALFSAMPLERATE is set, adapting rate from %u to %u", rate,
+        rate >> 1);
+    rate >>= 1;
+  }
+
+  /* rate and channels are not supposed to change in a continuous stream,
+   * so check this first before doing anything */
+
+  /* only set caps if they weren't already set for this continuous stream */
+  if (mad->channels != nchannels || mad->rate != rate) {
+    if (mad->caps_set) {
+      GST_DEBUG
+          ("Header changed from %d Hz/%d ch to %d Hz/%d ch, failed sync after seek ?",
+          mad->rate, mad->channels, rate, nchannels);
+      /* we're conservative on stream changes. However, our *initial* caps
+       * might have been wrong as well - mad ain't perfect in syncing. So,
+       * we count caps changes and change if we pass a limit treshold (3). */
+      if (nchannels != mad->pending_channels || rate != mad->pending_rate) {
+        mad->times_pending = 0;
+        mad->pending_channels = nchannels;
+        mad->pending_rate = rate;
+      }
+      /* Now, we already have a valid caps set and will continue to use
+       * that for a while longer, so we cans afely return TRUE here. */
+      if (++mad->times_pending < 3)
+        return TRUE;
+    }
+  }
+  gst_mad_update_info (mad);
+
+  if (mad->channels != nchannels || mad->rate != rate) {
+    GstCaps *caps;
+
+    /* we set the caps even when the pad is not connected so they
+     * can be gotten for streaminfo */
+    caps = gst_caps_new_simple ("audio/x-raw-int",
+        "endianness", G_TYPE_INT, G_BYTE_ORDER,
+        "signed", G_TYPE_BOOLEAN, TRUE,
+        "width", G_TYPE_INT, 16,
+        "depth", G_TYPE_INT, 16,
+        "rate", G_TYPE_INT, rate, "channels", G_TYPE_INT, nchannels, NULL);
+
+    if (gst_pad_set_explicit_caps (mad->srcpad, caps)) {
+      mad->caps_set = TRUE;     /* set back to FALSE on discont */
+      mad->channels = nchannels;
+      mad->rate = rate;
+
+      /* update sample count so we don't come up with crazy timestamps */
+      if (mad->total_samples && old_rate) {
+        mad->total_samples = mad->total_samples * rate / old_rate;
+      }
+    } else {
+      GST_ELEMENT_ERROR (mad, CORE, NEGOTIATION, (NULL),
+          ("Failed to negotiate %d Hz, %d channels", rate, nchannels));
+      return FALSE;
+    }
+    gst_caps_free (caps);
+  }
+
+  return TRUE;
+}
 
 static void
 gst_mad_chain (GstPad * pad, GstData * _data)
@@ -1112,17 +1238,20 @@ gst_mad_chain (GstPad * pad, GstData * _data)
     return;
   }
 
-  gst_mad_check_restart (mad);
+  /* restarts happen on discontinuities, ie. seek, flush, PAUSED to PLAYING */
+  if (gst_mad_check_restart (mad))
+    GST_DEBUG ("mad restarted");
 
   timestamp = GST_BUFFER_TIMESTAMP (buffer);
 
   /* handle timestamps */
   if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
-    /* if there is nothing queued (partial buffer), we prepare to set the
-     * timestamp on the next buffer */
+    /* if there is nothing left to process in our temporary buffer,
+     * we can set this timestamp on the next outgoing buffer */
     if (mad->tempsize == 0) {
-      /* we have to save the result here because we can't yet convert the timestamp
-       * to a sample offset yet, the samplerate might not be known yet */
+      /* we have to save the result here because we can't yet convert
+       * the timestamp to a sample offset yet,
+       * the samplerate might not be known yet */
       mad->last_ts = timestamp;
       mad->base_byte_offset = GST_BUFFER_OFFSET (buffer);
       mad->bytes_consumed = 0;
@@ -1135,19 +1264,28 @@ gst_mad_chain (GstPad * pad, GstData * _data)
   }
 
   /* handle data */
-  data = GST_BUFFER_DATA (buffer);
+  data = (gchar *) GST_BUFFER_DATA (buffer);
   size = GST_BUFFER_SIZE (buffer);
 
-
   /* process the incoming buffer in chunks of maximum MAD_BUFFER_MDLEN bytes;
    * this is the upper limit on processable chunk sizes set by mad */
   while (size > 0) {
     gint tocopy;
-    guchar *mad_input_buffer;
+    guchar *mad_input_buffer;   /* convenience pointer to tempbuffer */
 
-    tocopy = MIN (MAD_BUFFER_MDLEN, size);
+    tocopy =
+        MIN (MAD_BUFFER_MDLEN, MIN (size,
+            MAD_BUFFER_MDLEN * 3 - mad->tempsize));
+    if (tocopy == 0) {
+      GST_ELEMENT_ERROR (mad, STREAM, DECODE, (NULL),
+          ("mad claims to need more data than %u bytes, we don't have that much",
+              MAD_BUFFER_MDLEN * 3));
+      return;
+    }
 
     /* append the chunk to process to our internal temporary buffer */
+    GST_LOG ("tempbuffer size %d, copying %d bytes from incoming buffer",
+        mad->tempsize, tocopy);
     memcpy (mad->tempbuffer + mad->tempsize, data, tocopy);
     mad->tempsize += tocopy;
 
@@ -1159,20 +1297,38 @@ gst_mad_chain (GstPad * pad, GstData * _data)
 
     /* while we have data we can consume it */
     while (mad->tempsize >= 0) {
-      gint consumed;
-      guint nchannels;
+      gint consumed = 0;
       guint nsamples;
-      gint rate;
       guint64 time_offset;
       guint64 time_duration;
+      gboolean resync = TRUE;
+
+      mad->in_error = FALSE;
 
       mad_stream_buffer (&mad->stream, mad_input_buffer, mad->tempsize);
 
+      /* added separate header decoding to catch errors earlier, also fixes
+       * some weird decoding errors... */
+      GST_LOG ("decoding one frame now");
+
       if (mad_frame_decode (&mad->frame, &mad->stream) == -1) {
+        GST_LOG ("got error %d", mad->stream.error);
+
         /* not enough data, need to wait for next buffer? */
         if (mad->stream.error == MAD_ERROR_BUFLEN) {
-          break;
+          if (mad->stream.next_frame == mad_input_buffer) {
+            GST_LOG ("not enough data in tempbuffer (%d), breaking to get more",
+                mad->tempsize);
+            break;
+          } else {
+            GST_LOG ("sync error, flushing unneeded data");
+            goto next;
+          }
         }
+        /* we are in an error state */
+        mad->in_error = TRUE;
+        GST_DEBUG ("mad_frame_decode had an error: %s",
+            mad_stream_errorstr (&mad->stream));
         if (!MAD_RECOVERABLE (mad->stream.error)) {
           GST_ELEMENT_ERROR (mad, STREAM, DECODE, (NULL), (NULL));
           return;
@@ -1180,6 +1336,8 @@ gst_mad_chain (GstPad * pad, GstData * _data)
           /* lost sync, force a resync */
           signed long tagsize;
 
+          GST_INFO ("recoverable lost sync error");
+
           tagsize = id3_tag_query (mad->stream.this_frame,
               mad->stream.bufend - mad->stream.this_frame);
 
@@ -1197,13 +1355,16 @@ gst_mad_chain (GstPad * pad, GstData * _data)
              * id3 tags, so we need to flush one byte less than the tagsize */
             mad_stream_skip (&mad->stream, tagsize - 1);
 
+            /* When we skip, we don't want to call sync */
+            resync = FALSE;
+
             tag = id3_tag_parse (data, tagsize);
             if (tag) {
               GstTagList *list;
 
               list = gst_mad_id3_to_tag_list (tag);
               id3_tag_delete (tag);
-              GST_DEBUG_OBJECT (mad, "found tag");
+              GST_DEBUG ("found tag");
               gst_element_found_tags (GST_ELEMENT (mad), list);
               if (mad->tags) {
                 gst_tag_list_insert (mad->tags, list, GST_TAG_MERGE_PREPEND);
@@ -1218,70 +1379,65 @@ gst_mad_chain (GstPad * pad, GstData * _data)
             }
           }
         }
+        //Should not sync here if mad_skip has been used before, the offset
+        //is "pending" inside mad and will be applied on next call to decode.
+        if (resync) {
+          unsigned char const *before_sync, *after_sync;
+
+          before_sync = mad->stream.ptr.byte;
+          if (mad_stream_sync (&mad->stream) != 0) {
+            consumed = MAD_BUFFER_GUARD < mad->tempsize ?
+                mad->tempsize - MAD_BUFFER_GUARD : 0;
+            GST_DEBUG_OBJECT (mad,
+                "mad_stream_sync failed, skipping all %u bytes we have",
+                consumed);
+          } else {
+            after_sync = mad->stream.ptr.byte;
+            /* a succesful resync should make us drop bytes as consumed, so
+               calculate from the byte pointers before and after resync */
+            consumed = after_sync - before_sync;
+            GST_DEBUG_OBJECT (mad, "resynchronization consumes %d bytes",
+                consumed);
+            GST_DEBUG_OBJECT (mad, "synced to data: 0x%0x 0x%0x",
+                *mad->stream.ptr.byte, *(mad->stream.ptr.byte + 1));
+
+            /* recoverable errors pass */
+          }
+          resync = FALSE;
+        }
 
-        mad_frame_mute (&mad->frame);
-        mad_synth_mute (&mad->synth);
-        mad_stream_sync (&mad->stream);
-        /* recoverable errors pass */
         goto next;
       }
 
       if (mad->check_for_xing) {
-        int bitrate, time;
+        int bitrate = 0, time = 0;
         GstTagList *list;
         int frame_len = mad->stream.next_frame - mad->stream.this_frame;
 
         /* Assume Xing headers can only be the first frame in a mp3 file */
         if (mpg123_parse_xing_header (&mad->frame.header,
                 mad->stream.this_frame, frame_len, &bitrate, &time)) {
+          mad->xing_found = TRUE;
           list = gst_tag_list_new ();
           gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
               GST_TAG_DURATION, (gint64) time * 1000 * 1000 * 1000,
               GST_TAG_BITRATE, bitrate, NULL);
-          gst_element_found_tags (GST_ELEMENT (mad), list);
-          if (GST_PAD_IS_USABLE (mad->srcpad)) {
-            gst_pad_push (mad->srcpad, GST_DATA (gst_event_new_tag (list)));
-          } else {
-            gst_tag_list_free (list);
-          }
+          gst_element_found_tags_for_pad (GST_ELEMENT (mad), mad->srcpad, 0,
+              list);
         }
 
         mad->check_for_xing = FALSE;
         goto next;
       }
-      nchannels = MAD_NCHANNELS (&mad->frame.header);
+
+      /* if we're not resyncing/in error, check if caps need to be set again */
+      if (!mad->in_error) {
+        if (!gst_mad_check_caps_reset (mad))
+          goto end;
+      }
       nsamples = MAD_NSBSAMPLES (&mad->frame.header) *
           (mad->stream.options & MAD_OPTION_HALFSAMPLERATE ? 16 : 32);
 
-#if MAD_VERSION_MINOR <= 12
-      rate = mad->header.sfreq;
-#else
-      rate = mad->frame.header.samplerate;
-#endif
-
-      gst_mad_update_info (mad);
-
-      if (mad->channels != nchannels || mad->rate != rate) {
-        GstCaps *caps;
-
-        if (mad->stream.options & MAD_OPTION_HALFSAMPLERATE)
-          rate >>= 1;
-
-        /* we set the caps even when the pad is not connected so they
-         * can be gotten for streaminfo */
-        caps = gst_caps_new_simple ("audio/x-raw-int",
-            "endianness", G_TYPE_INT, G_BYTE_ORDER,
-            "signed", G_TYPE_BOOLEAN, TRUE,
-            "width", G_TYPE_INT, 16,
-            "depth", G_TYPE_INT, 16,
-            "rate", G_TYPE_INT, rate, "channels", G_TYPE_INT, nchannels, NULL);
-
-        gst_pad_set_explicit_caps (mad->srcpad, caps);
-        gst_caps_free (caps);
-        mad->channels = nchannels;
-        mad->rate = rate;
-      }
-
       if (mad->frame.header.samplerate == 0) {
         g_warning
             ("mad->frame.header.samplerate is 0; timestamps cannot be calculated");
@@ -1291,14 +1447,15 @@ gst_mad_chain (GstPad * pad, GstData * _data)
         /* if we have a pending timestamp, we can use it now to calculate the sample offset */
         if (GST_CLOCK_TIME_IS_VALID (mad->last_ts)) {
           GstFormat format = GST_FORMAT_DEFAULT;
+          gint64 total_samples;
 
           gst_pad_convert (mad->srcpad, GST_FORMAT_TIME, mad->last_ts, &format,
-              &mad->total_samples);
+              &total_samples);
+          mad->total_samples = total_samples;
           mad->last_ts = GST_CLOCK_TIME_NONE;
         }
-        time_offset = mad->total_samples * GST_SECOND
-            / mad->frame.header.samplerate;
-        time_duration = (nsamples * GST_SECOND / mad->frame.header.samplerate);
+        time_offset = mad->total_samples * (GST_SECOND / mad->rate);
+        time_duration = (nsamples * (GST_SECOND / mad->rate));
       }
 
       if (mad->index) {
@@ -1323,7 +1480,7 @@ gst_mad_chain (GstPad * pad, GstData * _data)
         left_ch = mad->synth.pcm.samples[0];
         right_ch = mad->synth.pcm.samples[1];
 
-        outbuffer = gst_buffer_new_and_alloc (nsamples * nchannels * 2);
+        outbuffer = gst_buffer_new_and_alloc (nsamples * mad->channels * 2);
         outdata = (gint16 *) GST_BUFFER_DATA (outbuffer);
 
         GST_BUFFER_TIMESTAMP (outbuffer) = time_offset;
@@ -1331,7 +1488,7 @@ gst_mad_chain (GstPad * pad, GstData * _data)
         GST_BUFFER_OFFSET (outbuffer) = mad->total_samples;
 
         /* output sample(s) in 16-bit signed native-endian PCM */
-        if (nchannels == 1) {
+        if (mad->channels == 1) {
           gint count = nsamples;
 
           while (count--) {
@@ -1346,6 +1503,8 @@ gst_mad_chain (GstPad * pad, GstData * _data)
           }
         }
 
+        GST_DEBUG ("Pushing buf with ts %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuffer)));
         gst_pad_push (mad->srcpad, GST_DATA (outbuffer));
       }
 
@@ -1366,12 +1525,22 @@ gst_mad_chain (GstPad * pad, GstData * _data)
 
     next:
       /* figure out how many bytes mad consumed */
-      consumed = mad->stream.next_frame - mad_input_buffer;
+      /* if consumed is already set, it's from the resync higher up, so
+         we need to use that value instead.  Otherwise, recalculate from
+         mad's consumption */
+      if (consumed == 0)
+        consumed = mad->stream.next_frame - mad_input_buffer;
+
+      if (mad->stream.skiplen > consumed)
+        consumed = mad->stream.skiplen;
+      GST_LOG ("mad consumed %d bytes", consumed);
       /* move out pointer to where mad want the next data */
       mad_input_buffer += consumed;
       mad->tempsize -= consumed;
       mad->bytes_consumed += consumed;
+      mad->stream.skiplen = 0;
     }
+    /* we only get here from breaks, tempsize never actually drops below 0 */
     memmove (mad->tempbuffer, mad_input_buffer, mad->tempsize);
   }
 
@@ -1400,6 +1569,8 @@ gst_mad_change_state (GstElement * element)
       mad->total_samples = 0;
       mad->rate = 0;
       mad->channels = 0;
+      mad->caps_set = FALSE;
+      mad->times_pending = mad->pending_rate = mad->pending_channels = 0;
       mad->vbr_average = 0;
       mad->segment_start = 0;
       mad->new_header = TRUE;
@@ -1424,6 +1595,7 @@ gst_mad_change_state (GstElement * element)
       mad_frame_finish (&mad->frame);
       mad_stream_finish (&mad->stream);
       mad->restart = TRUE;
+      mad->check_for_xing = TRUE;
       if (mad->tags) {
         gst_tag_list_free (mad->tags);
         mad->tags = NULL;
