@@ -1,20 +1,38 @@
-
-
+/* GStreamer
+ * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
+ * Copyright (C) <2003> David Schleef <ds@schleef.org>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 #include <gst/gst.h>
+#include "gsttextrender.h"
 #include "gsttextoverlay.h"
-/*#include "gsttexttestsrc.h"*/
-/*#include "gstsubparse.h"*/
-/*#include "SDL_blit.h"*/
+
+GST_DEBUG_CATEGORY (pango_debug);
+#define GST_CAT_DEFAULT pango_debug
 
 static GstElementDetails textoverlay_details = {
   "Text Overlay",
   "Filter/Editor/Video",
   "Adds text strings on top of a video buffer",
-  "Gustavo J. A. M. Carneiro <gjc@inescporto.pt>"
+  "David Schleef <ds@schleef.org>"
 };
 
 enum
@@ -25,7 +43,7 @@ enum
   ARG_HALIGN,
   ARG_X0,
   ARG_Y0,
-  ARG_FONT_DESC,
+  ARG_FONT_DESC
 };
 
 
@@ -88,6 +106,7 @@ gst_textoverlay_get_type (void)
       0,
       (GInstanceInitFunc) gst_textoverlay_init,
     };
+
     textoverlay_type =
         g_type_register_static (GST_TYPE_ELEMENT, "GstTextOverlay",
         &textoverlay_info, 0);
@@ -199,7 +218,7 @@ render_text (GstTextOverlay * overlay)
 
   pango_layout_get_pixel_extents (overlay->layout, &ink_rect, &logical_rect);
   resize_bitmap (overlay, ink_rect.width, ink_rect.height + ink_rect.y);
-  pango_ft2_render_layout (&overlay->bitmap, overlay->layout, 0, 0);
+  pango_ft2_render_layout (&overlay->bitmap, overlay->layout, -ink_rect.x, 0);
   overlay->baseline_y = ink_rect.y;
 }
 
@@ -209,23 +228,88 @@ render_text (GstTextOverlay * overlay)
 /*     return GST_PAD_LINK_DONE; */
 /* } */
 
-
-static GstPadLinkReturn
-gst_textoverlay_video_sinkconnect (GstPad * pad, const GstCaps * caps)
+static GList *
+gst_textoverlay_linkedpads (GstPad * pad)
 {
+  GstPad *otherpad;
   GstTextOverlay *overlay;
-  GstStructure *structure;
 
   overlay = GST_TEXTOVERLAY (gst_pad_get_parent (pad));
+  if (pad == overlay->text_sinkpad)
+    return NULL;
+  otherpad = (pad == overlay->video_sinkpad) ?
+      overlay->srcpad : overlay->video_sinkpad;
+
+  return g_list_append (NULL, otherpad);
+}
+
+static GstPadLinkReturn
+gst_textoverlay_link (GstPad * pad, const GstCaps * caps)
+{
+  GstPad *otherpad;
+  GstTextOverlay *overlay;
+  GstStructure *structure;
+  GstPadLinkReturn ret;
+
+  overlay = GST_TEXTOVERLAY (gst_pad_get_parent (pad));
+  otherpad = (pad == overlay->video_sinkpad) ?
+      overlay->srcpad : overlay->video_sinkpad;
+
+  ret = gst_pad_try_set_caps (otherpad, caps);
+  if (GST_PAD_LINK_FAILED (ret))
+    return ret;
 
   structure = gst_caps_get_structure (caps, 0);
   overlay->width = overlay->height = 0;
   gst_structure_get_int (structure, "width", &overlay->width);
   gst_structure_get_int (structure, "height", &overlay->height);
 
-  return gst_pad_try_set_caps (overlay->srcpad, caps);
+  return ret;
 }
 
+static GstCaps *
+gst_textoverlay_getcaps (GstPad * pad)
+{
+  GstPad *otherpad;
+  GstTextOverlay *overlay;
+  GstCaps *caps, *rcaps;
+  const GstCaps *tcaps;
+
+  overlay = GST_TEXTOVERLAY (gst_pad_get_parent (pad));
+  otherpad = (pad == overlay->video_sinkpad) ?
+      overlay->srcpad : overlay->video_sinkpad;
+
+  caps = gst_pad_get_allowed_caps (otherpad);
+  tcaps = gst_pad_get_pad_template_caps (pad);
+  rcaps = gst_caps_intersect (caps, tcaps);
+  gst_caps_free (caps);
+
+  return rcaps;
+}
+
+static gboolean
+gst_textoverlay_event (GstPad * pad, GstEvent * event)
+{
+  GstTextOverlay *overlay = GST_TEXTOVERLAY (gst_pad_get_parent (pad));
+
+  if (GST_EVENT_TYPE (event) == GST_EVENT_SEEK &&
+      GST_PAD_IS_LINKED (overlay->text_sinkpad)) {
+    gst_event_ref (event);
+    gst_pad_send_event (GST_PAD_PEER (overlay->text_sinkpad), event);
+    if (overlay->next_data) {
+      gst_data_unref (overlay->next_data);
+      overlay->next_data = NULL;
+    }
+    if (overlay->current_data) {
+      gst_data_unref (overlay->current_data);
+      overlay->current_data = NULL;
+    }
+    overlay->need_render = TRUE;
+    overlay->text_eos = FALSE;
+  }
+
+  return gst_pad_send_event (GST_PAD_PEER (overlay->video_sinkpad), event);
+}
 
 static void
 gst_text_overlay_blit_yuv420 (GstTextOverlay * overlay, FT_Bitmap * bitmap,
@@ -236,7 +320,7 @@ gst_text_overlay_blit_yuv420 (GstTextOverlay * overlay, FT_Bitmap * bitmap,
   int rowinc, bit_rowinc, uv_rowinc;
   guchar *p, *bitp, *u_p;
   int video_width = overlay->width, video_height = overlay->height;
-  int bitmap_x0 = x0 < 1 ? -(x0 - 1) : 1;       /* 1 pixel border */
+  int bitmap_x0 = 0;            //x0 < 1 ? -(x0 - 1) : 1;       /* 1 pixel border */
   int bitmap_y0 = y0 < 1 ? -(y0 - 1) : 1;       /* 1 pixel border */
   int bitmap_width = bitmap->width - bitmap_x0;
   int bitmap_height = bitmap->rows - bitmap_y0;
@@ -258,7 +342,7 @@ gst_text_overlay_blit_yuv420 (GstTextOverlay * overlay, FT_Bitmap * bitmap,
   x1 = x0 + bitmap_x0;
   p = pixbuf + video_width * y1 + x1;
   bitp = bitmap->buffer + bitmap->pitch * bitmap_y0 + bitmap_x0;
-  for (y = bitmap_y0; y < bitmap_height; y++) {
+  for (y = bitmap_y0; y < bitmap_y0 + bitmap_height; y++) {
     int n;
 
     for (n = bitmap_width; n > 0; --n) {
@@ -287,7 +371,7 @@ gst_text_overlay_blit_yuv420 (GstTextOverlay * overlay, FT_Bitmap * bitmap,
   skip_y = 0;
   skip_x = 0;
 
-  for (; y < bitmap_height; y++) {
+  for (; y < bitmap_y0 + bitmap_height; y++) {
     int n;
 
     x1 = x0 + bitmap_x0;
@@ -338,9 +422,9 @@ gst_textoverlay_video_chain (GstPad * pad, GstData * _data)
   y0 = overlay->y0;
   switch (overlay->valign) {
     case GST_TEXT_OVERLAY_VALIGN_BOTTOM:
-      y0 += overlay->bitmap.rows;
+      y0 = overlay->height - overlay->bitmap.rows - y0;
       break;
-    case GST_TEXT_OVERLAY_VALIGN_BASELINE:
+    case GST_TEXT_OVERLAY_VALIGN_BASELINE:     /* ? */
       y0 -= (overlay->bitmap.rows - overlay->baseline_y);
       break;
     case GST_TEXT_OVERLAY_VALIGN_TOP:
@@ -351,10 +435,10 @@ gst_textoverlay_video_chain (GstPad * pad, GstData * _data)
     case GST_TEXT_OVERLAY_HALIGN_LEFT:
       break;
     case GST_TEXT_OVERLAY_HALIGN_RIGHT:
-      x0 -= overlay->bitmap.width;
+      x0 = overlay->width - overlay->bitmap.width - x0;
       break;
     case GST_TEXT_OVERLAY_HALIGN_CENTER:
-      x0 -= overlay->bitmap.width / 2;
+      x0 = (overlay->width - overlay->bitmap.width) / 2;
       break;
   }
 
@@ -364,10 +448,20 @@ gst_textoverlay_video_chain (GstPad * pad, GstData * _data)
   gst_pad_push (overlay->srcpad, GST_DATA (buf));
 }
 
-#define PAST_END(buffer, time) \
-  (GST_BUFFER_TIMESTAMP (buffer) != GST_CLOCK_TIME_NONE && \
-   GST_BUFFER_DURATION (buffer) != GST_CLOCK_TIME_NONE && \
-   GST_BUFFER_TIMESTAMP (buffer) + GST_BUFFER_DURATION (buffer) \
+#define GST_DATA_TIMESTAMP(data) \
+  (GST_IS_EVENT (data) ? \
+     GST_EVENT_TIMESTAMP (GST_EVENT (data)) : \
+     GST_BUFFER_TIMESTAMP (GST_BUFFER (data)))
+#define GST_DATA_DURATION(data) \
+  (GST_IS_EVENT (data) ? \
+     ((GST_EVENT_TYPE (data) == GST_EVENT_FILLER) ? gst_event_filler_get_duration (GST_EVENT (data)) : \
+      GST_CLOCK_TIME_NONE) : \
+     GST_BUFFER_DURATION (GST_BUFFER (data)))
+
+#define PAST_END(data, time) \
+  (GST_DATA_TIMESTAMP (data) != GST_CLOCK_TIME_NONE && \
+   GST_DATA_DURATION (data) != GST_CLOCK_TIME_NONE && \
+   GST_DATA_TIMESTAMP (data) + GST_DATA_DURATION (data) \
      < (time))
 
 static void
@@ -381,8 +475,39 @@ gst_textoverlay_loop (GstElement * element)
   g_return_if_fail (GST_IS_TEXTOVERLAY (element));
   overlay = GST_TEXTOVERLAY (element);
 
-  video_frame = GST_BUFFER (gst_pad_pull (overlay->video_sinkpad));
+  do {
+    GST_DEBUG ("Attempting to pull next video frame");
+    video_frame = GST_BUFFER (gst_pad_pull (overlay->video_sinkpad));
+    if (GST_IS_EVENT (video_frame)) {
+      GstEvent *event = GST_EVENT (video_frame);
+      GstEventType type = GST_EVENT_TYPE (event);
+
+      GST_DEBUG ("Received event of type %d", type);
+      if (type == GST_EVENT_INTERRUPT) {
+        gst_event_unref (event);
+        return;
+      }
+      gst_pad_event_default (overlay->video_sinkpad, event);
+      if (type == GST_EVENT_EOS) {
+        if (!overlay->text_eos) {
+          /* EOS text stream */
+          GstData *data = NULL;
+
+          do {
+            if (data)
+              gst_data_unref (data);
+            data = gst_pad_pull (overlay->text_sinkpad);
+          } while (!GST_IS_EVENT (data) ||
+              GST_EVENT_TYPE (data) == GST_EVENT_EOS);
+          gst_data_unref (data);
+        }
+        return;
+      }
+      video_frame = NULL;
+    }
+  } while (!video_frame);
   now = GST_BUFFER_TIMESTAMP (video_frame);
+  GST_DEBUG ("Got video frame, time=%" GST_TIME_FORMAT, GST_TIME_ARGS (now));
 
   /*
    * This state machine has a bug that can't be resolved easily.
@@ -395,49 +520,93 @@ gst_textoverlay_loop (GstElement * element)
    * buffer timestamps and durations correctly.  (I think)
    */
 
-  while (overlay->next_buffer == NULL) {
-    GST_DEBUG ("attempting to pull a buffer");
+  while ((!overlay->current_data ||
+          PAST_END (overlay->current_data, now)) &&
+      overlay->next_data == NULL) {
+    GST_DEBUG ("attempting to pull text data");
 
     /* read all text buffers until we get one "in the future" */
-    if (!GST_PAD_IS_USABLE (overlay->text_sinkpad)) {
+    if (!GST_PAD_IS_USABLE (overlay->text_sinkpad) || overlay->text_eos) {
       break;
     }
-    overlay->next_buffer = GST_BUFFER (gst_pad_pull (overlay->text_sinkpad));
-    if (!overlay->next_buffer)
-      break;
+    do {
+      overlay->next_data = gst_pad_pull (overlay->text_sinkpad);
+      if (GST_IS_EVENT (overlay->next_data) &&
+          GST_EVENT_TYPE (overlay->next_data) != GST_EVENT_FILLER) {
+        GstEvent *event = GST_EVENT (overlay->next_data);
+        GstEventType type = GST_EVENT_TYPE (event);
 
-    if (PAST_END (overlay->next_buffer, now)) {
-      gst_buffer_unref (overlay->next_buffer);
-      overlay->next_buffer = NULL;
+        GST_DEBUG ("Received text data event of type %d", type);
+        overlay->next_data = NULL;
+        gst_event_unref (event);
+        if (type == GST_EVENT_EOS) {
+          overlay->text_eos = TRUE;
+          break;
+        } else if (type == GST_EVENT_INTERRUPT)
+          return;
+      }
+    } while (!overlay->next_data);
+
+    if (!overlay->next_data)
+      break;
+    if (PAST_END (overlay->next_data, now)) {
+      GST_DEBUG ("Received %s is past end (%" GST_TIME_FORMAT " + %"
+          GST_TIME_FORMAT " < %" GST_TIME_FORMAT ")",
+          GST_IS_EVENT (overlay->next_data) ? "event" : "buffer",
+          GST_TIME_ARGS (GST_DATA_TIMESTAMP (overlay->next_data)),
+          GST_TIME_ARGS (GST_DATA_DURATION (overlay->next_data)),
+          GST_TIME_ARGS (now));
+      gst_data_unref (overlay->next_data);
+      overlay->next_data = NULL;
+    } else {
+      GST_DEBUG ("Received new text %s of time %" GST_TIME_FORMAT
+          "and duration %" GST_TIME_FORMAT,
+          GST_IS_EVENT (overlay->next_data) ? "event" : "buffer",
+          GST_TIME_ARGS (GST_DATA_TIMESTAMP (overlay->next_data)),
+          GST_TIME_ARGS (GST_DATA_DURATION (overlay->next_data)));
     }
   }
 
-  if (overlay->next_buffer &&
-      (GST_BUFFER_TIMESTAMP (overlay->next_buffer) <= now ||
-          GST_BUFFER_TIMESTAMP (overlay->next_buffer) == GST_CLOCK_TIME_NONE)) {
-    GST_DEBUG ("using new buffer");
+  if (overlay->next_data &&
+      (GST_DATA_TIMESTAMP (overlay->next_data) <= now ||
+          GST_DATA_TIMESTAMP (overlay->next_data) == GST_CLOCK_TIME_NONE)) {
+    GST_DEBUG ("using new %s",
+        GST_IS_EVENT (overlay->next_data) ? "event" : "buffer");
 
-    if (overlay->current_buffer) {
-      gst_buffer_unref (overlay->current_buffer);
+    if (overlay->current_data) {
+      gst_data_unref (overlay->current_data);
     }
-    overlay->current_buffer = overlay->next_buffer;
-    overlay->next_buffer = NULL;
+    overlay->current_data = overlay->next_data;
+    overlay->next_data = NULL;
 
-    GST_DEBUG ("rendering '%*s'",
-        GST_BUFFER_SIZE (overlay->current_buffer),
-        GST_BUFFER_DATA (overlay->current_buffer));
-    pango_layout_set_markup (overlay->layout,
-        GST_BUFFER_DATA (overlay->current_buffer),
-        GST_BUFFER_SIZE (overlay->current_buffer));
+    if (GST_IS_BUFFER (overlay->current_data)) {
+      guint size = GST_BUFFER_SIZE (overlay->current_data);
+      guint8 *data = GST_BUFFER_DATA (overlay->current_data);
+
+      while (size > 0 &&
+          (data[size - 1] == '\r' ||
+              data[size - 1] == '\n' || data[size - 1] == '\0'))
+        size--;
+
+      GST_DEBUG ("rendering '%*s'", size,
+          GST_BUFFER_DATA (overlay->current_data));
+      /* somehow pango barfs over "\0" buffers... */
+      pango_layout_set_markup (overlay->layout,
+          (gchar *) GST_BUFFER_DATA (overlay->current_data), size);
+    } else {
+      GST_DEBUG ("Filler - no data");
+      pango_layout_set_markup (overlay->layout, "", 0);
+    }
     render_text (overlay);
     overlay->need_render = FALSE;
   }
 
-  if (overlay->current_buffer && PAST_END (overlay->current_buffer, now)) {
-    GST_DEBUG ("dropping old buffer");
+  if (overlay->current_data && PAST_END (overlay->current_data, now)) {
+    GST_DEBUG ("dropping old %s",
+        GST_IS_EVENT (overlay->current_data) ? "event" : "buffer");
 
-    gst_buffer_unref (overlay->current_buffer);
-    overlay->current_buffer = NULL;
+    gst_buffer_unref (overlay->current_data);
+    overlay->current_data = NULL;
 
     overlay->need_render = TRUE;
   }
@@ -468,12 +637,11 @@ gst_textoverlay_change_state (GstElement * element)
     case GST_STATE_PLAYING_TO_PAUSED:
       break;
     case GST_STATE_PAUSED_TO_READY:
+      overlay->text_eos = FALSE;
       break;
   }
 
-  parent_class->change_state (element);
-
-  return GST_STATE_SUCCESS;
+  return parent_class->change_state (element);
 }
 
 static void
@@ -481,6 +649,11 @@ gst_textoverlay_finalize (GObject * object)
 {
   GstTextOverlay *overlay = GST_TEXTOVERLAY (object);
 
+  if (overlay->default_text) {
+    g_free (overlay->default_text);
+    overlay->default_text = NULL;
+  }
+
   if (overlay->layout) {
     g_object_unref (overlay->layout);
     overlay->layout = NULL;
@@ -500,22 +673,30 @@ gst_textoverlay_init (GstTextOverlay * overlay)
   overlay->video_sinkpad =
       gst_pad_new_from_template (gst_static_pad_template_get
       (&video_sink_template_factory), "video_sink");
-/*     gst_pad_set_chain_function(overlay->video_sinkpad, gst_textoverlay_video_chain); */
-  gst_pad_set_link_function (overlay->video_sinkpad,
-      gst_textoverlay_video_sinkconnect);
+  gst_pad_set_link_function (overlay->video_sinkpad, gst_textoverlay_link);
+  gst_pad_set_getcaps_function (overlay->video_sinkpad,
+      gst_textoverlay_getcaps);
+  gst_pad_set_internal_link_function (overlay->video_sinkpad,
+      gst_textoverlay_linkedpads);
   gst_element_add_pad (GST_ELEMENT (overlay), overlay->video_sinkpad);
 
   /* text sink */
   overlay->text_sinkpad =
       gst_pad_new_from_template (gst_static_pad_template_get
       (&text_sink_template_factory), "text_sink");
-/*     gst_pad_set_link_function(overlay->text_sinkpad, gst_textoverlay_text_sinkconnect); */
+  gst_pad_set_internal_link_function (overlay->text_sinkpad,
+      gst_textoverlay_linkedpads);
   gst_element_add_pad (GST_ELEMENT (overlay), overlay->text_sinkpad);
 
   /* (video) source */
   overlay->srcpad =
       gst_pad_new_from_template (gst_static_pad_template_get
       (&textoverlay_src_template_factory), "src");
+  gst_pad_set_link_function (overlay->srcpad, gst_textoverlay_link);
+  gst_pad_set_getcaps_function (overlay->srcpad, gst_textoverlay_getcaps);
+  gst_pad_set_internal_link_function (overlay->srcpad,
+      gst_textoverlay_linkedpads);
+  gst_pad_set_event_function (overlay->srcpad, gst_textoverlay_event);
   gst_element_add_pad (GST_ELEMENT (overlay), overlay->srcpad);
 
   overlay->layout =
@@ -524,12 +705,15 @@ gst_textoverlay_init (GstTextOverlay * overlay)
 
   overlay->halign = GST_TEXT_OVERLAY_HALIGN_CENTER;
   overlay->valign = GST_TEXT_OVERLAY_VALIGN_BASELINE;
-  overlay->x0 = overlay->y0 = 0;
+  overlay->x0 = overlay->y0 = 25;
 
   overlay->default_text = g_strdup ("");
   overlay->need_render = TRUE;
+  overlay->text_eos = FALSE;
 
   gst_element_set_loop_function (GST_ELEMENT (overlay), gst_textoverlay_loop);
+
+  GST_FLAG_SET (overlay, GST_ELEMENT_EVENT_AWARE);
 }
 
 
@@ -591,12 +775,12 @@ gst_textoverlay_set_property (GObject * object, guint prop_id,
 
       desc = pango_font_description_from_string (g_value_get_string (value));
       if (desc) {
-        g_message ("font description set: %s", g_value_get_string (value));
+        GST_LOG ("font description set: %s", g_value_get_string (value));
         pango_layout_set_font_description (overlay->layout, desc);
         pango_font_description_free (desc);
         render_text (overlay);
       } else
-        g_warning ("font description parse failed: %s",
+        GST_WARNING ("font description parse failed: %s",
             g_value_get_string (value));
       break;
     }
@@ -626,16 +810,21 @@ gst_textoverlay_get_property (GObject * object, guint prop_id, GValue * value,
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
-  if (!gst_element_register (plugin, "textoverlay", GST_RANK_PRIMARY,
-          GST_TYPE_TEXTOVERLAY))
+  if (!gst_element_register (plugin, "textoverlay", GST_RANK_NONE,
+          GST_TYPE_TEXTOVERLAY) ||
+      !gst_element_register (plugin, "textrender", GST_RANK_NONE,
+          GST_TYPE_TEXT_RENDER))
     return FALSE;
 
   /*texttestsrc_plugin_init(module, plugin); */
   /*subparse_plugin_init(module, plugin); */
+
+  GST_DEBUG_CATEGORY_INIT (pango_debug, "pango", 0, "Pango elements");
+
   return TRUE;
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     "textoverlay",
-    "Text overlay", plugin_init, VERSION, "GPL", GST_PACKAGE, GST_ORIGIN)
+    "Text overlay", plugin_init, VERSION, "LGPL", GST_PACKAGE, GST_ORIGIN)
