@@ -47,20 +47,7 @@ gst_polypsink_base_init (gpointer g_class)
           "signed = (boolean) TRUE, "
           "width = (int) 16, "
           "depth = (int) 16, "
-          "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, 16 ]"
-#if 0
-          ";audio/x-raw-float, "
-          "endianness = (int) { LITTLE_ENDIAN, BIG_ENDIAN }, "
-          "width = (int) 32, "
-          "rate = (int) [ 1, MAX ], "
-          "channels = (int) [ 1, 16 ];"
-          "audio/x-raw-int, "
-          "signed = (boolean) FALSE, "
-          "width = (int) 8, "
-          "depth = (int) 8, "
-          "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, 16 ]"
-#endif
-      )
+          "rate = (int) [ 1, MAX ], " "channels = (int) [ 1, 16 ]")
       );
 
   static const GstElementDetails details = {
