@@ -713,16 +713,22 @@ gst_vorbisenc_setup (VorbisEnc * vorbisenc)
   return TRUE;
 }
 
-/* prepare a buffer for transmission by passing data through libvorbis */
+/* convert an ogg packet to a GstBuffer */
 static GstBuffer *
-gst_vorbisenc_buffer_from_packet (VorbisEnc * vorbisenc, ogg_packet * packet)
+gst_vorbisenc_buffer_from_packet (VorbisEnc * vorbisenc, ogg_packet * packet,
+    gboolean isheader)
 {
   GstBuffer *outbuf;
   guint64 granulepos_delta, timestamp_delta;
 
-  /* header packets always need to have granulepos 0, 
+  GST_DEBUG_OBJECT (vorbisenc,
+      "converting ogg packet with packetno %" G_GINT64_FORMAT
+      " and granulepos %" G_GINT64_FORMAT " to GstBuffer",
+      packet->packetno, packet->granulepos);
+  /* header packets always need to have granulepos 0,
    * regardless of the initial delay */
-  if (packet->granulepos == 0) {
+  if (isheader) {
+    GST_DEBUG_OBJECT (vorbisenc, "header packet, not adjusting granulepos");
     granulepos_delta = 0;
     timestamp_delta = 0;
   } else {
@@ -734,13 +740,19 @@ gst_vorbisenc_buffer_from_packet (VorbisEnc * vorbisenc, ogg_packet * packet)
   outbuf = gst_buffer_new_and_alloc (packet->bytes);
   memcpy (GST_BUFFER_DATA (outbuf), packet->packet, packet->bytes);
   GST_BUFFER_OFFSET (outbuf) = vorbisenc->bytes_out;
+
+  /* hack: for all codecs used by Ogg, we set the granulepos as
+   * OFFSET_END so the muxer can mux based on granulepos, as required */
   GST_BUFFER_OFFSET_END (outbuf) = packet->granulepos + granulepos_delta;
   GST_BUFFER_TIMESTAMP (outbuf) =
       vorbis_granule_time_copy (&vorbisenc->vd,
       packet->granulepos) * GST_SECOND + timestamp_delta;
 
-  GST_DEBUG ("encoded buffer of %d bytes. granulepos = %" G_GINT64_FORMAT
-      " + %" G_GINT64_FORMAT " = %" G_GINT64_FORMAT, GST_BUFFER_SIZE (outbuf),
+  GST_DEBUG ("encoded buffer of %d bytes. timestamp = %" GST_TIME_FORMAT
+      " granulepos = %" G_GINT64_FORMAT
+      " + delta %" G_GINT64_FORMAT " = %" G_GINT64_FORMAT,
+      GST_BUFFER_SIZE (outbuf),
+      GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)),
       packet->granulepos, granulepos_delta,
       packet->granulepos + granulepos_delta);
 
@@ -760,12 +772,13 @@ gst_vorbisenc_push_buffer (VorbisEnc * vorbisenc, GstBuffer * buffer)
   }
 }
 
+/* push a regular non-header ogg packet out as a GstBuffer */
 static void
 gst_vorbisenc_push_packet (VorbisEnc * vorbisenc, ogg_packet * packet)
 {
   GstBuffer *outbuf;
 
-  outbuf = gst_vorbisenc_buffer_from_packet (vorbisenc, packet);
+  outbuf = gst_vorbisenc_buffer_from_packet (vorbisenc, packet, FALSE);
   gst_vorbisenc_push_buffer (vorbisenc, outbuf);
 }
 
@@ -836,7 +849,7 @@ gst_vorbisenc_chain (GstPad * pad, GstData * _data)
         return;
       case GST_EVENT_DISCONTINUOUS:
       {
-        guint64 val;
+        gint64 val;
 
         if (gst_event_discont_get_value (event, GST_FORMAT_TIME, &val)) {
           /* vorbis does not support discontinuities in the middle of
@@ -889,9 +902,9 @@ gst_vorbisenc_chain (GstPad * pad, GstData * _data)
           &header_comm, &header_code);
 
       /* create header buffers */
-      buf1 = gst_vorbisenc_buffer_from_packet (vorbisenc, &header);
-      buf2 = gst_vorbisenc_buffer_from_packet (vorbisenc, &header_comm);
-      buf3 = gst_vorbisenc_buffer_from_packet (vorbisenc, &header_code);
+      buf1 = gst_vorbisenc_buffer_from_packet (vorbisenc, &header, TRUE);
+      buf2 = gst_vorbisenc_buffer_from_packet (vorbisenc, &header_comm, TRUE);
+      buf3 = gst_vorbisenc_buffer_from_packet (vorbisenc, &header_code, TRUE);
 
       /* mark and put on caps */
       caps = gst_pad_get_caps (vorbisenc->srcpad);
