@@ -149,46 +149,46 @@ deep_notify_callback (GObject * object, GstObject * origin,
 
 typedef struct
 {
-  guint meta;
-  guint encoded;
-} TagFlagScore;
+  GstTagList *streaminfo;
+  GstTagList *metadata;
+}
+TagListDest;
 
 static void
-tag_flag_score (const GstTagList * list, const gchar * tag, gpointer user_data)
+tag_list_dest (const GstTagList * list, const gchar * tag, gpointer user_data)
 {
-  TagFlagScore *score = (TagFlagScore *) user_data;
+  TagListDest *dest = (TagListDest *) user_data;
   GstTagFlag flag;
 
   flag = gst_tag_get_flag (tag);
-  if (flag == GST_TAG_FLAG_META)
-    score->meta++;
   if (flag == GST_TAG_FLAG_ENCODED)
-    score->encoded++;
+    gst_tag_list_remove_tag (dest->metadata, tag);
+  else if (flag == GST_TAG_FLAG_META)
+    gst_tag_list_remove_tag (dest->streaminfo, tag);
+  else
+    /* default to streaminfo ? */
+    gst_tag_list_remove_tag (dest->metadata, tag);
 }
 
 void
 found_tag_callback (GObject * pipeline, GstElement * source, GstTagList * tags,
     GstMediaInfoPriv * priv)
 {
-  TagFlagScore score;
+  TagListDest dest;
 
-  score.meta = 0;
-  score.encoded = 0;
   GST_DEBUG ("element %s found tag", GST_STR_NULL (GST_ELEMENT_NAME (source)));
 
-  /* decide if it's likely to be metadata or streaminfo */
-  /* FIXME: this is a hack, there must be a better way,
-     but as long as elements can report both mixed we need to do this */
+  dest.streaminfo = gst_tag_list_copy (tags);
+  dest.metadata = gst_tag_list_copy (tags);
 
-  gst_tag_list_foreach (tags, tag_flag_score, &score);
+  gst_tag_list_foreach (tags, tag_list_dest, &dest);
 
-  if (score.meta > score.encoded) {
-    GST_DEBUG ("found tags from decoder, adding them as metadata");
-    priv->metadata = gst_tag_list_copy (tags);
-  } else {
-    GST_DEBUG ("found tags, adding them as streaminfo");
-    priv->streaminfo = gst_tag_list_copy (tags);
-  }
+  priv->metadata =
+      gst_tag_list_merge (priv->metadata, dest.metadata, GST_TAG_MERGE_REPLACE);
+
+  priv->streaminfo =
+      gst_tag_list_merge (priv->streaminfo, dest.streaminfo,
+      GST_TAG_MERGE_REPLACE);
 }
 
 void
@@ -372,7 +372,7 @@ gmi_set_mime (GstMediaInfo * info, const char *mime)
       || (strcmp (mime, "audio/x-wav") == 0))
     desc =
         g_strdup_printf
-        ("%s ! wavparse name=decoder ! audio/x-raw-int ! fakesink name=sink",
+        ("%s name=source ! wavparse name=decoder ! audio/x-raw-int ! fakesink name=sink",
         priv->source_name);
   else if (strcmp (mime, "audio/x-mod") == 0
       || strcmp (mime, "audio/x-s3m") == 0 || strcmp (mime, "audio/x-xm") == 0
@@ -394,6 +394,9 @@ gmi_set_mime (GstMediaInfo * info, const char *mime)
   }
   /* get a bunch of elements from the bin */
   priv->source = gst_bin_get_by_name (GST_BIN (priv->pipeline), "source");
+  if (!GST_IS_ELEMENT (priv->source))
+    g_error ("Could not create source element '%s'", priv->source_name);
+
   g_assert (GST_IS_ELEMENT (priv->source));
   g_object_set (G_OBJECT (priv->source), "location", priv->location, NULL);
   priv->decoder = gst_bin_get_by_name (GST_BIN (priv->pipeline), "decoder");
