@@ -63,7 +63,8 @@ enum
 enum
 {
   ARG_0,
-  ARG_NUM_PADS
+  ARG_NUM_PADS,
+  ARG_EOS
       /* FILL ME */
 };
 
@@ -89,13 +90,22 @@ static void gst_adder_init (GstAdder * adder);
 static void gst_adder_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
+static void gst_adder_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec);
+
 static GstPad *gst_adder_request_new_pad (GstElement * element,
     GstPadTemplate * temp, const gchar * unused);
+static void gst_adder_release_pad (GstElement * element, GstPad * pad);
 static GstElementStateReturn gst_adder_change_state (GstElement * element);
 
 /* we do need a loop function */
 static void gst_adder_loop (GstElement * element);
 
+static GstCaps *gst_adder_get_caps (GstPad * pad);
+
+static gboolean gst_adder_query (GstPad * pad,
+    GstQueryType type, GstFormat * format, gint64 * value);
+
 static GstElementClass *parent_class = NULL;
 
 /* static guint gst_adder_signals[LAST_SIGNAL] = { 0 }; */
@@ -130,6 +140,7 @@ gst_adder_link (GstPad * pad, const GstCaps * caps)
   GstStructure *structure;
   GstPadLinkReturn ret;
   GstElement *element;
+  const GSList *links;
 
   g_return_val_if_fail (caps != NULL, GST_PAD_LINK_REFUSED);
   g_return_val_if_fail (pad != NULL, GST_PAD_LINK_REFUSED);
@@ -143,21 +154,8 @@ gst_adder_link (GstPad * pad, const GstCaps * caps)
 
     if (otherpad != pad) {
       ret = gst_pad_try_set_caps (otherpad, caps);
-      if (GST_PAD_LINK_FAILED (ret)) {
-        return ret;
-      }
-    }
-    pads = g_list_next (pads);
-  }
-
-
-  pads = gst_element_get_pad_list (GST_ELEMENT (adder));
-  while (pads) {
-    GstPad *otherpad = GST_PAD (pads->data);
-
-    if (otherpad != pad) {
-      ret = gst_pad_try_set_caps (otherpad, caps);
-      if (GST_PAD_LINK_FAILED (ret)) {
+      if (ret == GST_PAD_LINK_REFUSED ||
+          (otherpad == adder->srcpad && GST_PAD_LINK_FAILED (ret))) {
         return ret;
       }
     }
@@ -183,6 +181,16 @@ gst_adder_link (GstPad * pad, const GstCaps * caps)
     gst_structure_get_int (structure, "rate", &adder->rate);
   }
 
+  for (links = adder->input_channels;
+      links != NULL; links = g_slist_next (links)) {
+    GstAdderInputChannel *input = (GstAdderInputChannel *) links->data;
+
+    if (input->sinkpad == pad) {
+      input->eos = FALSE;
+      break;
+    }
+  }
+
   return GST_PAD_LINK_OK;
 }
 
@@ -207,9 +215,16 @@ gst_adder_class_init (GstAdderClass * klass)
       g_param_spec_int ("num_pads", "number of pads", "Number Of Pads",
           0, G_MAXINT, 0, G_PARAM_READABLE));
 
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_EOS,
+      g_param_spec_boolean ("eos", "generate EOS",
+          "generate an EOS when all input channels did", FALSE,
+          G_PARAM_READWRITE));
+
   gobject_class->get_property = gst_adder_get_property;
+  gobject_class->set_property = gst_adder_set_property;
 
   gstelement_class->request_new_pad = gst_adder_request_new_pad;
+  gstelement_class->release_pad = gst_adder_release_pad;
   gstelement_class->change_state = gst_adder_change_state;
 }
 
@@ -219,10 +234,14 @@ gst_adder_init (GstAdder * adder)
   adder->srcpad =
       gst_pad_new_from_template (gst_static_pad_template_get
       (&gst_adder_src_template), "src");
+  gst_element_set_loop_function (GST_ELEMENT (adder),
+      GST_DEBUG_FUNCPTR (gst_adder_loop));
+  gst_pad_set_getcaps_function (adder->srcpad,
+      GST_DEBUG_FUNCPTR (gst_adder_get_caps));
+  gst_pad_set_link_function (adder->srcpad, GST_DEBUG_FUNCPTR (gst_adder_link));
+  gst_pad_set_query_function (adder->srcpad,
+      GST_DEBUG_FUNCPTR (gst_adder_query));
   gst_element_add_pad (GST_ELEMENT (adder), adder->srcpad);
-  gst_element_set_loop_function (GST_ELEMENT (adder), gst_adder_loop);
-  gst_pad_set_getcaps_function (adder->srcpad, gst_pad_proxy_getcaps);
-  gst_pad_set_link_function (adder->srcpad, gst_adder_link);
 
   adder->format = GST_ADDER_FORMAT_UNSET;
 
@@ -230,6 +249,8 @@ gst_adder_init (GstAdder * adder)
 
   adder->numsinkpads = 0;
   adder->input_channels = NULL;
+
+  adder->eos = FALSE;
 }
 
 static GstPad *
@@ -262,19 +283,66 @@ gst_adder_request_new_pad (GstElement * element, GstPadTemplate * templ,
   name = g_strdup_printf ("sink%d", adder->numsinkpads);
   input->sinkpad = gst_pad_new_from_template (templ, name);
   input->bytestream = gst_bytestream_new (input->sinkpad);
+  GST_DEBUG_OBJECT (adder, "created new bytestream %p", input->bytestream);
+  input->eos = FALSE;
 
   gst_element_add_pad (GST_ELEMENT (adder), input->sinkpad);
-  gst_pad_set_getcaps_function (input->sinkpad, gst_pad_proxy_getcaps);
+  gst_pad_set_getcaps_function (input->sinkpad, gst_adder_get_caps);
   gst_pad_set_link_function (input->sinkpad, gst_adder_link);
 
   /* add the input_channel to the list of input channels */
 
+  GST_DEBUG_OBJECT (adder, "adding input channel %p with sinkpad %s:%s",
+      input, GST_DEBUG_PAD_NAME (input->sinkpad));
   adder->input_channels = g_slist_append (adder->input_channels, input);
   adder->numsinkpads++;
 
+  g_free (name);
   return input->sinkpad;
 }
 
+static void
+gst_adder_release_pad (GstElement * element, GstPad * pad)
+{
+  GstAdder *adder;
+  gboolean found = FALSE;
+  GSList *inputs;
+
+  if (!GST_IS_ADDER (element))
+    return;
+
+  adder = GST_ADDER (element);
+
+  GST_DEBUG_OBJECT (adder, "asked to release pad %s:%s",
+      GST_DEBUG_PAD_NAME (pad));
+
+  inputs = adder->input_channels;
+
+  while (inputs) {
+    GstAdderInputChannel *input;
+
+    input = (GstAdderInputChannel *) inputs->data;
+    GST_DEBUG_OBJECT (adder, "looking at input channel %p with sinkpad %s:%s",
+        input, GST_DEBUG_PAD_NAME (input->sinkpad));
+    if (input->sinkpad == pad) {
+      GST_DEBUG_OBJECT (adder, "found pad to release: %s:%s",
+          GST_DEBUG_PAD_NAME (pad));
+      GST_DEBUG_OBJECT (adder, "destroying bytestream %p", input->bytestream);
+      gst_bytestream_destroy (input->bytestream);
+      adder->input_channels = g_slist_remove (adder->input_channels, input);
+      g_free (input);
+      gst_element_remove_pad (element, pad);
+      found = TRUE;
+      break;
+    }
+    inputs = inputs->next;
+  }
+  if (!found) {
+    g_warning ("the pad %s:%s given as an argument is not adder's !",
+        GST_DEBUG_PAD_NAME (pad));
+  }
+}
+
 static void
 gst_adder_get_property (GObject * object, guint prop_id, GValue * value,
     GParamSpec * pspec)
@@ -288,7 +356,32 @@ gst_adder_get_property (GObject * object, guint prop_id, GValue * value,
 
   switch (prop_id) {
     case ARG_NUM_PADS:
-      g_value_set_int (value, adder->numsinkpads);
+      /* FIXME: rename adder->numsinkpads to adder->nextsinkpadname or something */
+      g_value_set_int (value, g_slist_length (adder->input_channels));
+      break;
+    case ARG_EOS:
+      g_value_set_boolean (value, adder->eos);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
+static void
+gst_adder_set_property (GObject * object, guint prop_id, const GValue * value,
+    GParamSpec * pspec)
+{
+  GstAdder *adder;
+
+  /* it's not null if we got it, but it might not be ours */
+  g_return_if_fail (GST_IS_ADDER (object));
+
+  adder = GST_ADDER (object);
+
+  switch (prop_id) {
+    case ARG_EOS:
+      adder->eos = g_value_get_boolean (value);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -312,6 +405,7 @@ gst_adder_loop (GstElement * element)
    */
   GstAdder *adder;
   GstBuffer *buf_out;
+  gint valid_inputs;
 
   GSList *inputs;
 
@@ -321,6 +415,7 @@ gst_adder_loop (GstElement * element)
   g_return_if_fail (GST_IS_ADDER (element));
 
   adder = GST_ADDER (element);
+  valid_inputs = adder->numsinkpads;
 
   /* get new output buffer */
   /* FIXME the 1024 is arbitrary */
@@ -352,6 +447,7 @@ gst_adder_loop (GstElement * element)
     GST_LOG_OBJECT (adder, "  looking into channel %p", input);
 
     if (!GST_PAD_IS_USABLE (input->sinkpad)) {
+      valid_inputs--;
       GST_LOG_OBJECT (adder, "    adder ignoring pad %s:%s",
           GST_DEBUG_PAD_NAME (input->sinkpad));
       continue;
@@ -376,22 +472,37 @@ gst_adder_loop (GstElement * element)
           case GST_EVENT_EOS:
             /* if we get an EOS event from one of our sink pads, we assume that
                pad's finished handling data. just skip this pad. */
-            GST_DEBUG ("    got an EOS event");
+            GST_DEBUG ("    got an EOSish event");
             gst_event_unref (event);
+            input->eos = TRUE;
+            valid_inputs--;
             continue;
           case GST_EVENT_INTERRUPT:
-            gst_event_unref (event);
             GST_DEBUG ("    got an interrupt event");
             /* we have to call interrupt here, the scheduler will switch out
                this element ASAP or returns TRUE if we need to exit the loop */
             if (gst_element_interrupt (GST_ELEMENT (adder))) {
               gst_buffer_unref (buf_out);
+              gst_event_unref (event);
               return;
             }
+            break;
+          case GST_EVENT_DISCONTINUOUS:
+            GST_DEBUG ("    got a discont event");
+            input->eos = FALSE;
+            break;
           default:
-            GST_LOG_OBJECT (adder, "pulling again after event");
-            goto repeat;
+            if (input->eos) {
+              valid_inputs--;
+              GST_LOG_OBJECT (adder, "    adder ignoring pad (EOS) %s:%s",
+                  GST_DEBUG_PAD_NAME (input->sinkpad));
+              continue;
+            }
+            break;
         }
+        GST_LOG_OBJECT (adder, "pulling again after event");
+        gst_event_unref (event);
+        goto repeat;
       }
     } else {
       /* here's where the data gets copied. */
@@ -470,18 +581,54 @@ gst_adder_loop (GstElement * element)
     return;
   }
 
-  GST_BUFFER_TIMESTAMP (buf_out) = adder->timestamp;
-  if (adder->format == GST_ADDER_FORMAT_FLOAT)
-    adder->offset += GST_BUFFER_SIZE (buf_out) / adder->width / adder->channels;
-  else
-    adder->offset +=
-        GST_BUFFER_SIZE (buf_out) * 8 / adder->width / adder->channels;
-  adder->timestamp = adder->offset * GST_SECOND / adder->rate;
+  if (valid_inputs > 0 || adder->eos == FALSE) {
+    gint64 delta;
+    gint64 duration;
 
-  /* send it out */
+    delta = GST_BUFFER_SIZE (buf_out) * 8 / (adder->width * adder->channels);
+    duration = delta * GST_SECOND / adder->rate;
 
-  GST_LOG ("pushing buf_out");
-  gst_pad_push (adder->srcpad, GST_DATA (buf_out));
+    GST_BUFFER_TIMESTAMP (buf_out) = adder->timestamp;
+    GST_BUFFER_DURATION (buf_out) = duration;
+
+    adder->offset += delta;
+    adder->timestamp += duration;
+
+    /* send it out */
+    GST_LOG ("pushing buf_out");
+    gst_pad_push (adder->srcpad, GST_DATA (buf_out));
+  } else {
+    /* generate an EOS event */
+    gst_buffer_unref (buf_out);
+    GST_LOG ("forwarding EOS");
+    gst_pad_push (adder->srcpad, GST_DATA (gst_event_new (GST_EVENT_EOS)));
+    gst_element_set_eos (element);
+  }
+}
+
+
+static gboolean
+gst_adder_query (GstPad * pad, GstQueryType type, GstFormat * format,
+    gint64 * value)
+{
+  GstAdder *adder = GST_ADDER (gst_pad_get_parent (pad));
+
+  if (type != GST_QUERY_POSITION) {
+    return gst_pad_query_default (pad, type, format, value);
+  }
+
+  switch (*format) {
+    case GST_FORMAT_TIME:
+      *value = adder->timestamp;
+      break;
+    case GST_FORMAT_DEFAULT:
+      *value = adder->offset;
+      break;
+    default:
+      return gst_pad_query_default (pad, type, format, value);
+  }
+
+  return TRUE;
 }
 
 
@@ -513,6 +660,28 @@ gst_adder_change_state (GstElement * element)
   return GST_STATE_SUCCESS;
 }
 
+static GstCaps *
+gst_adder_get_caps (GstPad * pad)
+{
+  GstAdder *adder = GST_ADDER (gst_pad_get_parent (pad));
+  GstCaps *caps = gst_caps_new_any ();
+  GstCaps *tmp, *res;
+  GstPad *otherpad;
+  const GList *pads;
+
+  for (pads = gst_element_get_pad_list (GST_ELEMENT (adder));
+      pads != NULL; pads = pads->next) {
+    otherpad = GST_PAD (pads->data);
+
+    tmp = gst_pad_get_allowed_caps (otherpad);
+    res = gst_caps_intersect (caps, tmp);
+    gst_caps_free (tmp);
+    gst_caps_free (caps);
+    caps = res;
+  }
+
+  return caps;
+}
 
 static gboolean
 plugin_init (GstPlugin * plugin)
