@@ -55,13 +55,21 @@ struct _Audioscale {
 
   GstPad *sinkpad,*srcpad;
 
+  GstClockTime sample_duration;
+
   /* audio state */
   gboolean passthru;
-  gint64 offset;
+  gint64 gst_resample_offset;
 
-  resample_t *resample;
+  gint64* offsets;
+  gboolean increase; /* is the rate change an increase */
+  gint num_iterations; /* number of iterations through gst_audioscale/(increase|decrease)_rate */
+  
+  gst_resample_t gst_resample_template;
+  gst_resample_t* gst_resample;
+  GstBuffer* outbuf;
 
-  GstBuffer *outbuf;
+  GstEvent* saved_discont; /* if we get one before negotiation completed, we store it here */
 };
 
 struct _AudioscaleClass {
