@@ -1,9 +1,9 @@
 /* GStreamer
- * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
+ * Copyright (C) 1999 Erik Walthinsen <omega@cse.ogi.edu>
+ * Copyright (C) 2000,2001,2002,2003,2004,2005
+ *           Thomas Vander Stichele <thomas at apestaart dot org>
  *
  * gstlevel.c: signals RMS, peak and decaying peak levels
- * Copyright (C) 2000,2001,2002,2003
- *           Thomas Vander Stichele <thomas at apestaart dot org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -29,6 +29,9 @@
 #include "gstlevel.h"
 #include "math.h"
 
+GST_DEBUG_CATEGORY (level_debug);
+#define GST_CAT_DEFAULT level_debug
+
 /* elementfactory information */
 static GstElementDetails level_details = {
   "Level",
@@ -40,17 +43,27 @@ static GstElementDetails level_details = {
 /* pad templates */
 
 static GstStaticPadTemplate sink_template_factory =
-GST_STATIC_PAD_TEMPLATE ("level_sink",
+GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_AUDIO_INT_PAD_TEMPLATE_CAPS)
+    GST_STATIC_CAPS ("audio/x-raw-int, "
+        "rate = (int) [ 1, MAX ], "
+        "channels = (int) [ 1, 2 ], "
+        "endianness = (int) BYTE_ORDER, "
+        "width = (int) { 8, 16 }, "
+        "depth = (int) { 8, 16 }, " "signed = (boolean) true")
     );
 
 static GstStaticPadTemplate src_template_factory =
-GST_STATIC_PAD_TEMPLATE ("level_src",
+GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS (GST_AUDIO_INT_PAD_TEMPLATE_CAPS)
+    GST_STATIC_CAPS ("audio/x-raw-int, "
+        "rate = (int) [ 1, MAX ], "
+        "channels = (int) [ 1, 2 ], "
+        "endianness = (int) BYTE_ORDER, "
+        "width = (int) { 8, 16 }, "
+        "depth = (int) { 8, 16 }, " "signed = (boolean) true")
     );
 
 /* Filter signals and args */
@@ -74,7 +87,6 @@ static void gst_level_class_init (GstLevelClass * klass);
 static void gst_level_base_init (GstLevelClass * klass);
 static void gst_level_init (GstLevel * filter);
 
-static GstElementStateReturn gst_level_change_state (GstElement * element);
 static void gst_level_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_level_get_property (GObject * object, guint prop_id,
@@ -98,6 +110,7 @@ gst_level_get_type (void)
       sizeof (GstLevel), 0,
       (GInstanceInitFunc) gst_level_init
     };
+
     level_type = g_type_register_static (GST_TYPE_ELEMENT, "GstLevel",
         &level_info, 0);
   }
@@ -136,36 +149,25 @@ gst_level_link (GstPad * pad, const GstCaps * caps)
     return GST_PAD_LINK_REFUSED;
 
   /* allocate channel variable arrays */
-  if (filter->CS)
-    g_free (filter->CS);
-  if (filter->peak)
-    g_free (filter->peak);
-  if (filter->last_peak)
-    g_free (filter->last_peak);
-  if (filter->decay_peak)
-    g_free (filter->decay_peak);
-  if (filter->decay_peak_age)
-    g_free (filter->decay_peak_age);
-  if (filter->MS)
-    g_free (filter->MS);
-  if (filter->RMS_dB)
-    g_free (filter->RMS_dB);
+  g_free (filter->CS);
+  g_free (filter->peak);
+  g_free (filter->last_peak);
+  g_free (filter->decay_peak);
+  g_free (filter->decay_peak_age);
+  g_free (filter->RMS_dB);
   filter->CS = g_new (double, filter->channels);
   filter->peak = g_new (double, filter->channels);
   filter->last_peak = g_new (double, filter->channels);
   filter->decay_peak = g_new (double, filter->channels);
   filter->decay_peak_age = g_new (double, filter->channels);
-  filter->MS = g_new (double, filter->channels);
   filter->RMS_dB = g_new (double, filter->channels);
 
   for (i = 0; i < filter->channels; ++i) {
     filter->CS[i] = filter->peak[i] = filter->last_peak[i] =
         filter->decay_peak[i] = filter->decay_peak_age[i] =
-        filter->MS[i] = filter->RMS_dB[i] = 0.0;
+        filter->RMS_dB[i] = 0.0;
   }
 
-  filter->inited = TRUE;
-
   return GST_PAD_LINK_OK;
 }
 
@@ -173,10 +175,14 @@ static void inline
 gst_level_fast_16bit_chain (gint16 * in, guint num, gint channels,
     gint resolution, double *CS, double *peak)
 #include "filter.func"
+ ;
+
      static void inline
          gst_level_fast_8bit_chain (gint8 * in, guint num, gint channels,
     gint resolution, double *CS, double *peak)
 #include "filter.func"
+ ;
+
      static void gst_level_chain (GstPad * pad, GstData * _data)
 {
   GstBuffer *buf = GST_BUFFER (_data);
@@ -184,7 +190,7 @@ gst_level_fast_16bit_chain (gint16 * in, guint num, gint channels,
   gint16 *in_data;
 
   double CS = 0.0;
-  gint num_samples = 0;
+  gint num_int_samples = 0;     /* total number of interleaved samples */
   gint i;
 
   g_return_if_fail (pad != NULL);
@@ -195,48 +201,58 @@ gst_level_fast_16bit_chain (gint16 * in, guint num, gint channels,
   g_return_if_fail (filter != NULL);
   g_return_if_fail (GST_IS_LEVEL (filter));
 
+  if (!gst_pad_is_negotiated (pad)) {
+    GST_ELEMENT_ERROR (filter, CORE, NEGOTIATION, (NULL),
+        ("sinkpad not negotiated"));
+    gst_data_unref (_data);
+    return;
+  }
+
   for (i = 0; i < filter->channels; ++i)
-    filter->CS[i] = filter->peak[i] = filter->MS[i] = filter->RMS_dB[i] = 0.0;
+    filter->peak[i] = filter->RMS_dB[i] = 0.0;
 
   in_data = (gint16 *) GST_BUFFER_DATA (buf);
 
-  num_samples = GST_BUFFER_SIZE (buf) / (filter->width / 8);
-  if (num_samples % filter->channels != 0)
+  num_int_samples = GST_BUFFER_SIZE (buf) / (filter->width / 8);
+  if (num_int_samples % filter->channels != 0)
     g_warning
         ("WARNING: level: programming error, data not properly interleaved");
+  GST_LOG_OBJECT (filter,
+      "processing incoming buffer of %d bytes and %d samples",
+      GST_BUFFER_SIZE (buf), num_int_samples);
 
   for (i = 0; i < filter->channels; ++i) {
+    CS = 0.0;
     switch (filter->width) {
       case 16:
-        gst_level_fast_16bit_chain (in_data + i, num_samples,
+        gst_level_fast_16bit_chain (in_data + i, num_int_samples,
             filter->channels, filter->width - 1, &CS, &filter->peak[i]);
         break;
       case 8:
-        gst_level_fast_8bit_chain (((gint8 *) in_data) + i, num_samples,
+        gst_level_fast_8bit_chain (((gint8 *) in_data) + i, num_int_samples,
             filter->channels, filter->width - 1, &CS, &filter->peak[i]);
         break;
     }
-    /* g_print ("DEBUG: CS %f, peak %f\n", CS, filter->peak[i]); */
+    GST_LOG_OBJECT (filter,
+        "channel %d, cumulative sum %f, peak %f, over %d samples/%d channels",
+        i, CS, filter->peak[i], num_int_samples, filter->channels);
     filter->CS[i] += CS;
 
   }
-  gst_pad_push (filter->srcpad, GST_DATA (buf));
 
-  filter->num_samples += num_samples;
+  filter->num_samples += num_int_samples / filter->channels;
 
   for (i = 0; i < filter->channels; ++i) {
-    filter->decay_peak_age[i] += num_samples;
-    /* g_print ("filter peak info [%d]: peak %f, age %f\n", i, 
-       filter->last_peak[i], filter->decay_peak_age[i]); */
+    filter->decay_peak_age[i] += num_int_samples / filter->channels;
     /* update running peak */
     if (filter->peak[i] > filter->last_peak[i])
       filter->last_peak[i] = filter->peak[i];
 
     /* update decay peak */
     if (filter->peak[i] >= filter->decay_peak[i]) {
-      /* g_print ("new peak, %f\n", filter->peak[i]); */
       filter->decay_peak[i] = filter->peak[i];
       filter->decay_peak_age[i] = 0;
+      GST_LOG_OBJECT (filter, "channel %d peak refreshed, not decaying", i);
     } else {
       /* make decay peak fall off if too old */
       if (filter->decay_peak_age[i] > filter->rate * filter->decay_peak_ttl) {
@@ -245,15 +261,19 @@ gst_level_fast_16bit_chain (gint16 * in, guint num, gint channels,
         double length;          /* length of buffer in seconds */
 
 
-        length = (double) num_samples / (filter->channels * filter->rate);
+        length = (double) num_int_samples / (filter->channels * filter->rate);
         falloff_dB = filter->decay_peak_falloff * length;
         falloff = pow (10, falloff_dB / -20.0);
 
-        /* g_print ("falloff: length %f, dB falloff %f, falloff factor %e\n",
-           length, falloff_dB, falloff); */
+        GST_LOG_OBJECT (filter,
+            "falloff: length %f, dB falloff %f, falloff factor %e",
+            length, falloff_dB, falloff);
         filter->decay_peak[i] *= falloff;
-        /* g_print ("peak is %f samples old, decayed with factor %e to %f\n",
-           filter->decay_peak_age[i], falloff, filter->decay_peak[i]); */
+        GST_LOG_OBJECT (filter,
+            "peak is %f samples old, decayed with factor %e to %f",
+            filter->decay_peak_age[i], falloff, filter->decay_peak[i]);
+      } else {
+        GST_LOG_OBJECT (filter, "peak not old enough, not decaying");
       }
     }
   }
@@ -262,19 +282,31 @@ gst_level_fast_16bit_chain (gint16 * in, guint num, gint channels,
 
   if (filter->num_samples >= filter->interval * (gdouble) filter->rate) {
     if (filter->signal) {
-      gdouble RMS, peak, endtime;
+      gdouble RMS, endtime;
+      gdouble RMSdB, lastdB, decaydB;
+
+      GST_LOG_OBJECT (filter, "emitting, timestamp %" GST_TIME_FORMAT,
+          GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)));
+      endtime = (double) GST_BUFFER_TIMESTAMP (buf) / GST_SECOND
+          + (double) num_int_samples / (filter->rate * filter->channels);
 
       for (i = 0; i < filter->channels; ++i) {
-        RMS = sqrt (filter->CS[i] / (filter->num_samples / filter->channels));
-        peak = filter->last_peak[i];
-        num_samples = GST_BUFFER_SIZE (buf) / (filter->width / 8);
-        endtime = (double) GST_BUFFER_TIMESTAMP (buf) / GST_SECOND
-            + (double) num_samples / (double) filter->rate;
+        RMS = sqrt (filter->CS[i] / filter->num_samples);
+        GST_LOG_OBJECT (filter,
+            "CS: %f, num_samples %f, channel %d, RMS %f",
+            filter->CS[i], filter->num_samples, i, RMS);
 
+        /* RMS values are calculated in amplitude, so 20 * log 10 */
+        RMSdB = 20 * log10 (RMS);
+        /* peak values are square sums, ie. power, so 10 * log 10 */
+        lastdB = 10 * log10 (filter->last_peak[i]);
+        decaydB = 10 * log10 (filter->decay_peak[i]);
+
+        GST_LOG_OBJECT (filter,
+            "time %f, channel %d, RMS %f dB, peak %f dB, decay %f dB",
+            endtime, i, RMSdB, lastdB, decaydB);
         g_signal_emit (G_OBJECT (filter), gst_filter_signals[SIGNAL_LEVEL], 0,
-            endtime, i,
-            20 * log10 (RMS), 20 * log10 (filter->last_peak[i]),
-            20 * log10 (filter->decay_peak[i]));
+            endtime, i, RMSdB, lastdB, decaydB);
         /* we emitted, so reset cumulative and normal peak */
         filter->CS[i] = 0.0;
         filter->last_peak[i] = 0.0;
@@ -282,23 +314,8 @@ gst_level_fast_16bit_chain (gint16 * in, guint num, gint channels,
     }
     filter->num_samples = 0;
   }
-}
 
-static GstElementStateReturn
-gst_level_change_state (GstElement * element)
-{
-  GstLevel *filter = GST_LEVEL (element);
-
-  switch (GST_STATE_TRANSITION (element)) {
-    case GST_STATE_PAUSED_TO_PLAYING:
-      if (!filter->inited)
-        return GST_STATE_FAILURE;
-      break;
-    default:
-      break;
-  }
-
-  return GST_ELEMENT_CLASS (parent_class)->change_state (element);
+  gst_pad_push (filter->srcpad, GST_DATA (buf));
 }
 
 static void
@@ -307,7 +324,6 @@ gst_level_set_property (GObject * object, guint prop_id,
 {
   GstLevel *filter;
 
-  /* it's not null if we got it, but it might not be ours */
   g_return_if_fail (GST_IS_LEVEL (object));
   filter = GST_LEVEL (object);
 
@@ -335,7 +351,6 @@ gst_level_get_property (GObject * object, guint prop_id,
 {
   GstLevel *filter;
 
-  /* it's not null if we got it, but it might not be ours */
   g_return_if_fail (GST_IS_LEVEL (object));
   filter = GST_LEVEL (object);
 
@@ -368,8 +383,6 @@ gst_level_base_init (GstLevelClass * klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&src_template_factory));
   gst_element_class_set_details (element_class, &level_details);
-
-  element_class->change_state = gst_level_change_state;
 }
 
 static void
@@ -401,12 +414,29 @@ gst_level_class_init (GstLevelClass * klass)
   gobject_class->set_property = gst_level_set_property;
   gobject_class->get_property = gst_level_get_property;
 
+  /**
+   * GstLevel::level:
+   * @gstelement: the #GstLevel object which received the signal
+   * @double:     the time (in seconds) for this level notification
+   * @int:        the channel index [0..n-1] for this level notification
+   * @double:     the average RMS (amplitude) value over this section, in dB
+   * @double:     the peak RMS (amplitude) value over this section, in dB
+   * @double:     the decay peak RMS (amplitude) value over this section, in dB
+   *
+   * The average RMS gives an indication of the power or "fullness" of audio.
+   * The peak RMS gives an indication of the highest amplitude the audio
+   * has reached in this track.
+   * The decay peak RMS is a value that smears out the peak RMS value over
+   * a longer time period; it is commonly used in VU meters.
+   */
   gst_filter_signals[SIGNAL_LEVEL] =
       g_signal_new ("level", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (GstLevelClass, level), NULL, NULL,
-      gstlevel_cclosure_marshal_VOID__DOUBLE_INT_DOUBLE_DOUBLE_DOUBLE,
+      gst_level_marshal_VOID__DOUBLE_INT_DOUBLE_DOUBLE_DOUBLE,
       G_TYPE_NONE, 5,
       G_TYPE_DOUBLE, G_TYPE_INT, G_TYPE_DOUBLE, G_TYPE_DOUBLE, G_TYPE_DOUBLE);
+
+  GST_DEBUG_CATEGORY_INIT (level_debug, "level", 0, "Level calculation");
 }
 
 static void
@@ -416,19 +446,19 @@ gst_level_init (GstLevel * filter)
       gst_pad_new_from_template (gst_static_pad_template_get
       (&sink_template_factory), "sink");
   gst_pad_set_link_function (filter->sinkpad, gst_level_link);
+  gst_pad_set_getcaps_function (filter->sinkpad, gst_pad_proxy_getcaps);
+  gst_element_add_pad (GST_ELEMENT (filter), filter->sinkpad);
+  gst_pad_set_chain_function (filter->sinkpad, gst_level_chain);
+
   filter->srcpad =
       gst_pad_new_from_template (gst_static_pad_template_get
       (&src_template_factory), "src");
   gst_pad_set_link_function (filter->srcpad, gst_level_link);
-
-  gst_element_add_pad (GST_ELEMENT (filter), filter->sinkpad);
-  gst_pad_set_chain_function (filter->sinkpad, gst_level_chain);
-  filter->srcpad = gst_pad_new ("src", GST_PAD_SRC);
+  gst_pad_set_getcaps_function (filter->srcpad, gst_pad_proxy_getcaps);
   gst_element_add_pad (GST_ELEMENT (filter), filter->srcpad);
 
   filter->CS = NULL;
   filter->peak = NULL;
-  filter->MS = NULL;
   filter->RMS_dB = NULL;
 
   filter->rate = 0;
