@@ -28,26 +28,29 @@
 #include "ebml-read.h"
 #include "ebml-ids.h"
 
-enum {
+GST_DEBUG_CATEGORY_STATIC (ebmlread_debug);
+#define GST_CAT_DEFAULT ebmlread_debug
+
+enum
+{
   /* FILL ME */
   LAST_SIGNAL
 };
 
-static void gst_ebml_read_class_init   (GstEbmlReadClass *klass);
-static void gst_ebml_read_init         (GstEbmlRead      *ebml);
-static GstElementStateReturn
-	    gst_ebml_read_change_state (GstElement       *element);
+static void gst_ebml_read_class_init (GstEbmlReadClass * klass);
+static void gst_ebml_read_init (GstEbmlRead * ebml);
+static GstElementStateReturn gst_ebml_read_change_state (GstElement * element);
 
 static GstElementClass *parent_class = NULL;
 
 GType
-gst_ebml_read_get_type (void) 
+gst_ebml_read_get_type (void)
 {
   static GType gst_ebml_read_type = 0;
 
   if (!gst_ebml_read_type) {
     static const GTypeInfo gst_ebml_read_info = {
-      sizeof (GstEbmlReadClass),      
+      sizeof (GstEbmlReadClass),
       NULL,
       NULL,
       (GClassInitFunc) gst_ebml_read_class_init,
@@ -59,33 +62,37 @@ gst_ebml_read_get_type (void)
     };
 
     gst_ebml_read_type =
-	g_type_register_static (GST_TYPE_ELEMENT, "GstEbmlRead",
-				&gst_ebml_read_info, 0);
+        g_type_register_static (GST_TYPE_ELEMENT, "GstEbmlRead",
+        &gst_ebml_read_info, 0);
   }
 
   return gst_ebml_read_type;
 }
 
 static void
-gst_ebml_read_class_init (GstEbmlReadClass *klass) 
+gst_ebml_read_class_init (GstEbmlReadClass * klass)
 {
   GstElementClass *gstelement_class = (GstElementClass *) klass;
 
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
+  GST_DEBUG_CATEGORY_INIT (ebmlread_debug, "ebmlread",
+      0, "EBML stream helper class");
+
   gstelement_class->change_state = gst_ebml_read_change_state;
 }
 
 static void
-gst_ebml_read_init (GstEbmlRead *ebml)
+gst_ebml_read_init (GstEbmlRead * ebml)
 {
   ebml->sinkpad = NULL;
   ebml->bs = NULL;
   ebml->level = NULL;
+  ebml->id_cache = 0;
 }
 
 static GstElementStateReturn
-gst_ebml_read_change_state (GstElement *element)
+gst_ebml_read_change_state (GstElement * element)
 {
   GstEbmlRead *ebml = GST_EBML_READ (element);
 
@@ -114,6 +121,66 @@ gst_ebml_read_change_state (GstElement *element)
   return GST_STATE_SUCCESS;
 }
 
+/*
+ * Event handler. Basic:
+ * - EOS: end-of-file, stop processing, forward EOS.
+ * - Interrupt: stop processing.
+ * - Discont: shouldn't be handled here but in the seek handler. Error.
+ * - Flush: ignore, since we check for flush flags manually. Don't forward.
+ * - Others: warn, ignore.
+ * Return value indicates whether to continue processing.
+ */
+
+static gboolean
+gst_ebml_read_use_event (GstEbmlRead * ebml, GstEvent * event)
+{
+  if (!event) {
+    GST_ELEMENT_ERROR (ebml, RESOURCE, READ, (NULL), (NULL));
+    return FALSE;
+  }
+
+  GST_LOG ("Received event of type %d", GST_EVENT_TYPE (event));
+
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_EOS:
+      gst_pad_event_default (ebml->sinkpad, event);
+      return FALSE;
+
+    case GST_EVENT_INTERRUPT:
+      gst_event_unref (event);
+      return FALSE;
+
+    case GST_EVENT_DISCONTINUOUS:
+      GST_WARNING_OBJECT (ebml, "Unexpected discont - might lose sync");
+      gst_event_unref (event);
+      return TRUE;
+
+    case GST_EVENT_FLUSH:
+      gst_event_unref (event);
+      return TRUE;
+
+    default:
+      GST_WARNING ("don't know how to handle event %d", GST_EVENT_TYPE (event));
+      gst_pad_event_default (ebml->sinkpad, event);
+      return FALSE;
+  }
+
+  /* happy */
+  g_assert_not_reached ();
+  return FALSE;
+}
+
+static gboolean
+gst_ebml_read_handle_event (GstEbmlRead * ebml)
+{
+  GstEvent *event = NULL;
+  guint32 remaining;
+
+  gst_bytestream_get_status (ebml->bs, &remaining, &event);
+
+  return gst_ebml_read_use_event (ebml, event);
+}
+
 /*
  * Return: the amount of levels in the hierarchy that the
  * current element lies higher than the previous one.
@@ -122,7 +189,7 @@ gst_ebml_read_change_state (GstElement *element)
  */
 
 static guint
-gst_ebml_read_element_level_up (GstEbmlRead *ebml)
+gst_ebml_read_element_level_up (GstEbmlRead * ebml)
 {
   guint num = 0;
   guint64 pos = gst_bytestream_tell (ebml->bs);
@@ -148,30 +215,23 @@ gst_ebml_read_element_level_up (GstEbmlRead *ebml)
  */
 
 static gint
-gst_ebml_read_element_id (GstEbmlRead *ebml,
-			  guint32     *id,
-			  guint       *level_up)
+gst_ebml_read_element_id (GstEbmlRead * ebml, guint32 * id, guint * level_up)
 {
   guint8 *data;
   gint len_mask = 0x80, read = 1, n = 1;
   guint32 total;
 
-  while (gst_bytestream_peek_bytes (ebml->bs, &data, 1) != 1) {
-    GstEvent *event = NULL;
-    guint32 remaining;
+  if (ebml->id_cache) {
+    *id = ebml->id_cache;
+    if (level_up)
+      *level_up = 0;
 
-    /* Here, we might encounter EOS */
-    gst_bytestream_get_status (ebml->bs, &remaining, &event);
-    if (event) {
-      gst_pad_event_default (ebml->sinkpad, event);
-    } else {
-      guint64 pos = gst_bytestream_tell (ebml->bs);
-      gst_event_unref (event);
-      GST_ELEMENT_ERROR (ebml, RESOURCE, READ, (NULL),
-			 ("Read error at position %llu (0x%llx)",
-			 pos, pos));
+    return 0;
+  }
+
+  while (gst_bytestream_peek_bytes (ebml->bs, &data, 1) != 1) {
+    if (!gst_ebml_read_handle_event (ebml))
       return -1;
-    }
   }
   total = data[0];
   while (read <= 4 && !(total & len_mask)) {
@@ -180,28 +240,29 @@ gst_ebml_read_element_id (GstEbmlRead *ebml,
   }
   if (read > 4) {
     guint64 pos = gst_bytestream_tell (ebml->bs);
+
     GST_ELEMENT_ERROR (ebml, STREAM, DEMUX, (NULL),
-		       ("Invalid EBML ID size tag (0x%x) at position %llu (0x%llx)",
-		       data[0], pos, pos));
+        ("Invalid EBML ID size tag (0x%x) at position %llu (0x%llx)",
+            data[0], pos, pos));
     return -1;
   }
 
-  if (gst_bytestream_peek_bytes (ebml->bs, &data, read) != read) {
-    guint64 pos = gst_bytestream_tell (ebml->bs);
-    GST_ELEMENT_ERROR (ebml, RESOURCE, READ, (NULL),
-		       ("Read error at position %llu (0x%llx)", pos, pos));
-    return -1;
+  while (gst_bytestream_peek_bytes (ebml->bs, &data, read) != read) {
+    if (!gst_ebml_read_handle_event (ebml))
+      return -1;
   }
   while (n < read)
     total = (total << 8) | data[n++];
 
-  *id = total;
+  *id = ebml->id_cache = total;
 
   /* level */
   if (level_up)
     *level_up = gst_ebml_read_element_level_up (ebml);
 
-  return read;
+  gst_bytestream_flush_fast (ebml->bs, read);
+
+  return 0;
 }
 
 /*
@@ -210,18 +271,15 @@ gst_ebml_read_element_id (GstEbmlRead *ebml,
  */
 
 static gint
-gst_ebml_read_element_length (GstEbmlRead *ebml,
-			      guint64     *length)
+gst_ebml_read_element_length (GstEbmlRead * ebml, guint64 * length)
 {
   guint8 *data;
   gint len_mask = 0x80, read = 1, n = 1, num_ffs = 0;
   guint64 total;
 
-  if (gst_bytestream_peek_bytes (ebml->bs, &data, 1) != 1) {
-    guint64 pos = gst_bytestream_tell (ebml->bs);
-    GST_ELEMENT_ERROR (ebml, RESOURCE, READ, (NULL),
-		       ("Read error at position %llu (0x%llx)", pos, pos));
-    return -1;
+  while (gst_bytestream_peek_bytes (ebml->bs, &data, 1) != 1) {
+    if (!gst_ebml_read_handle_event (ebml))
+      return -1;
   }
   total = data[0];
   while (read <= 8 && !(total & len_mask)) {
@@ -230,19 +288,18 @@ gst_ebml_read_element_length (GstEbmlRead *ebml,
   }
   if (read > 8) {
     guint64 pos = gst_bytestream_tell (ebml->bs);
+
     GST_ELEMENT_ERROR (ebml, STREAM, DEMUX, (NULL),
-		       ("Invalid EBML length size tag (0x%x) at position %llu (0x%llx)",
-		       data[0], pos, pos));
+        ("Invalid EBML length size tag (0x%x) at position %llu (0x%llx)",
+            data[0], pos, pos));
     return -1;
   }
 
   if ((total &= (len_mask - 1)) == len_mask - 1)
     num_ffs++;
-  if (gst_bytestream_peek_bytes (ebml->bs, &data, read) != read) {
-    guint64 pos = gst_bytestream_tell (ebml->bs);
-    GST_ELEMENT_ERROR (ebml, RESOURCE, READ, (NULL),
-		       ("Read error at position %llu (0x%llx)", pos, pos));
-    return -1;
+  while (gst_bytestream_peek_bytes (ebml->bs, &data, read) != read) {
+    if (!gst_ebml_read_handle_event (ebml))
+      return -1;
   }
   while (n < read) {
     if (data[n] == 0xff)
@@ -265,18 +322,13 @@ gst_ebml_read_element_length (GstEbmlRead *ebml,
  */
 
 static GstBuffer *
-gst_ebml_read_element_data (GstEbmlRead *ebml,
-			    guint64      length)
+gst_ebml_read_element_data (GstEbmlRead * ebml, guint64 length)
 {
   GstBuffer *buf = NULL;
 
   if (gst_bytestream_peek (ebml->bs, &buf, length) != length) {
-    guint64 pos = gst_bytestream_tell (ebml->bs);
-    GST_ELEMENT_ERROR (ebml, RESOURCE, READ, (NULL),
-		       ("Read error at position %llu (0x%llx)", pos, pos));
-    if (buf)
-      gst_buffer_unref (buf);
-    return NULL;
+    if (!gst_ebml_read_handle_event (ebml))
+      return NULL;
   }
 
   gst_bytestream_flush_fast (ebml->bs, length);
@@ -291,8 +343,7 @@ gst_ebml_read_element_data (GstEbmlRead *ebml,
  */
 
 guint32
-gst_ebml_peek_id (GstEbmlRead *ebml,
-		  guint       *level_up)
+gst_ebml_peek_id (GstEbmlRead * ebml, guint * level_up)
 {
   guint32 id;
 
@@ -309,8 +360,7 @@ gst_ebml_peek_id (GstEbmlRead *ebml,
  */
 
 GstEvent *
-gst_ebml_read_seek (GstEbmlRead *ebml,
-		    guint64      offset)
+gst_ebml_read_seek (GstEbmlRead * ebml, guint64 offset)
 {
   guint32 remaining;
   GstEvent *event = NULL;
@@ -319,8 +369,10 @@ gst_ebml_read_seek (GstEbmlRead *ebml,
   /* first, flush remaining buffers */
   gst_bytestream_get_status (ebml->bs, &remaining, &event);
   if (event) {
-    g_warning ("Unexpected event before seek");
-    gst_event_unref (event);
+    GST_WARNING ("Unexpected event before seek");
+    if (!gst_ebml_read_use_event (ebml, event))
+      return NULL;
+    event = NULL;
   }
   if (remaining)
     gst_bytestream_flush_fast (ebml->bs, remaining);
@@ -328,7 +380,7 @@ gst_ebml_read_seek (GstEbmlRead *ebml,
   /* now seek */
   if (!gst_bytestream_seek (ebml->bs, offset, GST_SEEK_METHOD_SET)) {
     GST_ELEMENT_ERROR (ebml, RESOURCE, SEEK, (NULL),
-		       ("Seek to position %llu (0x%llx) failed", offset, offset));
+        ("Seek to position %llu (0x%llx) failed", offset, offset));
     return NULL;
   }
 
@@ -346,11 +398,15 @@ gst_ebml_read_seek (GstEbmlRead *ebml,
       GST_WARNING ("No discontinuity event after seek - seek failed");
       break;
     } else if (GST_EVENT_TYPE (event) != GST_EVENT_DISCONTINUOUS) {
-      gst_pad_event_default (ebml->sinkpad, event);
+      if (!gst_ebml_read_use_event (ebml, event))
+        return NULL;
       event = NULL;
     }
   }
 
+  /* at the end of a seek, we have no cached ID anymore */
+  ebml->id_cache = 0;
+
   return event;
 }
 
@@ -359,7 +415,7 @@ gst_ebml_read_seek (GstEbmlRead *ebml,
  */
 
 gboolean
-gst_ebml_read_skip (GstEbmlRead *ebml)
+gst_ebml_read_skip (GstEbmlRead * ebml)
 {
   gint bytes;
   guint32 id, remaining;
@@ -378,40 +434,84 @@ gst_ebml_read_skip (GstEbmlRead *ebml)
   gst_bytestream_get_status (ebml->bs, &remaining, &event);
   if (event) {
     g_warning ("Unexpected event before skip");
-    gst_event_unref (event);
+    if (!gst_ebml_read_use_event (ebml, event))
+      return FALSE;
   }
 
-  if (remaining >= length)
-    return gst_bytestream_flush (ebml->bs, length);
+  if (remaining >= length) {
+    ebml->id_cache = 0;
+    gst_bytestream_flush_fast (ebml->bs, length);
+
+    return TRUE;
+  }
 
   if (!(event = gst_ebml_read_seek (ebml,
-			gst_bytestream_tell (ebml->bs) + length)))
+              gst_bytestream_tell (ebml->bs) + length))) {
     return FALSE;
+  }
 
   gst_event_unref (event);
+  ebml->id_cache = 0;
 
   return TRUE;
 }
 
+/*
+ * Make sure there's one whole next chunk available
+ * for uninterrupted reading.
+ */
+
+gboolean
+gst_ebml_read_reserve (GstEbmlRead * ebml)
+{
+  guint64 length;
+  gint bytes;
+  guint8 *data;
+  guint32 id;
+
+  /* make sure the ID is cached */
+  if (gst_ebml_read_element_id (ebml, &id, NULL) < 0)
+    return FALSE;
+
+  /* now we can peek the length */
+  if ((bytes = gst_ebml_read_element_length (ebml, &length)) < 0)
+    return FALSE;
+  length += bytes;
+
+  /* now check if we have that amount available */
+  while (gst_bytestream_peek_bytes (ebml->bs, &data, length) != length) {
+    if (!gst_ebml_read_handle_event (ebml))
+      return FALSE;
+  }
+
+  /* yes, available */
+  return TRUE;
+}
+
 /*
  * Read the next element as a GstBuffer (binary).
  */
 
 gboolean
-gst_ebml_read_buffer (GstEbmlRead *ebml,
-		      guint32     *id,
-		      GstBuffer  **buf)
+gst_ebml_read_buffer (GstEbmlRead * ebml, guint32 * id, GstBuffer ** buf)
 {
   gint bytes;
   guint64 length;
 
   if ((bytes = gst_ebml_read_element_id (ebml, id, NULL)) < 0)
     return FALSE;
+
   gst_bytestream_flush_fast (ebml->bs, bytes);
 
   if ((bytes = gst_ebml_read_element_length (ebml, &length)) < 0)
     return FALSE;
+
   gst_bytestream_flush_fast (ebml->bs, bytes);
+  ebml->id_cache = 0;
+  if (length == 0) {
+    *buf = gst_buffer_new ();
+    return TRUE;
+  }
 
   return ((*buf = gst_ebml_read_element_data (ebml, length)) != NULL);
 }
@@ -421,9 +521,7 @@ gst_ebml_read_buffer (GstEbmlRead *ebml,
  */
 
 gboolean
-gst_ebml_read_uint (GstEbmlRead *ebml,
-		    guint32     *id,
-		    guint64     *num)
+gst_ebml_read_uint (GstEbmlRead * ebml, guint32 * id, guint64 * num)
 {
   GstBuffer *buf;
   guint8 *data;
@@ -436,8 +534,8 @@ gst_ebml_read_uint (GstEbmlRead *ebml,
   size = GST_BUFFER_SIZE (buf);
   if (size < 1 || size > 8) {
     GST_ELEMENT_ERROR (ebml, STREAM, DEMUX, (NULL),
-		       ("Invalid integer element size %d at position %llu (0x%llu)",
-		       size, GST_BUFFER_OFFSET (buf), GST_BUFFER_OFFSET (buf)));
+        ("Invalid integer element size %d at position %llu (0x%llu)",
+            size, GST_BUFFER_OFFSET (buf), GST_BUFFER_OFFSET (buf)));
     gst_buffer_unref (buf);
     return FALSE;
   }
@@ -457,9 +555,7 @@ gst_ebml_read_uint (GstEbmlRead *ebml,
  */
 
 gboolean
-gst_ebml_read_sint (GstEbmlRead *ebml,
-		    guint32     *id,
-		    gint64      *num)
+gst_ebml_read_sint (GstEbmlRead * ebml, guint32 * id, gint64 * num)
 {
   GstBuffer *buf;
   guint8 *data;
@@ -472,8 +568,8 @@ gst_ebml_read_sint (GstEbmlRead *ebml,
   size = GST_BUFFER_SIZE (buf);
   if (size < 1 || size > 8) {
     GST_ELEMENT_ERROR (ebml, STREAM, DEMUX, (NULL),
-		       ("Invalid integer element size %d at position %llu (0x%llx)",
-		       size, GST_BUFFER_OFFSET (buf), GST_BUFFER_OFFSET (buf)));
+        ("Invalid integer element size %d at position %llu (0x%llx)",
+            size, GST_BUFFER_OFFSET (buf), GST_BUFFER_OFFSET (buf)));
     gst_buffer_unref (buf);
     return FALSE;
   }
@@ -501,9 +597,7 @@ gst_ebml_read_sint (GstEbmlRead *ebml,
  */
 
 gboolean
-gst_ebml_read_float (GstEbmlRead *ebml,
-		     guint32     *id,
-		     gdouble     *num)
+gst_ebml_read_float (GstEbmlRead * ebml, guint32 * id, gdouble * num)
 {
   GstBuffer *buf;
   guint8 *data;
@@ -517,15 +611,15 @@ gst_ebml_read_float (GstEbmlRead *ebml,
 
   if (size != 4 && size != 8 && size != 10) {
     GST_ELEMENT_ERROR (ebml, STREAM, DEMUX, (NULL),
-		       ("Invalid float element size %d at position %llu (0x%llx)",
-		       size, GST_BUFFER_OFFSET (buf), GST_BUFFER_OFFSET (buf)));
+        ("Invalid float element size %d at position %llu (0x%llx)",
+            size, GST_BUFFER_OFFSET (buf), GST_BUFFER_OFFSET (buf)));
     gst_buffer_unref (buf);
     return FALSE;
   }
 
   if (size == 10) {
     GST_ELEMENT_ERROR (ebml, CORE, NOT_IMPLEMENTED, (NULL),
-		       ("FIXME! 10-byte floats unimplemented"));
+        ("FIXME! 10-byte floats unimplemented"));
     gst_buffer_unref (buf);
     return FALSE;
   }
@@ -534,10 +628,10 @@ gst_ebml_read_float (GstEbmlRead *ebml,
     gfloat f;
 
 #if (G_BYTE_ORDER == G_BIG_ENDIAN)
-    f = * (gfloat *) data;
+    f = *(gfloat *) data;
 #else
     while (size > 0) {
-      ((guint8 *) &f)[size - 1] = data[4 - size];
+      ((guint8 *) & f)[size - 1] = data[4 - size];
       size--;
     }
 #endif
@@ -547,10 +641,10 @@ gst_ebml_read_float (GstEbmlRead *ebml,
     gdouble d;
 
 #if (G_BYTE_ORDER == G_BIG_ENDIAN)
-    d = * (gdouble *) data;
+    d = *(gdouble *) data;
 #else
     while (size > 0) {
-      ((guint8 *) &d)[size - 1] = data[8 - size];
+      ((guint8 *) & d)[size - 1] = data[8 - size];
       size--;
     }
 #endif
@@ -568,9 +662,7 @@ gst_ebml_read_float (GstEbmlRead *ebml,
  */
 
 gboolean
-gst_ebml_read_ascii (GstEbmlRead *ebml,
-		     guint32     *id,
-		     gchar      **str)
+gst_ebml_read_ascii (GstEbmlRead * ebml, guint32 * id, gchar ** str)
 {
   GstBuffer *buf;
 
@@ -591,23 +683,24 @@ gst_ebml_read_ascii (GstEbmlRead *ebml,
  */
 
 gboolean
-gst_ebml_read_utf8 (GstEbmlRead *ebml,
-		    guint32     *id,
-		    gchar      **str)
+gst_ebml_read_utf8 (GstEbmlRead * ebml, guint32 * id, gchar ** str)
 {
   return gst_ebml_read_ascii (ebml, id, str);
 }
 
 /*
- * Read the next element as a date (nanoseconds since 1/1/2000).
+ * Read the next element as a date.
+ * Returns the seconds since the unix epoch.
  */
 
 gboolean
-gst_ebml_read_date (GstEbmlRead *ebml,
-		    guint32     *id,
-		    gint64      *date)
+gst_ebml_read_date (GstEbmlRead * ebml, guint32 * id, gint64 * date)
 {
-  return gst_ebml_read_sint (ebml, id, date);
+  gint64 ebml_date;
+  gboolean res = gst_ebml_read_sint (ebml, id, &ebml_date);
+
+  *date = (ebml_date / GST_SECOND) + GST_EBML_DATE_OFFSET;
+  return res;
 }
 
 /*
@@ -616,8 +709,7 @@ gst_ebml_read_date (GstEbmlRead *ebml,
  */
 
 gboolean
-gst_ebml_read_master (GstEbmlRead *ebml,
-		      guint32     *id)
+gst_ebml_read_master (GstEbmlRead * ebml, guint32 * id)
 {
   gint bytes;
   guint64 length;
@@ -636,6 +728,7 @@ gst_ebml_read_master (GstEbmlRead *ebml,
   level->start = gst_bytestream_tell (ebml->bs);
   level->length = length;
   ebml->level = g_list_append (ebml->level, level);
+  ebml->id_cache = 0;
 
   return TRUE;
 }
@@ -645,10 +738,8 @@ gst_ebml_read_master (GstEbmlRead *ebml,
  */
 
 gboolean
-gst_ebml_read_binary (GstEbmlRead *ebml,
-		      guint32     *id,
-		      guint8     **binary,
-		      guint64     *length)
+gst_ebml_read_binary (GstEbmlRead * ebml,
+    guint32 * id, guint8 ** binary, guint64 * length)
 {
   GstBuffer *buf;
 
@@ -668,9 +759,7 @@ gst_ebml_read_binary (GstEbmlRead *ebml,
  */
 
 gboolean
-gst_ebml_read_header (GstEbmlRead *ebml,
-		      gchar      **doctype,
-		      guint       *version)
+gst_ebml_read_header (GstEbmlRead * ebml, gchar ** doctype, guint * version)
 {
   /* this function is the first to be called */
   guint32 id;
@@ -701,8 +790,8 @@ gst_ebml_read_header (GstEbmlRead *ebml,
       break;
 
     switch (id) {
-      /* is our read version uptodate? */
-      case GST_EBML_ID_EBMLREADVERSION: {
+        /* is our read version uptodate? */
+      case GST_EBML_ID_EBMLREADVERSION:{
         guint64 num;
 
         if (!gst_ebml_read_uint (ebml, &id, &num))
@@ -713,8 +802,8 @@ gst_ebml_read_header (GstEbmlRead *ebml,
         break;
       }
 
-      /* we only handle 8 byte lengths at max */
-      case GST_EBML_ID_EBMLMAXSIZELENGTH: {
+        /* we only handle 8 byte lengths at max */
+      case GST_EBML_ID_EBMLMAXSIZELENGTH:{
         guint64 num;
 
         if (!gst_ebml_read_uint (ebml, &id, &num))
@@ -725,8 +814,8 @@ gst_ebml_read_header (GstEbmlRead *ebml,
         break;
       }
 
-      /* we handle 4 byte IDs at max */
-      case GST_EBML_ID_EBMLMAXIDLENGTH: {
+        /* we handle 4 byte IDs at max */
+      case GST_EBML_ID_EBMLMAXIDLENGTH:{
         guint64 num;
 
         if (!gst_ebml_read_uint (ebml, &id, &num))
@@ -737,12 +826,12 @@ gst_ebml_read_header (GstEbmlRead *ebml,
         break;
       }
 
-      case GST_EBML_ID_DOCTYPE: {
+      case GST_EBML_ID_DOCTYPE:{
         gchar *text;
 
         if (!gst_ebml_read_ascii (ebml, &id, &text))
           return FALSE;
-        g_assert (id == GST_EBML_ID_DOCTYPE); 
+        g_assert (id == GST_EBML_ID_DOCTYPE);
         if (doctype) {
           if (doctype)
             g_free (*doctype);
@@ -752,12 +841,12 @@ gst_ebml_read_header (GstEbmlRead *ebml,
         break;
       }
 
-      case GST_EBML_ID_DOCTYPEREADVERSION: {
+      case GST_EBML_ID_DOCTYPEREADVERSION:{
         guint64 num;
 
         if (!gst_ebml_read_uint (ebml, &id, &num))
           return FALSE;
-        g_assert (id == GST_EBML_ID_DOCTYPEREADVERSION); 
+        g_assert (id == GST_EBML_ID_DOCTYPEREADVERSION);
         if (version)
           *version = num;
         break;
@@ -767,7 +856,7 @@ gst_ebml_read_header (GstEbmlRead *ebml,
         GST_WARNING ("Unknown data type 0x%x in EBML header (ignored)", id);
         /* pass-through */
 
-      /* we ignore these two, as they don't tell us anything we care about */
+        /* we ignore these two, as they don't tell us anything we care about */
       case GST_EBML_ID_VOID:
       case GST_EBML_ID_EBMLVERSION:
       case GST_EBML_ID_DOCTYPEVERSION:
