@@ -521,7 +521,7 @@ gst_mpeg2subt_parse_header (GstMpeg2Subt * mpeg2subt)
         mpeg2subt->subtitle_alpha);
 }
 
-inline int
+static inline int
 gst_get_nibble (guchar * buffer, RLE_state * state)
 {
   if (state->aligned) {
@@ -551,7 +551,7 @@ gst_setup_palette (GstMpeg2Subt * mpeg2subt, guchar * indexes, guchar * alpha)
   }
 }
 
-inline guint
+static inline guint
 gst_get_rle_code (guchar * buffer, RLE_state * state)
 {
   gint code;
@@ -635,7 +635,7 @@ gst_draw_rle_line (GstMpeg2Subt * mpeg2subt, guchar * buffer, RLE_state * state)
   }
 }
 
-inline void
+static inline void
 gst_merge_uv_data (GstMpeg2Subt * mpeg2subt, guchar * buffer, RLE_state * state)
 {
   gint x;
@@ -704,6 +704,7 @@ gst_mpeg2subt_merge_title (GstMpeg2Subt * mpeg2subt, GstBuffer * buf)
       " using %s colour table", GST_BUFFER_TIMESTAMP (buf),
       mpeg2subt->forced_display ? "menu" : "subtitle");
 
+  state.next = 0;
   state.id = 0;
   state.aligned = 1;
   state.offset[0] = mpeg2subt->offset[0];
@@ -908,7 +909,7 @@ gst_mpeg2subt_handle_dvd_event (GstMpeg2Subt * mpeg2subt, GstEvent * event,
     gst_update_still_frame (mpeg2subt);
   } else if (from_sub_pad && !strcmp (event_type, "dvd-spu-clut-change")) {
     /* Take a copy of the colour table */
-    guchar name[16];
+    gchar name[16];
     int i;
     gint value;
 
