@@ -207,6 +207,7 @@ static GstTagEntryMatch tag_matches[] = {
   {GST_TAG_MUSICBRAINZ_ALBUMARTISTID, "MUSICBRAINZ_ALBUMARTISTID"},
   {GST_TAG_MUSICBRAINZ_TRMID, "MUSICBRAINZ_TRMID"},
   {GST_TAG_MUSICBRAINZ_SORTNAME, "MUSICBRAINZ_SORTNAME"},
+  {GST_TAG_LANGUAGE_CODE, "LANGUAGE"},
   {NULL, NULL}
 };
 
@@ -326,13 +327,24 @@ gst_vorbis_tag_add (GstTagList * list, const gchar * tag, const gchar * value)
       }
       break;
     case G_TYPE_STRING:{
-      gchar *valid;
+      gchar *valid = NULL;
 
-      if (!g_utf8_validate (value, -1, (const gchar **) &valid)) {
-        valid = g_strndup (value, valid - value);
-        GST_DEBUG ("Invalid vorbis comment tag, truncated it to %s\n", valid);
-      } else {
-        valid = g_strdup (value);
+      /* specialcase for language code */
+      if (strcmp (tag, "LANGUAGE") == 0) {
+        const gchar *s = strchr (value, '[');
+
+        if (s && strchr (s, ']') == s + 4) {
+          valid = g_strndup (s + 1, 3);
+        }
+      }
+
+      if (!valid) {
+        if (!g_utf8_validate (value, -1, (const gchar **) &valid)) {
+          valid = g_strndup (value, valid - value);
+          GST_DEBUG ("Invalid vorbis comment tag, truncated it to %s", valid);
+        } else {
+          valid = g_strdup (value);
+        }
       }
       gst_tag_list_add (list, GST_TAG_MERGE_APPEND, gst_tag, valid, NULL);
       g_free (valid);
@@ -373,7 +385,7 @@ gst_tag_list_from_vorbiscomment_buffer (const GstBuffer * buffer,
   data += 4;									\
   size -= 4;									\
   if (cur_size > size) goto error;						\
-  cur = data;									\
+  cur = (gchar*) data;								\
 }G_STMT_END
   gchar *cur, *value;
   guint cur_size;
@@ -483,6 +495,7 @@ gst_tag_to_vorbis_comments (const GstTagList * list, const gchar * tag)
         if (!gst_tag_list_get_double_index (list, tag, i, &value))
           g_assert_not_reached ();
         result = g_strdup_printf ("%s=%f", vorbis_tag, value);
+        break;
       }
       default:
         GST_DEBUG ("Couldn't write tag %s", tag);
@@ -617,9 +630,8 @@ gst_vorbis_tag_chain (GstPad * pad, GstData * data)
 
   if (GST_BUFFER_DATA (buffer)[0] == 3) {
     gchar *vendor;
-    GstTagList *list =
-        gst_tag_list_from_vorbiscomment_buffer (buffer, "\003vorbis", 7,
-        &vendor);
+    GstTagList *list = gst_tag_list_from_vorbiscomment_buffer (buffer,
+        (const guint8 *) "\003vorbis", 7, &vendor);
     const GstTagList *found_tags;
 
     gst_data_unref (data);
@@ -635,8 +647,8 @@ gst_vorbis_tag_chain (GstPad * pad, GstData * data)
       gst_tag_list_insert (list, found_tags,
           gst_tag_setter_get_merge_mode (GST_TAG_SETTER (tag)));
     data =
-        GST_DATA (gst_tag_list_to_vorbiscomment_buffer (list, "\003vorbis", 7,
-            vendor));
+        GST_DATA (gst_tag_list_to_vorbiscomment_buffer (list,
+            (const guint8 *) "\003vorbis", 7, vendor));
     gst_tag_list_free (list);
     g_free (vendor);
   }
