@@ -30,7 +30,9 @@ extern "C" {
 
 #include <stdio.h>
 #include <stdlib.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <errno.h>
 #include <string.h>
 #include <sys/time.h>
@@ -78,6 +80,7 @@ struct _GstUDPSink {
   gchar *host;
     
   guint mtu;
+  guint ttl;
     
   GstClock *clock;
 };
