@@ -18,11 +18,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
-
 #ifndef __GST_VIDEOTESTSRC_H__
 #define __GST_VIDEOTESTSRC_H__
 
-
 #include <gst/gst.h>
 
 G_BEGIN_DECLS
@@ -50,7 +48,7 @@ typedef struct _GstVideotestsrcClass GstVideotestsrcClass;
 struct _GstVideotestsrc {
   GstElement element;
 
-  GstPad *sinkpad,*srcpad;
+  GstPad *sinkpad, *srcpad;
 
   /* parameters */
   gboolean sync;
@@ -60,23 +58,29 @@ struct _GstVideotestsrc {
   gint width;
   gint height;
   struct fourcc_list_struct *fourcc;
-  
+
   /* private */
-  gint64 timestamp_offset;
-  gint64 n_frames;
+  gint64 timestamp_offset;		/* base offset */
+  GstClockTime running_time;		/* total running time */
+  gint64 n_frames;			/* total frames sent */
   gint bpp;
   gdouble rate;
   int type;
   GstClock *clock;
+  gint num_buffers, num_buffers_left;
+  gboolean need_discont;
+  gboolean loop;
+  gint64 segment_start_frame;
+  gint64 segment_end_frame;
 
-  void (*make_image)(GstVideotestsrc *v, unsigned char *dest, int w, int h);
+  void (*make_image) (GstVideotestsrc *v, unsigned char *dest, int w, int h);
 };
 
 struct _GstVideotestsrcClass {
   GstElementClass parent_class;
 };
 
-GType gst_videotestsrc_get_type(void) G_GNUC_CONST;
+GType gst_videotestsrc_get_type (void) G_GNUC_CONST;
 
 G_END_DECLS
 
