@@ -69,8 +69,8 @@ static GstStaticPadTemplate src_template_factory =
     GST_STATIC_CAPS ("audio/x-raw-int, "
         "endianness = (int) little_endian, "
         "signed = (boolean) { true, false }, "
-        "width = (int) { 8, 16 }, "
-        "depth = (int) { 8, 16 }, "
+        "width = (int) { 8, 16, 24, 32 }, "
+        "depth = (int) { 8, 16, 24, 32 }, "
         "rate = (int) [ 8000, 48000 ], "
         "channels = (int) [ 1, 2 ]; "
         "audio/mpeg, "
@@ -229,7 +229,7 @@ gst_wavparse_destroy_sourcepad (GstWavParse * wavparse)
   }
 }
 
-static void
+static gboolean
 gst_wavparse_create_sourcepad (GstWavParse * wavparse)
 {
   gst_wavparse_destroy_sourcepad (wavparse);
@@ -238,6 +238,13 @@ gst_wavparse_create_sourcepad (GstWavParse * wavparse)
   wavparse->srcpad =
       gst_pad_new_from_template (gst_static_pad_template_get
       (&src_template_factory), "src");
+
+  if (!wavparse->srcpad) {
+    GST_ELEMENT_ERROR (wavparse, CORE, PAD,
+        ("Failed to allocate new pad from template"), (NULL));
+    return FALSE;
+  }
+
   gst_pad_use_explicit_caps (wavparse->srcpad);
   gst_pad_set_formats_function (wavparse->srcpad, gst_wavparse_get_formats);
   gst_pad_set_convert_function (wavparse->srcpad, gst_wavparse_pad_convert);
@@ -247,6 +254,8 @@ gst_wavparse_create_sourcepad (GstWavParse * wavparse)
   gst_pad_set_event_function (wavparse->srcpad, gst_wavparse_srcpad_event);
   gst_pad_set_event_mask_function (wavparse->srcpad,
       gst_wavparse_get_event_masks);
+
+  return TRUE;
 }
 
 static void
@@ -545,29 +554,52 @@ gst_wavparse_fmt (GstWavParse * wav)
     return FALSE;
   }
 
-  /* Note: gst_riff_create_audio_caps might nedd to fix values in
-   * the header header depending on the format, so call it first */
-  caps = gst_riff_create_audio_caps (header->format, NULL, header, NULL);
-
   wav->format = header->format;
   wav->rate = header->rate;
   wav->channels = header->channels;
+  if (wav->channels == 0) {
+    GST_ELEMENT_ERROR (wav, STREAM, FAILED, (NULL),
+        ("Stream claims to contain zero channels - invalid data"));
+    g_free (header);
+    return FALSE;
+  }
   wav->blockalign = header->blockalign;
   wav->width = (header->blockalign * 8) / header->channels;
   wav->depth = header->size;
   wav->bps = header->av_bps;
+  if (wav->bps <= 0) {
+    GST_ELEMENT_ERROR (wav, STREAM, FAILED, (NULL),
+        ("Stream claims to bitrate of <= zero - invalid data"));
+    g_free (header);
+    return FALSE;
+  }
 
+  caps = gst_riff_create_audio_caps (header->format, NULL, header, NULL);
   g_free (header);
 
   if (caps) {
-    gst_wavparse_create_sourcepad (wav);
-    gst_pad_set_explicit_caps (wav->srcpad, caps);
+    if (!gst_wavparse_create_sourcepad (wav))
+      return FALSE;
+
+    if (!gst_pad_set_explicit_caps (wav->srcpad, caps)) {
+      gchar *caps_str = gst_caps_to_string (caps);
+
+      GST_ELEMENT_ERROR (wav, CORE, PAD, (NULL),
+          ("failed to negotiate (try_set_caps with \"%s\" returned REFUSED)",
+              caps_str));
+      g_free (caps_str);
+      gst_caps_free (caps);
+      return FALSE;
+    }
+
     gst_caps_free (caps);
     gst_element_add_pad (GST_ELEMENT (wav), wav->srcpad);
     gst_element_no_more_pads (GST_ELEMENT (wav));
     GST_DEBUG ("frequency %d, channels %d", wav->rate, wav->channels);
   } else {
-    GST_ELEMENT_ERROR (wav, STREAM, TYPE_NOT_FOUND, (NULL), (NULL));
+    GST_ELEMENT_ERROR (wav, STREAM, TYPE_NOT_FOUND, (NULL),
+        ("No caps found for format 0x%x, %d channels, %d Hz",
+            wav->format, wav->channels, wav->rate));
     return FALSE;
   }
 
@@ -743,6 +775,8 @@ gst_wavparse_loop (GstElement * element)
       GstBuffer *buf = NULL;
 
       desired = MIN (wav->dataleft, MAX_BUFFER_SIZE);
+      if (desired >= wav->blockalign)
+        desired -= (desired % wav->blockalign);
       if (!(buf = gst_riff_read_element_data (riff, desired, &got_bytes))) {
         GST_WARNING_OBJECT (wav, "trying to read %d bytes failed", desired);
         return;
