@@ -73,6 +73,7 @@ gst_autoexp_mode_get_type (void)
     {AE_NONE, "3", "None"},
     {0, NULL, NULL},
   };
+
   if (!autoexp_mode_type) {
     autoexp_mode_type =
         g_enum_register_static ("GstAutoExposureMode", autoexp_modes);
@@ -100,7 +101,7 @@ enum
   ARG_TRANSFER_SCALE,
   ARG_DEPTH,
   ARG_PORT,
-  ARG_AUTOEXP,
+  ARG_AUTOEXP
 };
 
 static void gst_qcamsrc_base_init (gpointer g_class);
@@ -140,6 +141,7 @@ gst_qcamsrc_get_type (void)
       (GInstanceInitFunc) gst_qcamsrc_init,
       NULL
     };
+
     qcamsrc_type =
         g_type_register_static (GST_TYPE_ELEMENT, "GstQCamSrc", &qcamsrc_info,
         0);
@@ -386,19 +388,22 @@ gst_qcamsrc_change_state (GstElement * element)
 {
   g_return_val_if_fail (GST_IS_QCAMSRC (element), FALSE);
 
-  /* if going down into NULL state, close the file if it's open */
-  if (GST_STATE_PENDING (element) == GST_STATE_NULL) {
-    if (GST_FLAG_IS_SET (element, GST_QCAMSRC_OPEN))
-      gst_qcamsrc_close (GST_QCAMSRC (element));
-    /* otherwise (READY or higher) we need to open the sound card */
-  } else {
-    if (!GST_FLAG_IS_SET (element, GST_QCAMSRC_OPEN)) {
-      gst_info ("qcamsrc: opening\n");
-      if (!gst_qcamsrc_open (GST_QCAMSRC (element))) {
-        gst_info ("qcamsrc: open failed\n");
-        return GST_STATE_FAILURE;
+  switch (GST_STATE_TRANSITION (element)) {
+    case GST_STATE_READY_TO_NULL:
+      if (GST_FLAG_IS_SET (element, GST_QCAMSRC_OPEN))
+        gst_qcamsrc_close (GST_QCAMSRC (element));
+      break;
+    case GST_STATE_NULL_TO_READY:
+      if (!GST_FLAG_IS_SET (element, GST_QCAMSRC_OPEN)) {
+        GST_DEBUG ("opening");
+        if (!gst_qcamsrc_open (GST_QCAMSRC (element))) {
+          GST_DEBUG ("open failed");
+          return GST_STATE_FAILURE;
+        }
       }
-    }
+      break;
+    default:
+      break;
   }
 
   if (GST_ELEMENT_CLASS (parent_class)->change_state)
@@ -411,7 +416,8 @@ static gboolean
 gst_qcamsrc_open (GstQCamSrc * qcamsrc)
 {
   if (qc_open (qcamsrc->qcam)) {
-    g_warning ("qcamsrc: Cannot open QuickCam.\n");
+    GST_ELEMENT_ERROR (qcamsrc, RESOURCE, OPEN_READ, (NULL),
+        ("Failed to open QuickCam"));
     return FALSE;
   }
 
