@@ -38,8 +38,39 @@
 #include <sys/types.h>
 #include <linux/types.h>
 #define _LINUX_TIME_H
+#define __user
 #include <linux/videodev2.h>
 
+/*
+ * See bug #135919, the Suse9 (and Mandrake10) videodev2 headers
+ * contain a bug where (for userspace applications) the v4l2_buffer
+ * struct is not declared, so applications have to declare it.
+ * Declaration straightly ripped out from <linux/videodev2.h>.
+ */
+#ifdef GST_V4L2_MISSING_BUFDECL
+struct v4l2_buffer
+{
+	__u32			index;
+	enum v4l2_buf_type      type;
+	__u32			bytesused;
+	__u32			flags;
+	enum v4l2_field		field;
+	struct timeval		timestamp;
+	struct v4l2_timecode	timecode;
+	__u32			sequence;
+
+	/* memory location */
+	enum v4l2_memory        memory;
+	union {
+		__u32           offset;
+		unsigned long   userptr;
+	} m;
+	__u32			length;
+
+	__u32			reserved[2];
+};
+#endif /* GST_V4L2_MISSING_BUFDECL */
+
 
 #define GST_TYPE_V4L2ELEMENT \
 		(gst_v4l2element_get_type())
@@ -51,10 +82,13 @@
 		(G_TYPE_CHECK_INSTANCE_TYPE((obj), GST_TYPE_V4L2ELEMENT))
 #define GST_IS_V4L2ELEMENT_CLASS(obj) \
 		(G_TYPE_CHECK_CLASS_TYPE((klass), GST_TYPE_V4L2ELEMENT))
+#define GST_V4L2ELEMENT_GET_CLASS(obj) \
+		(G_TYPE_INSTANCE_GET_CLASS ((obj), GST_TYPE_V4L2ELEMENT, GstV4l2ElementClass))
 
 
 typedef	struct _GstV4l2Element		GstV4l2Element;
 typedef	struct _GstV4l2ElementClass	GstV4l2ElementClass;
+typedef struct _GstV4l2Xv		GstV4l2Xv;
 
 struct _GstV4l2Element {
 	GstElement element;
@@ -77,16 +111,21 @@ struct _GstV4l2Element {
 	GList *colors;
 
 	/* X-overlay */
-	GstXWindowListener *overlay;
+	GstV4l2Xv *xv;
 	XID xwindow_id;
 
-	/* caching values */
-	gchar *display;
+	/* properties */
+	gchar *norm;
+	gchar *channel;
+	gulong frequency;
 };
 
 struct _GstV4l2ElementClass {
 	GstElementClass parent_class;
 
+	/* probed devices */
+	GList *devices;
+
 	/* signals */
 	void     (*open)            (GstElement  *element,
 	                             const gchar *device);
@@ -97,6 +136,4 @@ struct _GstV4l2ElementClass {
 
 GType gst_v4l2element_get_type (void);
 
-gboolean gst_v4l2element_factory_init (GstPlugin *plugin);
-
 #endif /* __GST_V4L2ELEMENT_H__ */
