@@ -23,6 +23,7 @@
 
 #include <string.h>
 #include <sys/time.h>
+#include <unistd.h>
 #include "v4l2src_calls.h"
 #include "gstv4l2tuner.h"
 
@@ -150,6 +151,7 @@ gst_v4l2src_get_type (void)
       gst_v4l2src_init,
       NULL
     };
+
     v4l2src_type = g_type_register_static (GST_TYPE_V4L2ELEMENT,
         "GstV4l2Src", &v4l2src_info, 0);
     GST_DEBUG_CATEGORY_INIT (v4l2src_debug, "v4l2src", 0, "v4l2src element");
@@ -254,6 +256,8 @@ gst_v4l2src_init (GTypeInstance * instance, gpointer g_class)
 
   /* fps */
   v4l2src->use_fixed_fps = TRUE;
+
+  v4l2src->is_capturing = FALSE;
 }
 
 
@@ -383,7 +387,7 @@ gst_v4l2src_v4l2fourcc_to_caps (guint32 fourcc)
   switch (fourcc) {
     case V4L2_PIX_FMT_MJPEG:   /* Motion-JPEG */
     case V4L2_PIX_FMT_JPEG:    /* JFIF JPEG */
-      structure = gst_structure_new ("video/x-jpeg", NULL);
+      structure = gst_structure_new ("image/jpeg", NULL);
       break;
     case V4L2_PIX_FMT_RGB332:
     case V4L2_PIX_FMT_RGB555:
@@ -424,6 +428,7 @@ gst_v4l2src_v4l2fourcc_to_caps (guint32 fourcc)
           r_mask = 0xf800;
           g_mask = 0x07e0;
           b_mask = 0x001f;
+          break;
         case V4L2_PIX_FMT_RGB24:
           bpp = depth = 24;
           endianness = G_BIG_ENDIAN;
@@ -466,8 +471,6 @@ gst_v4l2src_v4l2fourcc_to_caps (guint32 fourcc)
       break;
     }
     case V4L2_PIX_FMT_GREY:    /*  8  Greyscale     */
-    case V4L2_PIX_FMT_YUV422P: /* 16  YVU422 planar */
-    case V4L2_PIX_FMT_YUV411P: /* 16  YVU411 planar */
     case V4L2_PIX_FMT_NV12:    /* 12  Y/CbCr 4:2:0  */
     case V4L2_PIX_FMT_NV21:    /* 12  Y/CrCb 4:2:0  */
     case V4L2_PIX_FMT_YYUV:    /* 16  YUV 4:2:2     */
@@ -480,7 +483,9 @@ gst_v4l2src_v4l2fourcc_to_caps (guint32 fourcc)
     case V4L2_PIX_FMT_YUYV:
     case V4L2_PIX_FMT_YVU420:
     case V4L2_PIX_FMT_UYVY:
-    case V4L2_PIX_FMT_Y41P:{
+    case V4L2_PIX_FMT_Y41P:
+    case V4L2_PIX_FMT_YUV422P:
+    case V4L2_PIX_FMT_YUV411P:{
       guint32 fcc = 0;
 
       switch (fourcc) {
@@ -503,8 +508,14 @@ gst_v4l2src_v4l2fourcc_to_caps (guint32 fourcc)
           fcc = GST_MAKE_FOURCC ('U', 'Y', 'V', 'Y');
           break;
         case V4L2_PIX_FMT_Y41P:
+          fcc = GST_MAKE_FOURCC ('Y', '4', '1', 'P');
+          break;
+        case V4L2_PIX_FMT_YUV411P:
           fcc = GST_MAKE_FOURCC ('Y', '4', '1', 'B');
           break;
+        case V4L2_PIX_FMT_YUV422P:
+          fcc = GST_MAKE_FOURCC ('Y', '4', '2', 'B');
+          break;
         default:
           g_assert_not_reached ();
           break;
@@ -568,12 +579,8 @@ gst_v4l2_fourcc_from_structure (GstStructure * structure)
   guint32 fourcc = 0;
   const gchar *mimetype = gst_structure_get_name (structure);
 
-  if (!strcmp (mimetype, "video/x-raw-yuv") ||
-      !strcmp (mimetype, "video/x-raw-rgb")) {
-    if (!strcmp (mimetype, "video/x-raw-rgb"))
-      fourcc = GST_MAKE_FOURCC ('R', 'G', 'B', ' ');
-    else
-      gst_structure_get_fourcc (structure, "format", &fourcc);
+  if (!strcmp (mimetype, "video/x-raw-yuv")) {
+    gst_structure_get_fourcc (structure, "format", &fourcc);
 
     switch (fourcc) {
       case GST_MAKE_FOURCC ('I', '4', '2', '0'):
@@ -592,39 +599,42 @@ gst_v4l2_fourcc_from_structure (GstStructure * structure)
       case GST_MAKE_FOURCC ('Y', 'V', '1', '2'):
         fourcc = V4L2_PIX_FMT_YVU420;
         break;
-      case GST_MAKE_FOURCC ('R', 'G', 'B', ' '):{
-        gint depth, endianness, r_mask;
+      case GST_MAKE_FOURCC ('Y', '4', '1', 'B'):
+        fourcc = V4L2_PIX_FMT_YUV411P;
+        break;
+      case GST_MAKE_FOURCC ('Y', '4', '2', 'B'):
+        fourcc = V4L2_PIX_FMT_YUV422P;
+        break;
+    }
+  } else if (!strcmp (mimetype, "video/x-raw-rgb")) {
+    gint depth, endianness, r_mask;
 
-        gst_structure_get_int (structure, "depth", &depth);
-        gst_structure_get_int (structure, "endianness", &endianness);
-        gst_structure_get_int (structure, "red_mask", &r_mask);
+    gst_structure_get_int (structure, "depth", &depth);
+    gst_structure_get_int (structure, "endianness", &endianness);
+    gst_structure_get_int (structure, "red_mask", &r_mask);
 
-        switch (depth) {
-          case 8:
-            fourcc = V4L2_PIX_FMT_RGB332;
-            break;
-          case 15:
-            fourcc = (endianness == G_LITTLE_ENDIAN) ?
-                V4L2_PIX_FMT_RGB555 : V4L2_PIX_FMT_RGB555X;
-            break;
-          case 16:
-            fourcc = (endianness == G_LITTLE_ENDIAN) ?
-                V4L2_PIX_FMT_RGB565 : V4L2_PIX_FMT_RGB565X;
-            break;
-          case 24:
-            fourcc = (r_mask == 0xFF) ? V4L2_PIX_FMT_BGR24 : V4L2_PIX_FMT_RGB24;
-            break;
-          case 32:
-            fourcc = (r_mask == 0xFF) ? V4L2_PIX_FMT_BGR32 : V4L2_PIX_FMT_RGB32;
-            break;
-        }
-      default:
+    switch (depth) {
+      case 8:
+        fourcc = V4L2_PIX_FMT_RGB332;
+        break;
+      case 15:
+        fourcc = (endianness == G_LITTLE_ENDIAN) ?
+            V4L2_PIX_FMT_RGB555 : V4L2_PIX_FMT_RGB555X;
+        break;
+      case 16:
+        fourcc = (endianness == G_LITTLE_ENDIAN) ?
+            V4L2_PIX_FMT_RGB565 : V4L2_PIX_FMT_RGB565X;
+        break;
+      case 24:
+        fourcc = (r_mask == 0xFF) ? V4L2_PIX_FMT_BGR24 : V4L2_PIX_FMT_RGB24;
+        break;
+      case 32:
+        fourcc = (r_mask == 0xFF) ? V4L2_PIX_FMT_BGR32 : V4L2_PIX_FMT_RGB32;
         break;
-      }
     }
   } else if (strcmp (mimetype, "video/x-dv") == 0) {
     fourcc = V4L2_PIX_FMT_DV;
-  } else if (strcmp (mimetype, "video/x-jpeg") == 0) {
+  } else if (strcmp (mimetype, "image/jpeg") == 0) {
     fourcc = V4L2_PIX_FMT_JPEG;
   }
 
@@ -707,6 +717,7 @@ gst_v4l2src_link (GstPad * pad, const GstCaps * caps)
   struct v4l2_fmtdesc *format;
   int w, h;
   GstStructure *structure;
+  gboolean was_capturing;
 
   v4l2src = GST_V4L2SRC (gst_pad_get_parent (pad));
   v4l2element = GST_V4L2ELEMENT (v4l2src);
@@ -715,6 +726,10 @@ gst_v4l2src_link (GstPad * pad, const GstCaps * caps)
 
   /* clean up if we still haven't cleaned up our previous
    * capture session */
+  if ((was_capturing = v4l2src->is_capturing)) {
+    if (!gst_v4l2src_capture_stop (v4l2src))
+      return GST_PAD_LINK_REFUSED;
+  }
   if (GST_V4L2_IS_ACTIVE (v4l2element)) {
     if (!gst_v4l2src_capture_deinit (v4l2src))
       return GST_PAD_LINK_REFUSED;
@@ -733,6 +748,10 @@ gst_v4l2src_link (GstPad * pad, const GstCaps * caps)
   /* we found the pixelformat! - try it out */
   if (gst_v4l2src_set_capture (v4l2src, format, w, h)) {
     if (gst_v4l2src_capture_init (v4l2src)) {
+      if (was_capturing || GST_STATE (v4l2src) == GST_STATE_PLAYING)
+        if (!gst_v4l2src_capture_start (v4l2src))
+          return GST_PAD_LINK_REFUSED;
+
       return GST_PAD_LINK_OK;
     }
   }
@@ -750,14 +769,16 @@ gst_v4l2src_getcaps (GstPad * pad)
   int min_w, max_w, min_h, max_h;
   GSList *walk;
   GstStructure *structure;
+  gdouble fps;
 
   if (!GST_V4L2_IS_OPEN (GST_V4L2ELEMENT (v4l2src))) {
-    return gst_caps_new_any ();
+    return gst_caps_copy (gst_pad_get_pad_template_caps (pad));
   }
 
   /* build our own capslist */
   caps = gst_caps_new_empty ();
   walk = v4l2src->formats;
+  fps = gst_v4l2src_get_fps (v4l2src);
   while (walk) {
     format = (struct v4l2_fmtdesc *) walk->data;
     walk = g_slist_next (walk);
@@ -767,6 +788,11 @@ gst_v4l2src_getcaps (GstPad * pad)
             &min_w, &max_w, &min_h, &max_h)) {
       continue;
     }
+    /* template */
+    min_w = CLAMP (min_w, 1, 4096);
+    min_h = CLAMP (min_h, 1, 4096);
+    max_w = CLAMP (max_w, min_w, 4096);
+    max_h = CLAMP (max_h, min_h, 4096);
 
     /* add to list */
     structure = gst_v4l2src_v4l2fourcc_to_caps (format->pixelformat);
@@ -775,7 +801,7 @@ gst_v4l2src_getcaps (GstPad * pad)
       gst_structure_set (structure,
           "width", GST_TYPE_INT_RANGE, min_w, max_w,
           "height", GST_TYPE_INT_RANGE, min_h, max_h,
-          "framerate", GST_TYPE_DOUBLE_RANGE, (double) 0, G_MAXDOUBLE, NULL);
+          "framerate", G_TYPE_DOUBLE, fps, NULL);
 
       gst_caps_append_structure (caps, structure);
     }
@@ -785,20 +811,51 @@ gst_v4l2src_getcaps (GstPad * pad)
 }
 
 static GstData *
-gst_v4l2src_get (GstPad * pad)
+gst_v4l2src_get_read (GstV4l2Src * v4l2src)
+{
+  gint amount;
+  gint buffersize;
+
+  buffersize = v4l2src->format.fmt.pix.sizeimage;
+  do {
+    GstBuffer *buf =
+        gst_pad_alloc_buffer (v4l2src->srcpad, GST_BUFFER_OFFSET_NONE,
+        buffersize);
+    amount =
+        read (GST_V4L2ELEMENT (v4l2src)->video_fd, GST_BUFFER_DATA (buf),
+        buffersize);
+    if (amount == buffersize) {
+      gdouble fps = gst_v4l2src_get_fps (v4l2src);
+
+      GST_BUFFER_TIMESTAMP (buf) = v4l2src->handled * GST_SECOND / fps;
+      GST_BUFFER_DURATION (buf) = GST_SECOND / fps;
+      return GST_DATA (buf);
+    } else if (amount == -1) {
+      if (errno == EAGAIN || errno == EINTR) {
+        continue;
+      } else {
+        GST_ELEMENT_ERROR (v4l2src, RESOURCE, SYNC, (NULL),
+            ("error read()ing a buffer on device %s: %s",
+                GST_V4L2ELEMENT (v4l2src)->device, g_strerror (errno)));
+        gst_data_unref (GST_DATA (buf));
+        return NULL;
+      }
+    } else {
+      GST_ELEMENT_ERROR (v4l2src, RESOURCE, SYNC, (NULL),
+          ("error read()ing a buffer on device %s: got only %d bytes instead of expected %d",
+              GST_V4L2ELEMENT (v4l2src)->device, amount, buffersize));
+      gst_data_unref (GST_DATA (buf));
+      return NULL;
+    }
+  } while (TRUE);
+}
+
+static GstData *
+gst_v4l2src_get_mmap (GstV4l2Src * v4l2src)
 {
-  GstV4l2Src *v4l2src;
   GstBuffer *buf;
   gint i, num = -1;
-  gdouble fps = 0;
-
-  v4l2src = GST_V4L2SRC (gst_pad_get_parent (pad));
-
-  if (v4l2src->use_fixed_fps && (fps = gst_v4l2src_get_fps (v4l2src)) == 0) {
-    GST_ELEMENT_ERROR (v4l2src, RESOURCE, SETTINGS, (NULL),
-        ("could not get frame rate for element"));
-    return NULL;
-  }
+  gdouble fps = gst_v4l2src_get_fps (v4l2src);
 
   if (v4l2src->need_writes > 0) {
     /* use last frame */
@@ -878,9 +935,7 @@ gst_v4l2src_get (GstPad * pad)
     g_assert (num != -1);
     GST_LOG_OBJECT (v4l2src, "buffer %d needs %d writes", num,
         v4l2src->need_writes + 1);
-    i = v4l2src->pool->buffers[num].buffer.bytesused >
-        0 ? v4l2src->pool->buffers[num].buffer.bytesused : v4l2src->pool->
-        buffers[num].length;
+    i = v4l2src->format.fmt.pix.sizeimage;
     /* check if this is the last buffer in the queue. If so do a memcpy to put it back asap
        to avoid framedrops and deadlocks because of stupid elements */
     if (gst_atomic_int_read (&v4l2src->pool->refcount) == v4l2src->breq.count) {
@@ -920,13 +975,35 @@ gst_v4l2src_get (GstPad * pad)
     }
   }
 
-  v4l2src->handled++;
-  g_signal_emit (G_OBJECT (v4l2src), gst_v4l2src_signals[SIGNAL_FRAME_CAPTURE],
-      0);
-
   return GST_DATA (buf);
 }
 
+static GstData *
+gst_v4l2src_get (GstPad * pad)
+{
+  GstV4l2Src *v4l2src = GST_V4L2SRC (gst_pad_get_parent (pad));
+  GstData *ret;
+
+  if (v4l2src->use_fixed_fps && gst_v4l2src_get_fps (v4l2src) == 0) {
+    GST_ELEMENT_ERROR (v4l2src, RESOURCE, SETTINGS, (NULL),
+        ("could not get frame rate for element"));
+    return NULL;
+  }
+
+  if (v4l2src->breq.memory == V4L2_MEMORY_MMAP) {
+    ret = gst_v4l2src_get_mmap (v4l2src);
+  } else {
+    ret = gst_v4l2src_get_read (v4l2src);
+  }
+
+  if (ret) {
+    v4l2src->handled++;
+    g_signal_emit (G_OBJECT (v4l2src),
+        gst_v4l2src_signals[SIGNAL_FRAME_CAPTURE], 0);
+  }
+  return ret;
+}
+
 static void
 gst_v4l2src_set_property (GObject * object,
     guint prop_id, const GValue * value, GParamSpec * pspec)
@@ -1015,7 +1092,8 @@ gst_v4l2src_change_state (GstElement * element)
       break;
     case GST_STATE_PAUSED_TO_PLAYING:
       /* queue all buffer, start streaming capture */
-      if (!gst_v4l2src_capture_start (v4l2src))
+      if (GST_V4L2ELEMENT (v4l2src)->buffer &&
+          !gst_v4l2src_capture_start (v4l2src))
         return GST_STATE_FAILURE;
       g_get_current_time (&time);
       v4l2src->substract_time = GST_TIMEVAL_TO_TIME (time) -
@@ -1027,12 +1105,13 @@ gst_v4l2src_change_state (GstElement * element)
       v4l2src->substract_time = GST_TIMEVAL_TO_TIME (time) -
           v4l2src->substract_time;
       /* de-queue all queued buffers */
-      if (!gst_v4l2src_capture_stop (v4l2src))
+      if (v4l2src->is_capturing && !gst_v4l2src_capture_stop (v4l2src))
         return GST_STATE_FAILURE;
       break;
     case GST_STATE_PAUSED_TO_READY:
       /* stop capturing, unmap all buffers */
-      if (!gst_v4l2src_capture_deinit (v4l2src))
+      if (GST_V4L2ELEMENT (v4l2src)->buffer &&
+          !gst_v4l2src_capture_deinit (v4l2src))
         return GST_STATE_FAILURE;
       break;
     case GST_STATE_READY_TO_NULL:
