@@ -66,11 +66,11 @@ static GstStaticPadTemplate gst_xvimagesink_sink_template_factory =
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("video/x-raw-rgb, "
-        "framerate = (double) [ 1.0, 100.0 ], "
+        "framerate = (double) [ 1.0, 1000.0 ], "
         "width = (int) [ 1, MAX ], "
         "height = (int) [ 1, MAX ]; "
         "video/x-raw-yuv, "
-        "framerate = (double) [ 1.0, 100.0 ], "
+        "framerate = (double) [ 1.0, 1000.0 ], "
         "width = (int) [ 1, MAX ], " "height = (int) [ 1, MAX ]")
     );
 
@@ -147,7 +147,7 @@ gst_xvimagesink_check_xshm_calls (GstXContext * xcontext)
     goto beach;
   }
 
-  xvimage->SHMInfo.shmaddr = shmat (xvimage->SHMInfo.shmid, 0, 0);
+  xvimage->SHMInfo.shmaddr = shmat (xvimage->SHMInfo.shmid, NULL, 0);
   if (xvimage->SHMInfo.shmaddr == ((void *) -1)) {
     GST_WARNING ("Failed to shmat: %s", g_strerror (errno));
     goto beach;
@@ -167,7 +167,7 @@ gst_xvimagesink_check_xshm_calls (GstXContext * xcontext)
   XSync (xcontext->disp, FALSE);
 
   shmdt (xvimage->SHMInfo.shmaddr);
-  shmctl (xvimage->SHMInfo.shmid, IPC_RMID, 0);
+  shmctl (xvimage->SHMInfo.shmid, IPC_RMID, NULL);
 
   /* To be sure, reset the SHMInfo entry */
   xvimage->SHMInfo.shmaddr = ((void *) -1);
@@ -230,7 +230,7 @@ gst_xvimagesink_xvimage_new (GstXvImageSink * xvimagesink,
       goto beach;
     }
 
-    xvimage->SHMInfo.shmaddr = shmat (xvimage->SHMInfo.shmid, 0, 0);
+    xvimage->SHMInfo.shmaddr = shmat (xvimage->SHMInfo.shmid, NULL, 0);
     if (xvimage->SHMInfo.shmaddr == ((void *) -1)) {
       GST_ELEMENT_ERROR (xvimagesink, RESOURCE, WRITE, (NULL),
           ("Failed to shmat: %s", g_strerror (errno)));
@@ -299,7 +299,7 @@ gst_xvimagesink_xvimage_destroy (GstXvImageSink * xvimagesink,
       shmdt (xvimage->SHMInfo.shmaddr);
     }
     if (xvimage->SHMInfo.shmid > 0)
-      shmctl (xvimage->SHMInfo.shmid, IPC_RMID, 0);
+      shmctl (xvimage->SHMInfo.shmid, IPC_RMID, NULL);
     if (xvimage->xvimage)
       XFree (xvimage->xvimage);
   } else
@@ -360,6 +360,7 @@ gst_xvimagesink_xvimage_put (GstXvImageSink * xvimagesink, GstXvImage * xvimage)
 
   XSync (xvimagesink->xcontext->disp, FALSE);
 
+  xvimagesink->need_expose = FALSE;
   g_mutex_unlock (xvimagesink->x_lock);
 }
 
@@ -402,7 +403,7 @@ gst_xvimagesink_xwindow_decorate (GstXvImageSink * xvimagesink,
 /* This function handles a GstXWindow creation
  * The width and height are the actual pixel size on the display */
 static GstXWindow *
-gst_xvimagesink_xwindow_new (GstXvImageSink * xvimagesink,
+gst_xvimagesink_own_xwindow_new (GstXvImageSink * xvimagesink,
     gint width, gint height)
 {
   GstXWindow *xwindow = NULL;
@@ -444,6 +445,43 @@ gst_xvimagesink_xwindow_new (GstXvImageSink * xvimagesink,
   return xwindow;
 }
 
+static GstXWindow *
+gst_xvimagesink_ext_xwindow_new (GstXvImageSink * xvimagesink)
+{
+  GstXWindow *xwindow;
+  XWindowAttributes attr;
+
+  xwindow = g_new0 (GstXWindow, 1);
+  xwindow->win = xvimagesink->saved_xwindow_id;
+  /* We get window geometry, set the event we want to receive,
+     and create a GC */
+  g_mutex_lock (xvimagesink->x_lock);
+  XGetWindowAttributes (xvimagesink->xcontext->disp, xwindow->win, &attr);
+  xwindow->width = attr.width;
+  xwindow->height = attr.height;
+  xwindow->internal = FALSE;
+  GST_LOG_OBJECT (xvimagesink, "xwindow with xid %ld is %dx%d", xwindow->win,
+      xwindow->width, xwindow->height);
+  XSelectInput (xvimagesink->xcontext->disp, xwindow->win, ExposureMask |
+      StructureNotifyMask | PointerMotionMask | KeyPressMask | KeyReleaseMask);
+
+  xwindow->gc = XCreateGC (xvimagesink->xcontext->disp, xwindow->win, 0, NULL);
+  g_mutex_unlock (xvimagesink->x_lock);
+
+  return xwindow;
+}
+
+static GstXWindow *
+gst_xvimagesink_xwindow_new (GstXvImageSink * xvimagesink, gint width,
+    gint height)
+{
+  if (xvimagesink->saved_xwindow_id == 0) {
+    return gst_xvimagesink_own_xwindow_new (xvimagesink, width, height);
+  } else {
+    return gst_xvimagesink_ext_xwindow_new (xvimagesink);
+  }
+}
+
 /* This function destroys a GstXWindow */
 static void
 gst_xvimagesink_xwindow_destroy (GstXvImageSink * xvimagesink,
@@ -687,17 +725,17 @@ static GstCaps *
 gst_xvimagesink_get_xv_support (GstXvImageSink * xvimagesink,
     GstXContext * xcontext)
 {
-  gint i, nb_adaptors;
+  guint i, nb_adaptors;
   XvAdaptorInfo *adaptors;
-  gint nb_formats;
   XvImageFormatValues *formats = NULL;
   GstCaps *caps = NULL;
+  gint dummy, nb_formats;
 
   g_return_val_if_fail (xcontext != NULL, NULL);
 
   /* First let's check that XVideo extension is available */
-  if (!XQueryExtension (xcontext->disp, "XVideo", &i, &i, &i)) {
-    GST_ELEMENT_ERROR (xvimagesink, RESOURCE, SETTINGS, (NULL),
+  if (!XQueryExtension (xcontext->disp, "XVideo", &dummy, &dummy, &dummy)) {
+    GST_ELEMENT_ERROR (xvimagesink, RESOURCE, NOT_FOUND, (NULL),
         ("XVideo extension is not available"));
     return NULL;
   }
@@ -774,8 +812,26 @@ gst_xvimagesink_get_xv_support (GstXvImageSink * xvimagesink,
     switch (formats[i].type) {
       case XvRGB:
       {
+        /* our caps system handles 24/32bpp RGB as big-endian. */
+        gint endianness = xcontext->endianness, red_mask = formats[i].red_mask;
+
+        if ((xcontext->bpp == 24 || xcontext->bpp == 32) &&
+            endianness == G_LITTLE_ENDIAN) {
+          endianness = G_BIG_ENDIAN;
+          /* so for some unknown reason (beats me), the red and blue masks
+           * *are* actually in big-endian order, but in little endian format.
+           * Don't ask. I've learned not to. */
+          formats[i].red_mask = GUINT32_TO_BE (formats[i].blue_mask);
+          formats[i].green_mask = GUINT32_TO_BE (formats[i].green_mask);
+          formats[i].blue_mask = GUINT32_TO_BE (red_mask);
+          if (xcontext->bpp == 24) {
+            formats[i].red_mask >>= 8;
+            formats[i].green_mask >>= 8;
+            formats[i].blue_mask >>= 8;
+          }
+        }
         format_caps = gst_caps_new_simple ("video/x-raw-rgb",
-            "endianness", G_TYPE_INT, xcontext->endianness,
+            "endianness", G_TYPE_INT, endianness,
             "depth", G_TYPE_INT, xcontext->depth,
             "bpp", G_TYPE_INT, xcontext->bpp,
             "blue_mask", G_TYPE_INT, formats[i].red_mask,
@@ -783,7 +839,7 @@ gst_xvimagesink_get_xv_support (GstXvImageSink * xvimagesink,
             "red_mask", G_TYPE_INT, formats[i].blue_mask,
             "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
             "height", GST_TYPE_INT_RANGE, 1, G_MAXINT,
-            "framerate", GST_TYPE_DOUBLE_RANGE, 1.0, 100.0, NULL);
+            "framerate", GST_TYPE_DOUBLE_RANGE, 1.0, 1000.0, NULL);
 
         /* For RGB caps we store them and the image
            format so that we can get back the format
@@ -807,7 +863,7 @@ gst_xvimagesink_get_xv_support (GstXvImageSink * xvimagesink,
             "format", GST_TYPE_FOURCC, formats[i].id,
             "width", GST_TYPE_INT_RANGE, 1, G_MAXINT,
             "height", GST_TYPE_INT_RANGE, 1, G_MAXINT,
-            "framerate", GST_TYPE_DOUBLE_RANGE, 1.0, 100.0, NULL);
+            "framerate", GST_TYPE_DOUBLE_RANGE, 1.0, 1000.0, NULL);
         break;
       default:
         g_assert_not_reached ();
@@ -961,20 +1017,6 @@ gst_xvimagesink_xcontext_get (GstXvImageSink * xvimagesink)
       (ImageByteOrder (xcontext->disp) ==
       LSBFirst) ? G_LITTLE_ENDIAN : G_BIG_ENDIAN;
 
-  /* our caps system handles 24/32bpp RGB as big-endian. */
-  if ((xcontext->bpp == 24 || xcontext->bpp == 32) &&
-      xcontext->endianness == G_LITTLE_ENDIAN) {
-    xcontext->endianness = G_BIG_ENDIAN;
-    xcontext->visual->red_mask = GUINT32_TO_BE (xcontext->visual->red_mask);
-    xcontext->visual->green_mask = GUINT32_TO_BE (xcontext->visual->green_mask);
-    xcontext->visual->blue_mask = GUINT32_TO_BE (xcontext->visual->blue_mask);
-    if (xcontext->bpp == 24) {
-      xcontext->visual->red_mask >>= 8;
-      xcontext->visual->green_mask >>= 8;
-      xcontext->visual->blue_mask >>= 8;
-    }
-  }
-
   xcontext->caps = gst_xvimagesink_get_xv_support (xvimagesink, xcontext);
 
   if (!xcontext->caps) {
@@ -1097,7 +1139,7 @@ gst_xvimagesink_xcontext_clear (GstXvImageSink * xvimagesink)
       xvimagesink->xcontext->xv_port_id, 0);
 
   XCloseDisplay (xvimagesink->xcontext->disp);
-
+  GST_DEBUG ("xcontext cleared");
   g_mutex_unlock (xvimagesink->x_lock);
 
   g_free (xvimagesink->xcontext);
@@ -1195,8 +1237,7 @@ gst_xvimagesink_sink_link (GstPad * pad, const GstCaps * caps)
 {
   GstXvImageSink *xvimagesink;
   GstStructure *structure;
-  gint im_format = 0;
-  gboolean ret;
+  guint32 im_format = 0;
   gint video_width, video_height;
   gint video_par_n, video_par_d;        /* video's PAR */
   gint display_par_n, display_par_d;    /* display's PAR */
@@ -1214,12 +1255,12 @@ gst_xvimagesink_sink_link (GstPad * pad, const GstCaps * caps)
       GST_PTR_FORMAT, xvimagesink->xcontext->caps, caps);
 
   structure = gst_caps_get_structure (caps, 0);
-  ret = gst_structure_get_int (structure, "width", &video_width);
-  ret &= gst_structure_get_int (structure, "height", &video_height);
-  ret &= gst_structure_get_double (structure, "framerate",
-      &xvimagesink->framerate);
-  if (!ret)
+  if (!gst_structure_get_int (structure, "width", &video_width)
+      || !gst_structure_get_int (structure, "height", &video_height)
+      || !gst_structure_get_double (structure, "framerate",
+          &xvimagesink->framerate)) {
     return GST_PAD_LINK_REFUSED;
+  }
 
   g_mutex_lock (xvimagesink->stream_lock);
 
@@ -1342,6 +1383,7 @@ gst_xvimagesink_change_state (GstElement * element)
       if (!xvimagesink->xcontext &&
           !(xvimagesink->xcontext = gst_xvimagesink_xcontext_get (xvimagesink)))
         return GST_STATE_FAILURE;
+
       /* update object's par with calculated one if not set yet */
       if (!xvimagesink->par) {
         xvimagesink->par = g_new0 (GValue, 1);
@@ -1355,6 +1397,18 @@ gst_xvimagesink_change_state (GstElement * element)
       XSynchronize (xvimagesink->xcontext->disp, xvimagesink->synchronous);
       g_mutex_unlock (xvimagesink->x_lock);
       gst_xvimagesink_update_colorbalance (xvimagesink);
+
+      if (xvimagesink->saved_xwindow_id != 0) {
+        GstXWindow *win;
+
+        if ((win = gst_xvimagesink_xwindow_new (xvimagesink, 0, 0)))
+          xvimagesink->xwindow = win;
+        else {
+          gst_xvimagesink_xcontext_clear (xvimagesink);
+          xvimagesink->xcontext = NULL;
+          return GST_STATE_FAILURE;
+        }
+      }
       break;
     case GST_STATE_READY_TO_PAUSED:
       xvimagesink->time = 0;
@@ -1404,6 +1458,59 @@ gst_xvimagesink_change_state (GstElement * element)
   return GST_STATE_SUCCESS;
 }
 
+static void
+gst_xvimagesink_handle_event (GstXvImageSink * xvimagesink, GstEvent * event,
+    GstPad * pad)
+{
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_FILLER:
+    {
+      GstClockTime ts = GST_EVENT_TIMESTAMP (event);
+      GstClockTime dur = gst_event_filler_get_duration (event);
+      GstClockTimeDiff step;
+
+      if (!GST_CLOCK_TIME_IS_VALID (ts)) {
+        ts = xvimagesink->time;
+      }
+      if (GST_CLOCK_TIME_IS_VALID (dur)) {
+        ts += dur;
+      }
+
+      gst_event_unref (event);
+      event = NULL;
+
+      if (xvimagesink->framerate > 0)
+        step = GST_SECOND / xvimagesink->framerate;
+      else
+        step = GST_CLOCK_DIFF (ts, xvimagesink->time);
+
+      /*
+       * Advance the clock to the timestamp of the event, in steps
+       * at the framerate so we send navigation events and redraw nicely.
+       */
+      if (step > 0 && GST_VIDEOSINK_CLOCK (xvimagesink)) {
+        while (xvimagesink->time < ts) {
+          GstClockTime next_ts = MIN (xvimagesink->time + step, ts);
+
+          if (next_ts > xvimagesink->time) {
+            gst_element_wait (GST_ELEMENT (xvimagesink), next_ts);
+          }
+          xvimagesink->time = next_ts;
+
+          g_mutex_lock (xvimagesink->stream_lock);
+          gst_xvimagesink_handle_xevents (xvimagesink, pad);
+          g_mutex_unlock (xvimagesink->stream_lock);
+          gst_xvimagesink_send_pending_navigation (xvimagesink);
+        }
+      }
+
+      break;
+    }
+    default:
+      gst_pad_event_default (pad, event);
+  }
+}
+
 static void
 gst_xvimagesink_chain (GstPad * pad, GstData * data)
 {
@@ -1412,11 +1519,10 @@ gst_xvimagesink_chain (GstPad * pad, GstData * data)
 
   g_return_if_fail (GST_IS_PAD (pad));
   g_return_if_fail (data != NULL);
-
   xvimagesink = GST_XVIMAGESINK (gst_pad_get_parent (pad));
-
+  g_return_if_fail (xvimagesink != NULL);
   if (GST_IS_EVENT (data)) {
-    gst_pad_event_default (pad, GST_EVENT (data));
+    gst_xvimagesink_handle_event (xvimagesink, GST_EVENT (data), pad);
     return;
   }
 
@@ -1427,7 +1533,6 @@ gst_xvimagesink_chain (GstPad * pad, GstData * data)
   }
   GST_LOG_OBJECT (xvimagesink, "clock wait: %" GST_TIME_FORMAT,
       GST_TIME_ARGS (xvimagesink->time));
-
   if (GST_VIDEOSINK_CLOCK (xvimagesink)) {
     gst_element_wait (GST_ELEMENT (xvimagesink), xvimagesink->time);
   }
@@ -1467,11 +1572,10 @@ gst_xvimagesink_chain (GstPad * pad, GstData * data)
   }
 
   gst_buffer_unref (buf);
-
   gst_xvimagesink_handle_xevents (xvimagesink, pad);
-  gst_xvimagesink_send_pending_navigation (xvimagesink);
 
   g_mutex_unlock (xvimagesink->stream_lock);
+  gst_xvimagesink_send_pending_navigation (xvimagesink);
 }
 
 /* Buffer management */
@@ -1483,10 +1587,8 @@ gst_xvimagesink_buffer_free (GstBuffer * buffer)
   GstXvImage *xvimage;
 
   xvimage = GST_BUFFER_PRIVATE (buffer);
-
   g_assert (GST_IS_XVIMAGESINK (xvimage->xvimagesink));
   xvimagesink = xvimage->xvimagesink;
-
   /* If our geometry changed we can't reuse that image. */
   if ((xvimage->width != xvimagesink->video_width) ||
       (xvimage->height != xvimagesink->video_height))
@@ -1509,19 +1611,16 @@ gst_xvimagesink_buffer_alloc (GstPad * pad, guint64 offset, guint size)
   gboolean not_found = TRUE;
 
   xvimagesink = GST_XVIMAGESINK (gst_pad_get_parent (pad));
-
   g_mutex_lock (xvimagesink->pool_lock);
-
   /* Walking through the pool cleaning unusable images and searching for a
      suitable one */
   while (not_found && xvimagesink->image_pool) {
     xvimage = xvimagesink->image_pool->data;
-
     if (xvimage) {
       /* Removing from the pool */
-      xvimagesink->image_pool = g_slist_delete_link (xvimagesink->image_pool,
+      xvimagesink->image_pool =
+          g_slist_delete_link (xvimagesink->image_pool,
           xvimagesink->image_pool);
-
       /* We check for geometry or image format changes */
       if ((xvimage->width != xvimagesink->video_width) ||
           (xvimage->height != xvimagesink->video_height) ||
@@ -1537,21 +1636,19 @@ gst_xvimagesink_buffer_alloc (GstPad * pad, guint64 offset, guint size)
   }
 
   g_mutex_unlock (xvimagesink->pool_lock);
-
   if (!xvimage) {
     /* We found no suitable image in the pool. Creating... */
     GST_LOG_OBJECT (xvimagesink, "no usable image in pool, creating xvimage");
-    xvimage = gst_xvimagesink_xvimage_new (xvimagesink,
-        xvimagesink->video_width, xvimagesink->video_height);
+    xvimage =
+        gst_xvimagesink_xvimage_new (xvimagesink, xvimagesink->video_width,
+        xvimagesink->video_height);
   }
 
   if (xvimage) {
     buffer = gst_buffer_new ();
-
     /* Storing some pointers in the buffer */
     GST_BUFFER_PRIVATE (buffer) = xvimage;
-
-    GST_BUFFER_DATA (buffer) = xvimage->xvimage->data;
+    GST_BUFFER_DATA (buffer) = (guint8 *) xvimage->xvimage->data;
     GST_BUFFER_FREE_DATA_FUNC (buffer) = gst_xvimagesink_buffer_free;
     GST_BUFFER_SIZE (buffer) = xvimage->size;
     return buffer;
@@ -1564,8 +1661,8 @@ gst_xvimagesink_buffer_alloc (GstPad * pad, guint64 offset, guint size)
 static gboolean
 gst_xvimagesink_interface_supported (GstImplementsInterface * iface, GType type)
 {
-  g_assert (type == GST_TYPE_NAVIGATION ||
-      type == GST_TYPE_X_OVERLAY || type == GST_TYPE_COLOR_BALANCE);
+  g_assert (type == GST_TYPE_NAVIGATION || type == GST_TYPE_X_OVERLAY
+      || type == GST_TYPE_COLOR_BALANCE);
   return TRUE;
 }
 
@@ -1597,7 +1694,6 @@ gst_xvimagesink_send_pending_navigation (GstXvImageSink * xvimagesink)
 
     if (event) {
       structure = event->event_data.structure.structure;
-
       if (!GST_PAD_PEER (GST_VIDEOSINK_PAD (xvimagesink))) {
         gst_event_unref (event);
         cur = g_slist_next (cur);
@@ -1605,6 +1701,7 @@ gst_xvimagesink_send_pending_navigation (GstXvImageSink * xvimagesink)
       }
 
       /* Converting pointer coordinates to the non scaled geometry */
+      g_mutex_lock (xvimagesink->stream_lock);
       if (gst_structure_get_double (structure, "pointer_x", &x)) {
         x *= xvimagesink->video_width;
         x /= xvimagesink->xwindow->width;
@@ -1615,9 +1712,12 @@ gst_xvimagesink_send_pending_navigation (GstXvImageSink * xvimagesink)
         y /= xvimagesink->xwindow->height;
         gst_structure_set (structure, "pointer_y", G_TYPE_DOUBLE, y, NULL);
       }
+      g_mutex_unlock (xvimagesink->stream_lock);
 
-      gst_pad_send_event (gst_pad_get_peer (GST_VIDEOSINK_PAD (xvimagesink)),
-          event);
+      GST_DATA_FLAG_UNSET (event, GST_EVENT_COMMON_FLAG_NEED_RESPONSE);
+
+      gst_pad_send_event (gst_pad_get_peer (GST_VIDEOSINK_PAD
+              (xvimagesink)), event);
     }
     cur = g_slist_next (cur);
   }
@@ -1633,8 +1733,8 @@ gst_xvimagesink_navigation_send_event (GstNavigation * navigation,
   GstEvent *event;
 
   event = gst_event_new (GST_EVENT_NAVIGATION);
-  event->event_data.structure.structure = structure;
 
+  event->event_data.structure.structure = structure;
   g_mutex_lock (xvimagesink->nav_lock);
   xvimagesink->pend_nav_events =
       g_slist_prepend (xvimagesink->pend_nav_events, event);
@@ -1652,30 +1752,22 @@ gst_xvimagesink_set_xwindow_id (GstXOverlay * overlay, XID xwindow_id)
 {
   GstXvImageSink *xvimagesink = GST_XVIMAGESINK (overlay);
   GstXWindow *xwindow = NULL;
-  XWindowAttributes attr;
 
   g_return_if_fail (GST_IS_XVIMAGESINK (xvimagesink));
-
   /* If we already use that window return */
-  if (xvimagesink->xwindow && (xwindow_id == xvimagesink->xwindow->win))
+  if (xvimagesink->xwindow && (xwindow_id == xvimagesink->saved_xwindow_id))
     return;
-
-  /* If the element has not initialized the X11 context try to do so */
-  if (!xvimagesink->xcontext &&
-      !(xvimagesink->xcontext = gst_xvimagesink_xcontext_get (xvimagesink)))
-    /* we have thrown a GST_ELEMENT_ERROR now */
+  xvimagesink->saved_xwindow_id = xwindow_id;
+  if (xvimagesink->xcontext == NULL)
     return;
 
   gst_xvimagesink_update_colorbalance (xvimagesink);
-
   /* We acquire the stream lock while setting this window in the element.
      We are basically cleaning tons of stuff replacing the old window, putting
      images while we do that would surely crash */
   g_mutex_lock (xvimagesink->stream_lock);
-
   /* Clear image pool as the images are unusable anyway */
   gst_xvimagesink_imagepool_clear (xvimagesink);
-
   /* Clear the xvimage */
   if (xvimagesink->xvimage) {
     gst_xvimagesink_xvimage_destroy (xvimagesink, xvimagesink->xvimage);
@@ -1688,41 +1780,15 @@ gst_xvimagesink_set_xwindow_id (GstXOverlay * overlay, XID xwindow_id)
     xvimagesink->xwindow = NULL;
   }
 
-  /* If the xid is 0 we go back to an internal window */
-  if (xwindow_id == 0) {
-    /* If no width/height caps nego did not happen window will be created
-       during caps nego then */
-    if (GST_VIDEOSINK_WIDTH (xvimagesink) && GST_VIDEOSINK_HEIGHT (xvimagesink)) {
-      xwindow = gst_xvimagesink_xwindow_new (xvimagesink,
-          GST_VIDEOSINK_WIDTH (xvimagesink),
-          GST_VIDEOSINK_HEIGHT (xvimagesink));
-    }
-  } else {
-    xwindow = g_new0 (GstXWindow, 1);
+  if (xwindow_id == 0 &&
+      (GST_VIDEOSINK_WIDTH (xvimagesink) == 0 ||
+          GST_VIDEOSINK_HEIGHT (xvimagesink) == 0))
+    return;
 
-    xwindow->win = xwindow_id;
-
-    /* We get window geometry, set the event we want to receive,
-       and create a GC */
-    g_mutex_lock (xvimagesink->x_lock);
-    XGetWindowAttributes (xvimagesink->xcontext->disp, xwindow->win, &attr);
-    xwindow->width = attr.width;
-    xwindow->height = attr.height;
-    xwindow->internal = FALSE;
-    GST_LOG_OBJECT (xvimagesink, "xwindow with xid %ld is %dx%d", xwindow_id,
-        xwindow->width, xwindow->height);
-    XSelectInput (xvimagesink->xcontext->disp, xwindow->win, ExposureMask |
-        StructureNotifyMask | PointerMotionMask | KeyPressMask |
-        KeyReleaseMask);
-
-    xwindow->gc = XCreateGC (xvimagesink->xcontext->disp,
-        xwindow->win, 0, NULL);
-    g_mutex_unlock (xvimagesink->x_lock);
-  }
-
-  if (xwindow)
+  if ((xwindow = gst_xvimagesink_xwindow_new (xvimagesink,
+              GST_VIDEOSINK_WIDTH (xvimagesink),
+              GST_VIDEOSINK_HEIGHT (xvimagesink))) != NULL)
     xvimagesink->xwindow = xwindow;
-
   g_mutex_unlock (xvimagesink->stream_lock);
 }
 
@@ -1745,6 +1811,12 @@ gst_xvimagesink_expose (GstXOverlay * overlay)
   if (!xvimagesink->xwindow)
     return;
 
+  /* Set the need_expose flag before we wait for the lock. 
+   * If it's clear by the time we get to put the image,
+   * the stream beat us to it so don't bother
+   */
+  xvimagesink->need_expose = TRUE;
+
   /* We don't want chain to iterate while we do that. We might act on random
      cur_image and different geometry */
   g_mutex_lock (xvimagesink->stream_lock);
@@ -1754,13 +1826,12 @@ gst_xvimagesink_expose (GstXOverlay * overlay)
   XGetWindowAttributes (xvimagesink->xcontext->disp,
       xvimagesink->xwindow->win, &attr);
   g_mutex_unlock (xvimagesink->x_lock);
-
   xvimagesink->xwindow->width = attr.width;
   xvimagesink->xwindow->height = attr.height;
-  GST_LOG_OBJECT (xvimagesink, "xwindow exposed is %dx%d", attr.width,
-      attr.height);
+  if (xvimagesink->need_expose && xvimagesink->cur_image) {
+    GST_LOG_OBJECT (xvimagesink, "xwindow exposed is %dx%d", attr.width,
+        attr.height);
 
-  if (xvimagesink->cur_image) {
     gst_xvimagesink_xvimage_put (xvimagesink, xvimagesink->cur_image);
   }
 
@@ -1781,7 +1852,6 @@ gst_xvimagesink_colorbalance_list_channels (GstColorBalance * balance)
   GstXvImageSink *xvimagesink = GST_XVIMAGESINK (balance);
 
   g_return_val_if_fail (GST_IS_XVIMAGESINK (xvimagesink), NULL);
-
   if (xvimagesink->xcontext)
     return xvimagesink->xcontext->channels_list;
   else
@@ -1796,13 +1866,10 @@ gst_xvimagesink_colorbalance_set_value (GstColorBalance * balance,
 
   g_return_if_fail (GST_IS_XVIMAGESINK (xvimagesink));
   g_return_if_fail (channel->label != NULL);
-
   xvimagesink->cb_changed = TRUE;
-
   /* Normalize val to [-1000, 1000] */
   value = -1000 + 2000 * (value - channel->min_value) /
       (channel->max_value - channel->min_value);
-
   if (g_ascii_strcasecmp (channel->label, "XV_HUE") == 0) {
     xvimagesink->hue = value;
   } else if (g_ascii_strcasecmp (channel->label, "XV_SATURATION") == 0) {
@@ -1828,7 +1895,6 @@ gst_xvimagesink_colorbalance_get_value (GstColorBalance * balance,
 
   g_return_val_if_fail (GST_IS_XVIMAGESINK (xvimagesink), 0);
   g_return_val_if_fail (channel->label != NULL, 0);
-
   if (g_ascii_strcasecmp (channel->label, "XV_HUE") == 0) {
     value = xvimagesink->hue;
   } else if (g_ascii_strcasecmp (channel->label, "XV_SATURATION") == 0) {
@@ -1844,7 +1910,6 @@ gst_xvimagesink_colorbalance_get_value (GstColorBalance * balance,
   /* Normalize val to [channel->min_value, channel->max_value] */
   value = channel->min_value + (channel->max_value - channel->min_value) *
       (value + 1000) / 2000;
-
   return value;
 }
 
@@ -1870,9 +1935,7 @@ gst_xvimagesink_set_property (GObject * object, guint prop_id,
   GstXvImageSink *xvimagesink;
 
   g_return_if_fail (GST_IS_XVIMAGESINK (object));
-
   xvimagesink = GST_XVIMAGESINK (object);
-
   switch (prop_id) {
     case ARG_HUE:
       xvimagesink->hue = g_value_get_int (value);
@@ -1932,9 +1995,7 @@ gst_xvimagesink_get_property (GObject * object, guint prop_id,
   GstXvImageSink *xvimagesink;
 
   g_return_if_fail (GST_IS_XVIMAGESINK (object));
-
   xvimagesink = GST_XVIMAGESINK (object);
-
   switch (prop_id) {
     case ARG_HUE:
       g_value_set_int (value, xvimagesink->hue);
@@ -1973,7 +2034,6 @@ gst_xvimagesink_finalize (GObject * object)
   GstXvImageSink *xvimagesink;
 
   xvimagesink = GST_XVIMAGESINK (object);
-
   if (xvimagesink->display_name) {
     g_free (xvimagesink->display_name);
     xvimagesink->display_name = NULL;
@@ -2018,10 +2078,8 @@ gst_xvimagesink_init (GstXvImageSink * xvimagesink)
   GST_VIDEOSINK_PAD (xvimagesink) =
       gst_pad_new_from_template (gst_static_pad_template_get
       (&gst_xvimagesink_sink_template_factory), "sink");
-
   gst_element_add_pad (GST_ELEMENT (xvimagesink),
       GST_VIDEOSINK_PAD (xvimagesink));
-
   gst_pad_set_chain_function (GST_VIDEOSINK_PAD (xvimagesink),
       gst_xvimagesink_chain);
   gst_pad_set_link_function (GST_VIDEOSINK_PAD (xvimagesink),
@@ -2032,33 +2090,26 @@ gst_xvimagesink_init (GstXvImageSink * xvimagesink)
       gst_xvimagesink_fixate);
   gst_pad_set_bufferalloc_function (GST_VIDEOSINK_PAD (xvimagesink),
       gst_xvimagesink_buffer_alloc);
-
   xvimagesink->display_name = NULL;
   xvimagesink->xcontext = NULL;
   xvimagesink->xwindow = NULL;
   xvimagesink->xvimage = NULL;
   xvimagesink->cur_image = NULL;
-
   xvimagesink->hue = xvimagesink->saturation = 0;
   xvimagesink->contrast = xvimagesink->brightness = 0;
   xvimagesink->cb_changed = FALSE;
-
   xvimagesink->framerate = 0;
   xvimagesink->video_width = 0;
   xvimagesink->video_height = 0;
-
   xvimagesink->x_lock = g_mutex_new ();
   xvimagesink->stream_lock = g_mutex_new ();
-
   xvimagesink->image_pool = NULL;
   xvimagesink->pool_lock = g_mutex_new ();
-
   xvimagesink->synchronous = FALSE;
+  xvimagesink->need_expose = FALSE;
   xvimagesink->par = NULL;
-
   GST_FLAG_SET (xvimagesink, GST_ELEMENT_THREAD_SUGGESTED);
   GST_FLAG_SET (xvimagesink, GST_ELEMENT_EVENT_AWARE);
-
   xvimagesink->nav_lock = g_mutex_new ();
   xvimagesink->pend_nav_events = NULL;
 }
@@ -2069,7 +2120,6 @@ gst_xvimagesink_base_init (gpointer g_class)
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
   gst_element_class_set_details (element_class, &gst_xvimagesink_details);
-
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_xvimagesink_sink_template_factory));
 }
@@ -2082,18 +2132,16 @@ gst_xvimagesink_class_init (GstXvImageSinkClass * klass)
 
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
-
   parent_class = g_type_class_ref (GST_TYPE_VIDEOSINK);
-
   g_object_class_install_property (gobject_class, ARG_CONTRAST,
-      g_param_spec_int ("contrast", "Contrast", "The contrast of the video",
-          -1000, 1000, 0, G_PARAM_READWRITE));
+      g_param_spec_int ("contrast", "Contrast",
+          "The contrast of the video", -1000, 1000, 0, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_BRIGHTNESS,
       g_param_spec_int ("brightness", "Brightness",
           "The brightness of the video", -1000, 1000, 0, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_HUE,
-      g_param_spec_int ("hue", "Hue", "The hue of the video", -1000, 1000, 0,
-          G_PARAM_READWRITE));
+      g_param_spec_int ("hue", "Hue", "The hue of the video", -1000, 1000,
+          0, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_SATURATION,
       g_param_spec_int ("saturation", "Saturation",
           "The saturation of the video", -1000, 1000, 0, G_PARAM_READWRITE));
@@ -2103,16 +2151,14 @@ gst_xvimagesink_class_init (GstXvImageSinkClass * klass)
   g_object_class_install_property (gobject_class, ARG_SYNCHRONOUS,
       g_param_spec_boolean ("synchronous", "Synchronous",
           "When enabled, runs "
-          "the X display in synchronous mode. (used only for debugging)", FALSE,
-          G_PARAM_READWRITE));
+          "the X display in synchronous mode. (used only for debugging)",
+          FALSE, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, ARG_PIXEL_ASPECT_RATIO,
       g_param_spec_string ("pixel-aspect-ratio", "Pixel Aspect Ratio",
           "The pixel aspect ratio of the device", "1/1", G_PARAM_READWRITE));
-
   gobject_class->finalize = gst_xvimagesink_finalize;
   gobject_class->set_property = gst_xvimagesink_set_property;
   gobject_class->get_property = gst_xvimagesink_get_property;
-
   gstelement_class->change_state = gst_xvimagesink_change_state;
 }
 
@@ -2147,28 +2193,23 @@ gst_xvimagesink_get_type (void)
     };
     static const GInterfaceInfo iface_info = {
       (GInterfaceInitFunc) gst_xvimagesink_interface_init,
-      NULL,
-      NULL,
+      NULL, NULL,
     };
     static const GInterfaceInfo navigation_info = {
       (GInterfaceInitFunc) gst_xvimagesink_navigation_init,
-      NULL,
-      NULL,
+      NULL, NULL,
     };
     static const GInterfaceInfo overlay_info = {
       (GInterfaceInitFunc) gst_xvimagesink_xoverlay_init,
-      NULL,
-      NULL,
+      NULL, NULL,
     };
     static const GInterfaceInfo colorbalance_info = {
       (GInterfaceInitFunc) gst_xvimagesink_colorbalance_init,
       NULL,
       NULL,
     };
-
     xvimagesink_type = g_type_register_static (GST_TYPE_VIDEOSINK,
         "GstXvImageSink", &xvimagesink_info, 0);
-
     g_type_add_interface_static (xvimagesink_type,
         GST_TYPE_IMPLEMENTS_INTERFACE, &iface_info);
     g_type_add_interface_static (xvimagesink_type, GST_TYPE_NAVIGATION,
@@ -2188,14 +2229,11 @@ plugin_init (GstPlugin * plugin)
   /* Loading the library containing GstVideoSink, our parent object */
   if (!gst_library_load ("gstvideo"))
     return FALSE;
-
   if (!gst_element_register (plugin, "xvimagesink",
           GST_RANK_PRIMARY, GST_TYPE_XVIMAGESINK))
     return FALSE;
-
   GST_DEBUG_CATEGORY_INIT (gst_debug_xvimagesink, "xvimagesink", 0,
       "xvimagesink element");
-
   return TRUE;
 }
 
