@@ -16,13 +16,14 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
- 
+
 #ifndef __GST_XVIMAGESINK_H__
 #define __GST_XVIMAGESINK_H__
 
-#include <gst/video/gstvideosink.h>
+#include <gst/video/videosink.h>
 
 #ifdef HAVE_XSHM
+#include <sys/types.h>
 #include <sys/ipc.h>
 #include <sys/shm.h>
 #endif /* HAVE_XSHM */
@@ -57,35 +58,39 @@ typedef struct _GstXContext GstXContext;
 typedef struct _GstXWindow GstXWindow;
 typedef struct _GstXvImage GstXvImage;
 typedef struct _GstXvImageFormat GstXvImageFormat;
-  
+
 typedef struct _GstXvImageSink GstXvImageSink;
 typedef struct _GstXvImageSinkClass GstXvImageSinkClass;
 
 /* Global X Context stuff */
 struct _GstXContext {
   Display *disp;
-  
+
   Screen *screen;
   gint screen_num;
-  
+
   Visual *visual;
-  
+
   Window root;
-  
+
   gulong white, black;
-  
+
   gint depth;
   gint bpp;
   gint endianness;
-  
+
+  gint width, height;
+  gint widthmm, heightmm;
+  GValue *par;                  /* calculated pixel aspect ratio */
+
   gboolean use_xshm;
-  
+
   XvPortID xv_port_id;
   gint im_format;
-  
+
   GList *formats_list;
   GList *channels_list;
-  
+
   GstCaps *caps;
 };
 
@@ -107,47 +112,54 @@ struct _GstXvImageFormat {
 struct _GstXvImage {
   /* Reference to the xvimagesink we belong to */
   GstXvImageSink *xvimagesink;
-  
+
   XvImage *xvimage;
-  
+
 #ifdef HAVE_XSHM
   XShmSegmentInfo SHMInfo;
 #endif /* HAVE_XSHM */
-  
-  char *data;
-  gint width, height, size;
+
+  gint width, height, size, im_format;
 };
 
 struct _GstXvImageSink {
   /* Our element stuff */
   GstVideoSink videosink;
-  
+
   char *display_name;
-  
+
   GstXContext *xcontext;
   GstXWindow *xwindow;
   GstXvImage *xvimage;
   GstXvImage *cur_image;
-  
+
   gdouble framerate;
-  
+
   gint brightness;
   gint contrast;
   gint hue;
   gint saturation;
   gboolean cb_changed;
-  
+
   GMutex *x_lock;
-  
-  /* Unused */
-  gint pixel_width, pixel_height;
+  GMutex *stream_lock;
+
+  guint video_width, video_height;     /* size of incoming video;
+                                        * used as the size for XvImage */
+  GValue *par;                         /* object-set pixel aspect ratio */
 
   GstClockTime time;
- 
+
   GMutex *pool_lock;
   GSList *image_pool;
 
   gboolean synchronous;
+  gboolean need_expose;
+
+  GMutex *nav_lock;
+  GSList *pend_nav_events;
+
+  Window saved_xwindow_id;
 };
 
 struct _GstXvImageSinkClass {
